/**************************************************************

catfile_dlg.cpp (C-Munipack project)
The preview dialog for catalog files
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <glib.h>
#include <glib/gstdio.h>

#include "catfile_dlg.h"
#include "configuration.h"
#include "configuration.h"
#include "showheader_dlg.h"
#include "chartexport_dlg.h"
#include "utils.h"
#include "profile.h"
#include "object_dlg.h"
#include "observer_dlg.h"
#include "choosestars_dlg.h"
#include "main.h"
#include "ctxhelp.h"

enum tMenuId
{
	MENU_FILE = 1,
	MENU_EDIT,
	MENU_VIEW,
	MENU_TOOLS,
	MENU_HELP
};

enum tCommandId
{
	CMD_SAVE = 100,
	CMD_SAVE_AS,
	CMD_OPEN,
	CMD_CLOSE,
	CMD_PRINT,
	CMD_EXPORT,
	CMD_PROPERTIES,
	CMD_HEADER,
	CMD_STARS,
	CMD_RULERS,
	CMD_INSPECTOR,
	CMD_HELP,
	CMD_SET_VARIABLE,
	CMD_SET_COMPARISON,
	CMD_SET_CHECK,
	CMD_UNSET
};

enum tFieldId
{
	FIELD_FILE,
	FIELD_FILENAME,
	FIELD_DIRPATH,
	FIELD_FRAME,
	FIELD_DATETIME,
	FIELD_JULDAT,
	FIELD_FILTER,
	FIELD_EXPTIME,
	FIELD_PHOTOMETRY,
	FIELD_STARS
};

//-------------------------   MENU FOR PREVIEW WINDOW   ---------------------------

static const CMenuBar::tMenuItem FileMenu[] = {
	{ CMenuBar::MB_ITEM,	CMD_OPEN,		"_Open" },
	{ CMenuBar::MB_ITEM,	CMD_SAVE,		"_Save" },
	{ CMenuBar::MB_ITEM,	CMD_SAVE_AS,	"Save _as..." },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_PROPERTIES,	"Show _Properties" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_EXPORT,		"E_xport" },
	//{ CMenuBar::MB_ITEM,	CMD_PRINT,		"P_rint" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,	CMD_CLOSE,		"_Close" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem EditMenu[] = {
	{ CMenuBar::MB_ITEM,	CMD_HEADER,		"Edit _properties" },
	{ CMenuBar::MB_CHECKBTN, CMD_STARS,		"Change selected _stars" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ViewMenu[] = {
	{ CMenuBar::MB_CHECKBTN, CMD_RULERS,	"_Rulers" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ToolsMenu[] = {
	{ CMenuBar::MB_CHECKBTN, CMD_INSPECTOR,	"_Object inspector" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem HelpMenu[] = {
	{ CMenuBar::MB_ITEM,	CMD_HELP,		"_Show help", "help" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenu PreviewMenu[] = {
	{ "_File",	MENU_FILE,	FileMenu },
	{ "_Edit",	MENU_EDIT,	EditMenu },
	{ "_View",	MENU_VIEW,	ViewMenu },
	{ "_Tools", MENU_TOOLS,	ToolsMenu },
	{ "_Help",	MENU_HELP, HelpMenu, CMenuBar::MBF_RIGHT },
	{ NULL }
};

static const CPopupMenu::tPopupMenuItem SelectMenu[] = {
	{ CPopupMenu::MB_ITEM, CMD_SET_VARIABLE,	"_Variable" },
	{ CPopupMenu::MB_ITEM, CMD_SET_COMPARISON,	"_Comparison" },
	{ CPopupMenu::MB_ITEM, CMD_SET_CHECK,		"Chec_k" },
	{ CPopupMenu::MB_SEPARATOR },
	{ CPopupMenu::MB_ITEM, CMD_UNSET,			"_Unselect" },
	{ CPopupMenu::MB_END }
};

//-------------------------   PRIVATE DATA   ---------------------------

static const struct {
	const gchar *label;
	CmpackColor color;
} Types[CMPACK_SELECT_COUNT] = {
	{ NULL, CMPACK_COLOR_DEFAULT },
	{ "var", CMPACK_COLOR_RED },
	{ "comp", CMPACK_COLOR_GREEN },
	{ "check", CMPACK_COLOR_BLUE }
};

//------------------------  COMBO BOX STRINGS   ---------------------------------

static GtkToolItem *toolbar_new_button_from_stock(GtkWidget *toolbar, const gchar *stock_id,
	const gchar *tooltip)
{
	GtkToolItem *tbtn = gtk_tool_button_new_from_stock(stock_id);
	gtk_widget_set_tooltip_text(GTK_WIDGET(tbtn), tooltip);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

//-------------------------   PREVIEW DIALOG   --------------------------------

//
// Constructor
//
CCatFileDlg::CCatFileDlg(void):m_Updating(false), m_UpdatePos(true), m_UpdateZoom(true), 
	m_EditSelection(false), m_LastPosX(-1), m_LastPosY(-1), m_LastFocus(-1), 
	m_SelectedIndex(-1), m_SelectedStar(-1), m_InfoMode(INFO_NONE)
{
	GtkWidget *tbox, *hbox, *scrwnd;

	m_Rulers = CConfig::GetBool("PreviewDlg", "Rulers");
	m_Negative = CConfig::GetBool(CConfig::NEGATIVE_CHARTS);

	// Menu bar
	m_Menu.Create(PreviewMenu, false);
	m_Menu.RegisterCallback(MenuCallback, this);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_Menu.Handle(), false, false, 0);

	// Toolbar
	tbox = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(tbox), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(tbox), GTK_ORIENTATION_HORIZONTAL);
	gtk_box_pack_start(GTK_BOX(m_MainBox), tbox, false, false, 0);
	m_ZoomFit = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_FIT, "Fit the frame to the window");
	g_signal_connect(G_OBJECT(m_ZoomFit), "clicked", G_CALLBACK(button_clicked), this);
	m_ZoomOut = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_OUT, "Zoom out");
	g_signal_connect(G_OBJECT(m_ZoomOut), "clicked", G_CALLBACK(button_clicked), this);
	m_ZoomIn = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_IN, "Zoom in");
	g_signal_connect(G_OBJECT(m_ZoomIn), "clicked", G_CALLBACK(button_clicked), this);

	// Notice bar at the top
	m_NoteBox = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(m_NoteBox), GTK_SHADOW_NONE);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_NoteBox, FALSE, TRUE, 0);
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(m_NoteBox), hbox);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);
	m_NoteIcon = gtk_image_new();
	gtk_box_pack_start(GTK_BOX(hbox), m_NoteIcon, FALSE, TRUE, 8);
	m_NoteCap = gtk_label_new(NULL);
	gtk_box_pack_start(GTK_BOX(hbox), m_NoteCap, TRUE, TRUE, 8);
	gtk_misc_set_alignment(GTK_MISC(m_NoteCap), 0.0, 0.5);
	m_NoteBtn = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
	gtk_widget_set_tooltip_text(m_NoteBtn, "Save changes and leave editing mode");
	gtk_box_pack_start(GTK_BOX(hbox), m_NoteBtn, FALSE, TRUE, 8);
	g_signal_connect(G_OBJECT(m_NoteBtn), "clicked", G_CALLBACK(button_clicked), this);

	// Chart and Infobox
	hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(m_MainBox), hbox, TRUE, TRUE, 0);

	// Chart
	m_ChartData = cmpack_chart_data_new();
	m_Chart = cmpack_chart_view_new();
	cmpack_chart_view_set_selection_mode(CMPACK_CHART_VIEW(m_Chart), GTK_SELECTION_SINGLE);
	cmpack_chart_view_set_activation_mode(CMPACK_CHART_VIEW(m_Chart), CMPACK_ACTIVATION_DBLCLICK);
	cmpack_chart_view_show_scales(CMPACK_CHART_VIEW(m_Chart), m_Rulers);
	cmpack_chart_view_set_mouse_control(CMPACK_CHART_VIEW(m_Chart), TRUE);
	g_signal_connect(G_OBJECT(m_Chart), "zoom-changed", G_CALLBACK(zoom_changed), this);
	g_signal_connect(G_OBJECT(m_Chart), "mouse-moved", G_CALLBACK(mouse_moved), this);
	g_signal_connect(G_OBJECT(m_Chart), "selection-changed", G_CALLBACK(selection_changed), this);
	g_signal_connect(G_OBJECT(m_Chart), "item-activated", G_CALLBACK(item_activated), this);
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_Chart);
	gtk_box_pack_start(GTK_BOX(hbox), scrwnd, TRUE, TRUE, 0);

	// Info box
	m_InfoBox.RegisterCallback(InfoBoxCallback, this);
	gtk_box_pack_start(GTK_BOX(hbox), m_InfoBox.Handle(), FALSE, FALSE, 0);
	m_InfoBox.SetCaption("Object inspector");
	
	// Timers
	m_TimerId = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 100, GSourceFunc(timer_cb), this, NULL);

	// Make popup menus
	m_SelectMenu.Create(SelectMenu);

	// Show the dialog
	gtk_widget_show_all(m_MainBox);
	gtk_widget_hide(m_NoteBox);
	m_InfoBox.Show(false);
}

//
// Destructor
//
CCatFileDlg::~CCatFileDlg(void)
{
	g_signal_handlers_disconnect_by_func(G_OBJECT(m_Chart), (gpointer)zoom_changed, this);
	g_signal_handlers_disconnect_by_func(G_OBJECT(m_Chart), (gpointer)mouse_moved, this);
	g_source_remove(m_TimerId);
	g_object_unref(m_ChartData);
}

//
// Load catalog file
//
bool CCatFileDlg::LoadFile(const char *fpath)
{
	int res;
	CCatalog file;

	m_Selection.Clear();
	m_Tags.Clear();
	SetStatus(NULL);
	SetInfoMode(INFO_NONE);
	m_File.Create();
	if (file.Open(fpath, CMPACK_OPEN_READONLY, &res)) {
		m_File.Assign(file);
		m_Selection = *file.Selection();
		m_Tags = *file.Tags();
		UpdateChart();
		UpdateControls();
		return true;
	} else {
		char *msg = cmpack_formaterror(res);
		ShowError(GTK_WINDOW(m_pDlg), msg);
		cmpack_free(msg);
		return false;
	}
}

void CCatFileDlg::UpdateControls(void)
{
	m_Menu.Enable(CMD_SAVE, m_NotSaved);
	m_Menu.Enable(CMD_INSPECTOR, !m_EditSelection);
	m_Menu.Check(CMD_RULERS, m_Rulers);
	m_Menu.Check(CMD_INSPECTOR, m_InfoMode == INFO_OBJECT);
	m_Menu.Check(CMD_STARS, m_EditSelection);
}

void CCatFileDlg::UpdateChart(void)
{
	cmpack_chart_view_set_model(CMPACK_CHART_VIEW(m_Chart), NULL);
	cmpack_chart_data_clear(m_ChartData);
	m_File.Render(m_ChartData);
	cmpack_chart_view_set_model(CMPACK_CHART_VIEW(m_Chart), m_ChartData);
	m_SelectedIndex = cmpack_chart_data_find_item(m_ChartData, (intptr_t)m_SelectedStar);
	if (m_SelectedIndex>=0)
		cmpack_chart_view_select(CMPACK_CHART_VIEW(m_Chart), m_SelectedIndex);
	cmpack_chart_view_set_negative(CMPACK_CHART_VIEW(m_Chart), m_Negative);
	UpdateZoom();
}

void CCatFileDlg::MenuCallback(CCBObject *sender, int message, int wparam, void *lparam, void *cb_data)
{
	CCatFileDlg *pMe = (CCatFileDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}

void CCatFileDlg::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	// File menu
	case CMD_OPEN:
		Open();
		break;
	case CMD_SAVE:
		Save();
		break;
	case CMD_SAVE_AS:
		SaveAs();
		break;
	case CMD_CLOSE:
		Close();
		break;
	case CMD_PROPERTIES:
		ShowProperties();
		break;
	case CMD_EXPORT:
		ExportChart();
		break;

	// Edit menu
	case CMD_HEADER:
		EditHeader();
		break;
	case CMD_STARS:
		SetEditSelection(!m_EditSelection);
		break;

	// View menu
	case CMD_RULERS:
		m_Rulers = m_Menu.IsChecked(CMD_RULERS);
		cmpack_chart_view_show_scales(CMPACK_CHART_VIEW(m_Chart), m_Rulers);
		CConfig::SetBool("PreviewDlg", "Rulers", m_Rulers);
		break;

	// Tools menu
	case CMD_INSPECTOR:
		if (m_Menu.IsChecked(CMD_INSPECTOR)) 
			SetInfoMode(INFO_OBJECT);
		else if (m_InfoMode == INFO_OBJECT)
			SetInfoMode(INFO_NONE);
		break;

	// Help menu
	case CMD_HELP:
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_CATALOG_FILE_WINDOW);
		break;
	}
}

bool CCatFileDlg::SaveFile(const gchar *fpath)
{
	int res = 0;
	GError *error = NULL;

	// Make catalog file
	CCatalog dst;
	if (dst.Open(fpath, CMPACK_OPEN_CREATE, &res)) {
		dst.Assign(m_File, NULL);
		dst.SetSelection(m_Selection);
		dst.SetTags(m_Tags);
		dst.Close();
		// Copy FTS file
		gchar *src_fits = SetFileExtension(m_Path, FILE_EXTENSION_FITS);
		if (g_file_test(src_fits, G_FILE_TEST_IS_REGULAR)) {
			gchar *dst_fits = SetFileExtension(m_Path, FILE_EXTENSION_FITS);
			if (ComparePaths(src_fits, dst_fits)!=0) {
				if (!copy_file(src_fits, dst_fits, false, &error)) {
					ShowError(GTK_WINDOW(m_pDlg), error->message);
					g_error_free(error);
				}
			}
			g_free(dst_fits);
		}
		g_free(src_fits);
	} else {
		char *msg = cmpack_formaterror(res);
		ShowError(GTK_WINDOW(m_pDlg), msg);
		cmpack_free(msg);
	}
	return (res==0);
}

void CCatFileDlg::ShowProperties(void)
{
	CCatFileInfoDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.ShowModal(&m_File, m_Name, m_Path);
}

void CCatFileDlg::EditHeader(void)
{
	CCatEditHeaderDlg dlg(GTK_WINDOW(m_pDlg));
	if (dlg.Execute(m_File, m_Name)) {
		m_NotSaved = true;
		UpdateTitle();
		UpdateControls();
	}
}

void CCatFileDlg::SetEditSelection(bool enable)
{
	if (enable) {
		// Turn editing on
		m_EditSelection = true;
		SetInfoMode(INFO_NONE);
		cmpack_chart_view_set_selection_mode(CMPACK_CHART_VIEW(m_Chart), GTK_SELECTION_NONE);
		cmpack_chart_view_set_activation_mode(CMPACK_CHART_VIEW(m_Chart), CMPACK_ACTIVATION_CLICK);
		gtk_image_set_from_stock(GTK_IMAGE(m_NoteIcon), GTK_STOCK_EDIT, GTK_ICON_SIZE_LARGE_TOOLBAR);
		gtk_label_set_text(GTK_LABEL(m_NoteCap), "You can edit the selection of stars now. Click on a star to change its type.");
		gtk_widget_show_all(m_NoteBox);
	} else {
		// Turn editing off
		m_EditSelection = false;
		cmpack_chart_view_set_selection_mode(CMPACK_CHART_VIEW(m_Chart), GTK_SELECTION_SINGLE);
		cmpack_chart_view_set_activation_mode(CMPACK_CHART_VIEW(m_Chart), CMPACK_ACTIVATION_DBLCLICK);
		gtk_widget_hide(m_NoteBox);
	}
	UpdateStatus();
	UpdateControls();
}

void CCatFileDlg::button_clicked(GtkWidget *button, CCatFileDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CCatFileDlg::OnButtonClicked(GtkWidget *pBtn)
{
	double zoom;

	if (pBtn==GTK_WIDGET(m_ZoomIn)) {
		zoom = cmpack_chart_view_get_zoom(CMPACK_CHART_VIEW(m_Chart));
		cmpack_chart_view_set_zoom(CMPACK_CHART_VIEW(m_Chart), zoom + 5.0);
	} else 
	if (pBtn==GTK_WIDGET(m_ZoomOut)) {
		zoom = cmpack_chart_view_get_zoom(CMPACK_CHART_VIEW(m_Chart));
		cmpack_chart_view_set_zoom(CMPACK_CHART_VIEW(m_Chart), zoom - 5.0);
	} else
	if (pBtn==GTK_WIDGET(m_ZoomFit)) {
		cmpack_chart_view_set_auto_zoom(CMPACK_CHART_VIEW(m_Chart), TRUE);
	} else
	if (pBtn==GTK_WIDGET(m_NoteBtn)) {
		if (m_EditSelection)
			SetEditSelection(false);
	}
}

//
// Left button click
//
void CCatFileDlg::selection_changed(GtkWidget *pChart, CCatFileDlg *pDlg)
{
	pDlg->OnSelectionChanged();
}

void CCatFileDlg::OnSelectionChanged(void)
{
	int row;

	row = cmpack_chart_view_get_selected(CMPACK_CHART_VIEW(m_Chart));
	if (row>=0) {
		if (row != m_SelectedIndex) {
			m_SelectedIndex = row;
			m_SelectedStar = (int)cmpack_chart_data_get_param(m_ChartData, m_SelectedIndex);
			UpdateInfoBox();
		}
	} else {
		if (m_SelectedIndex>=0 || m_SelectedStar>=0) {
			m_SelectedIndex = m_SelectedStar = -1;
			UpdateInfoBox();
		}
	}
}

void CCatFileDlg::item_activated(GtkWidget *button, gint row, CCatFileDlg *pDlg)
{
	pDlg->OnItemActivated(row);
}

void CCatFileDlg::OnItemActivated(gint row)
{
	int star_id, index;
	CmpackSelectionType type;
	GdkEventButton ev;

	if (!m_EditSelection) {
		if (m_InfoMode != INFO_OBJECT)
			SetInfoMode(INFO_OBJECT);
		OnSelectionChanged();
	} else {
		if (row>=0) {
			star_id = (int)cmpack_chart_data_get_param(m_ChartData, row);
			index = m_Selection.IndexOf(star_id);
			if (index>=0)
				type = m_Selection.GetType(index);
			else
				type = CMPACK_SELECT_NONE;
			m_SelectMenu.Enable(CMD_SET_VARIABLE, type!=CMPACK_SELECT_VAR);
			m_SelectMenu.Enable(CMD_SET_COMPARISON, type!=CMPACK_SELECT_COMP);
			m_SelectMenu.Enable(CMD_SET_CHECK, type!=CMPACK_SELECT_CHECK);
			m_SelectMenu.Enable(CMD_UNSET, type!=CMPACK_SELECT_NONE);
			ev.button = 1;
			ev.time = gtk_get_current_event_time();
			switch (m_SelectMenu.Execute(&ev))
			{
			case CMD_SET_VARIABLE:
				UnselectType(CMPACK_SELECT_VAR);
				Select(row, CMPACK_SELECT_VAR);
				break;
			case CMD_SET_COMPARISON:
				if (!g_Project->Profile()->GetBool(CProfile::MULTIPLE_COMP_STARS))
					UnselectType(CMPACK_SELECT_COMP);
				Select(row, CMPACK_SELECT_COMP);
				break;
			case CMD_SET_CHECK:
				Select(row, CMPACK_SELECT_CHECK);
				break;
			case CMD_UNSET:
				Unselect(row);
				break;
			}
		}
	}
}

void CCatFileDlg::Select(int row, CmpackSelectionType type)
{
	int i, index, star_id;
	char buf[512];
	
	if (row>=0) {
		star_id = (int)cmpack_chart_data_get_param(m_ChartData, row);
		i = m_Selection.Select(star_id, type);
		index = m_Selection.GetIndex(i);
		if (index==1)
			strcpy(buf, Types[type].label);
		else
			sprintf(buf, "%s #%d", Types[type].label, index);
		cmpack_chart_data_set_tag(m_ChartData, row, buf);
		cmpack_chart_data_set_color(m_ChartData, row, Types[type].color);
		cmpack_chart_data_set_topmost(m_ChartData, row, TRUE);
		if (!m_NotSaved) {
			m_NotSaved = true;
			UpdateTitle();
			UpdateControls();
		}
	}
}

void CCatFileDlg::Unselect(int row)
{
	int star_id;

	if (row>=0) {
		star_id = (int)cmpack_chart_data_get_param(m_ChartData, row);
		m_Selection.Select(star_id, CMPACK_SELECT_NONE);
		cmpack_chart_data_set_tag(m_ChartData, row, NULL);
		cmpack_chart_data_set_color(m_ChartData, row, CMPACK_COLOR_DEFAULT);
		cmpack_chart_data_set_topmost(m_ChartData, row, FALSE);
		if (!m_NotSaved) {
			m_NotSaved = true;
			UpdateTitle();
			UpdateControls();
		}
	}
}

void CCatFileDlg::UnselectType(CmpackSelectionType type)
{
	int i, count;

	count = m_Selection.CountStars(type);
	if (count>0) {
		int *stars = (int*)g_malloc(count*sizeof(int));
		m_Selection.GetStarList(type, stars, count);
		for (i=0; i<count; i++) 
			Unselect(cmpack_chart_data_find_item(m_ChartData, stars[i]));
		g_free(stars);
		if (count>0 && !m_NotSaved) {
			m_NotSaved = true;
			UpdateTitle();
			UpdateControls();
		}
	}
}

void CCatFileDlg::mouse_moved(GtkWidget *button, CCatFileDlg *pDlg)
{
	pDlg->m_UpdatePos = true;
}

void CCatFileDlg::zoom_changed(GtkWidget *button, CCatFileDlg *pDlg)
{
	pDlg->m_UpdateZoom = true;
}

gboolean CCatFileDlg::timer_cb(CCatFileDlg *pDlg)
{
	if (pDlg->m_UpdatePos) {
		pDlg->m_UpdatePos = false;
		pDlg->UpdateStatus();
	}
	if (pDlg->m_UpdateZoom) {
		pDlg->m_UpdateZoom = false;
		pDlg->UpdateZoom();
	}
	return TRUE;
}

void CCatFileDlg::InfoBoxCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data)
{
	CCatFileDlg *pMe = (CCatFileDlg*)cb_data;

	switch (message)
	{
	case CInfoBox::CB_CLOSED:
		pMe->OnInfoBoxClosed();
		break;
	}
}

void CCatFileDlg::OnInfoBoxClosed(void)
{
	m_InfoMode = INFO_NONE;
	UpdateStatus();
	UpdateControls();
}

void CCatFileDlg::UpdateStatus(void)
{
	char	buf[256];
	CmpackCatObject obj;
	int		x, y, star_id, index;

	int item = cmpack_chart_view_get_focused(CMPACK_CHART_VIEW(m_Chart));
	if (item>=0) {
		if (m_LastFocus!=item) {
			m_LastFocus = item;
			star_id = (int)cmpack_chart_data_get_param(m_ChartData, item);
			index = m_File.FindObject(star_id);
			m_File.GetObject(index, CMPACK_OM_CENTER, &obj);
			sprintf(buf, "Object #%d: X = %.1f, Y = %.1f", 
				star_id, obj.center_x, obj.center_y);
			SetStatus(buf);
		}
	} else {
		m_LastFocus = -1;
		if (cmpack_chart_view_mouse_pos(CMPACK_CHART_VIEW(m_Chart), &x, &y)) {
			if (x!=m_LastPosX || y!=m_LastPosY) {
				m_LastPosX = x;
				m_LastPosY = y;
				sprintf(buf, "Cursor: X = %d, Y = %d", x, y);
				SetStatus(buf);
			}
		} else {
			if (m_LastPosX!=-1 || m_LastPosY!=-1) {
				m_LastPosX = m_LastPosY = -1;
				SetStatus(NULL);
			}
		}
	}
}

void CCatFileDlg::UpdateZoom(void)
{
	double zoom, min, max;

	zoom = cmpack_chart_view_get_zoom(CMPACK_CHART_VIEW(m_Chart));
	cmpack_chart_view_get_zoom_limits(CMPACK_CHART_VIEW(m_Chart), &min, &max);
	gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomIn), zoom < max);
	gtk_widget_set_sensitive(GTK_WIDGET(m_ZoomOut), zoom > min);
}

void CCatFileDlg::SetInfoMode(tInfoMode mode)
{
	m_InfoMode = mode;
	switch (m_InfoMode) 
	{
	case INFO_OBJECT:
		m_InfoBox.Show(true);
		break;
	default:
		m_InfoBox.Show(false);
		break;
	}
	UpdateInfoBox();
	UpdateControls();
}

void CCatFileDlg::UpdateInfoBox(void)
{
	int		index;
	char	buf[256];
	CmpackCatObject obj;

	if (m_InfoMode == INFO_OBJECT) {
		m_InfoBox.BeginUpdate();
		m_InfoBox.Clear();
		// Show information about selected star
		if (m_SelectedStar>=0) {
			sprintf(buf, "Object #%d", m_SelectedStar);
			m_InfoBox.AddTitle(1, buf);
			// Position
			index = m_File.FindObject(m_SelectedStar);
			m_File.GetObject(index, CMPACK_OM_CENTER | CMPACK_OM_MAGNITUDE, &obj);
			sprintf(buf, "Center(X) = %.1f", obj.center_x);
			m_InfoBox.AddText(buf);
			sprintf(buf, "Center(Y) = %.1f", obj.center_y);
			m_InfoBox.AddText(buf);
			// Brightness
			if (obj.refmag_valid) {
				sprintf(buf, "Mag = %.4f mag", obj.refmagnitude);
				m_InfoBox.AddText(buf);
			}
			// Selection
			index = m_Selection.IndexOf(m_SelectedStar);
			if (index>=0) {
				switch (m_Selection.GetType(index))
				{
				case CMPACK_SELECT_VAR:
					if (m_Selection.GetIndex(index)==1)
						sprintf(buf, "Variable star");
					else
						sprintf(buf, "Variable star #%d", m_Selection.GetIndex(index));
					m_InfoBox.AddText(buf);
					break;
				case CMPACK_SELECT_COMP:
					if (m_Selection.GetIndex(index)==1)
						sprintf(buf, "Comparison star");
					else
						sprintf(buf, "Comparison star #%d", m_Selection.GetIndex(index));
					m_InfoBox.AddText(buf);
					break;
				case CMPACK_SELECT_CHECK:
					sprintf(buf, "Check star #%d", m_Selection.GetIndex(index));
					m_InfoBox.AddText(buf);
					break;
				default:
					break;
				}
			}
		} else {
			m_InfoBox.AddText("Click on the object");
			m_InfoBox.AddText("to show its properties");
		}
		m_InfoBox.EndUpdate();
	}
}

void CCatFileDlg::ExportChart(void)
{
	CChartExportDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.Execute(m_ChartData, NULL);
}

//---------------------------   INFO DIALOG   --------------------------------

//
// Constructor
//
CCatFileInfoDlg::CCatFileInfoDlg(GtkWindow *pParent):CInfoDlg(pParent, 0),
	m_Name(NULL)
{
	GtkWidget *bbox;

	AddHeading(FIELD_FILE, 0, 0, "File");
	AddField(FIELD_FILENAME, 0, 1, "File name", PANGO_ELLIPSIZE_MIDDLE);
	AddField(FIELD_DIRPATH, 0, 2, "Location", PANGO_ELLIPSIZE_MIDDLE);
	AddSeparator(0, 3);
	AddHeading(FIELD_FRAME, 0, 4, "Frame");
	AddField(FIELD_DATETIME, 0, 5, "Date and time");
	AddField(FIELD_JULDAT, 0, 6, "Julian date");
	AddField(FIELD_FILTER, 0, 7, "Optical filter", PANGO_ELLIPSIZE_END);
	AddField(FIELD_EXPTIME, 0, 8, "Exposure duration");
	AddSeparator(0, 9);
	AddHeading(FIELD_PHOTOMETRY, 0, 10, "Photometry");
	AddField(FIELD_STARS, 0, 11, "Number of stars");
	gtk_widget_show_all(m_Tab);

	bbox = gtk_hbutton_box_new();
	m_HdrBtn = gtk_button_new_with_label("Show header");
	gtk_box_pack_start(GTK_BOX(bbox), m_HdrBtn, true, true, 0);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), bbox);
	gtk_widget_show_all(bbox);

	g_signal_connect(G_OBJECT(m_HdrBtn), "clicked", G_CALLBACK(button_clicked), this);
}

//
// Show dialog
//
void CCatFileInfoDlg::ShowModal(const CCatalog *file, const gchar *name, const gchar *path)
{
	gchar buf[FILENAME_MAX+32], *dirpath;
	CmpackDateTime dt;

	m_File = file;
	m_Name = name;

	// Window caption
	sprintf(buf, "%s - %s", m_Name, "properties");
	gtk_window_set_title(GTK_WINDOW(m_pDlg), buf);

	// Update properties
	SetField(FIELD_FILENAME, g_basename(path));
	dirpath = g_path_get_dirname(path);
	SetField(FIELD_DIRPATH, dirpath);
	g_free(dirpath);

	if (file->DateTime(&dt)) {
		sprintf(buf, "%04d-%02d-%02d %d:%02d:%02d", dt.date.year, dt.date.month, dt.date.day,
			dt.time.hour, dt.time.minute, dt.time.second);
		SetField(FIELD_DATETIME, buf);
		SetField(FIELD_JULDAT, file->JulianDate(), 7);
	} else {
		SetField(FIELD_DATETIME, "Not available");
		SetField(FIELD_JULDAT, "Not available");
	}
	if (file->Filter())
		SetField(FIELD_FILTER, file->Filter());
	else
		SetField(FIELD_FILTER, "");
	if (file->ExposureDuration()>0)
		SetField(FIELD_EXPTIME, file->ExposureDuration(), 3, "second(s)");
	else
		SetField(FIELD_EXPTIME, "Not available");

	if (file->ObjectCount()>=0)
		SetField(FIELD_STARS, file->ObjectCount());
	else
		SetField(FIELD_STARS, "Not available");

	CInfoDlg::ShowModal();
}

void CCatFileInfoDlg::button_clicked(GtkWidget *button, CCatFileInfoDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CCatFileInfoDlg::OnButtonClicked(GtkWidget *pBtn)
{
	if (pBtn==GTK_WIDGET(m_HdrBtn))
		ShowHeader();
}

void CCatFileInfoDlg::ShowHeader(void)
{
	CShowHeaderDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.Execute(m_File, m_Name);
}

//-------------------------   MAIN WINDOW   --------------------------------

CCatEditHeaderDlg::CCatEditHeaderDlg(GtkWindow *pParent)
{
	GtkWidget *hbox, *vbox, *tbox, *label, *scrwnd;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons(NULL, pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, NULL);
	gtk_dialog_widget_standard_tooltips(GTK_DIALOG(m_pDlg));

	// Dialog icon
	gchar *icon = get_icon_file("catalogfile");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	hbox = gtk_hbox_new(TRUE, 16);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), hbox, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 4);

	// Object + Location (left panel)
	tbox = gtk_table_new(6, 3, FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), tbox, TRUE, TRUE, 0);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 2, 12);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 5, 12);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 4);
	label = gtk_label_new("Object - designation");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	m_ObjName = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_ObjName, "Object's designation, e.g. RT And");
	gtk_entry_set_max_length(GTK_ENTRY(m_ObjName), MAX_OBJECT_LEN);
	gtk_table_attach(GTK_TABLE(tbox), m_ObjName, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	m_ObjBtn = gtk_button_new_with_label("More");
	gtk_widget_set_tooltip_text(m_ObjBtn, "Load object coordinates from presets or variable star catalogs");
	gtk_table_attach(GTK_TABLE(tbox), m_ObjBtn, 2, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("- right ascension");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	m_RA = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_RA, "Object's right ascension in hours, minutes and seconds, e.g. 12 34 56");
	gtk_entry_set_max_length(GTK_ENTRY(m_RA), 64);
	gtk_table_attach(GTK_TABLE(tbox), m_RA, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("[h m s]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 2, 3, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("- declination");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	m_Dec = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Dec, "Object's declination in degrees, minutes and seconds, e.g. -12 34 56");
	gtk_entry_set_max_length(GTK_ENTRY(m_Dec), 64);
	gtk_table_attach(GTK_TABLE(tbox), m_Dec, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("[\xC2\xB1""d m s]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 2, 3, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("Location - name");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
	m_LocName = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_LocName, "Observer's location designation, e.g. Brno, Czech Republic");
	gtk_entry_set_max_length(GTK_ENTRY(m_LocName), MAX_LOCATION_LEN);
	gtk_table_attach(GTK_TABLE(tbox), m_LocName, 1, 2, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
	m_LocBtn = gtk_button_new_with_label("More");
	gtk_widget_set_tooltip_text(m_LocBtn, "Load observer's coordinates from presets");
	gtk_table_attach(GTK_TABLE(tbox), m_LocBtn, 2, 3, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("- longitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
	m_Lon = gtk_entry_new_with_max_length(32);
	gtk_widget_set_tooltip_text(m_Lon, "Observer's longitude in degrees, minutes and seconds, use E or W suffix to indicate hemisphere, e.g. 12 34 56 E");
	gtk_table_attach(GTK_TABLE(tbox), m_Lon, 1, 2, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("[d m s E/W]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 2, 3, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("- latitude");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
	m_Lat = gtk_entry_new_with_max_length(32);
	gtk_widget_set_tooltip_text(m_Lat, "Observer's latitude in degrees, minutes and seconds, use N or S suffix to indicate hemisphere, e.g. 56 34 12 N");
	gtk_table_attach(GTK_TABLE(tbox), m_Lat, 1, 2, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("[d m s N/S]");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 2, 3, 5, 6, GTK_FILL, GTK_FILL, 0, 0);

	// Other fields (right panel)
	tbox = gtk_table_new(6, 2, FALSE);
	gtk_box_pack_start(GTK_BOX(hbox), tbox, TRUE, TRUE, 0);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 4);
	label = gtk_label_new("Observer's name");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	m_Observer = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Observer, "Observer's name");
	gtk_entry_set_max_length(GTK_ENTRY(m_Observer), MAX_OBSERVER_LEN);
	gtk_table_attach(GTK_TABLE(tbox), m_Observer, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("Telescope");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	m_Telescope = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Telescope, "Short description of the telescope");
	gtk_table_attach(GTK_TABLE(tbox), m_Telescope, 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("CCD Camera");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	m_Camera = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Camera, "Camera brand name and model");
	gtk_table_attach(GTK_TABLE(tbox), m_Camera, 1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("Color filter");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
	m_Filter = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Filter, "Color filter designation");
	gtk_table_attach(GTK_TABLE(tbox), m_Filter, 1, 2, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("Field of view");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
	m_FOV = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_FOV, "Size of view field, width x height");
	gtk_table_attach(GTK_TABLE(tbox), m_FOV, 1, 2,  4, 5, GTK_FILL, GTK_FILL, 0, 0);
	label = gtk_label_new("Orientation");
	gtk_misc_set_alignment(GTK_MISC(label), 1.0, 0.5);
	gtk_table_attach(GTK_TABLE(tbox), label, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
	m_Orientation = gtk_entry_new();
	gtk_widget_set_tooltip_text(m_Orientation, "View field orientation, e.g. top=N, left=W");
	gtk_table_attach(GTK_TABLE(tbox), m_Orientation, 1, 2, 5, 6, GTK_FILL, GTK_FILL, 0, 0);

	// Notes (bottom)
	vbox = gtk_vbox_new(FALSE, 4);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), vbox, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
	label = gtk_label_new("Remarks");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, TRUE, 0);
	m_Notes = gtk_text_view_new();
	gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(m_Notes), GTK_WRAP_WORD_CHAR);
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd),
		GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), 
		GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_Notes);
	gtk_widget_set_size_request(scrwnd, 640, 140);
	gtk_box_pack_start(GTK_BOX(vbox), scrwnd, FALSE, TRUE, 0);

	// Signals
	g_signal_connect(G_OBJECT(m_ObjBtn), "clicked", G_CALLBACK(button_clicked), this);
	g_signal_connect(G_OBJECT(m_LocBtn), "clicked", G_CALLBACK(button_clicked), this);

	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);
}

CCatEditHeaderDlg::~CCatEditHeaderDlg()
{
	gtk_widget_destroy(m_pDlg);
}

bool CCatEditHeaderDlg::Execute(CCatalog &file, const gchar *name)
{
	gchar title[512];

	SetData(file);
	sprintf(title, "%s - %s", name, "Edit header");
	gtk_window_set_title(GTK_WINDOW(m_pDlg), title);
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))==GTK_RESPONSE_ACCEPT) {
		GetData(file);
		return true;
	}
	return false;
}

void CCatEditHeaderDlg::SetData(CCatalog &file)
{
	// Object
	const CObjectCoords *obj = file.Object();
	if (obj->Name())
		gtk_entry_set_text(GTK_ENTRY(m_ObjName), obj->Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_ObjName), "");
	if (obj->RA())
		gtk_entry_set_text(GTK_ENTRY(m_RA), obj->RA());
	else
		gtk_entry_set_text(GTK_ENTRY(m_RA), "");
	if (obj->Dec()) 
		gtk_entry_set_text(GTK_ENTRY(m_Dec), obj->Dec());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Dec), "");

	// Location
	const CLocation *loc = file.Location();
	if (loc->Name())
		gtk_entry_set_text(GTK_ENTRY(m_LocName), loc->Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_LocName), "");
	if (loc->Lon())
		gtk_entry_set_text(GTK_ENTRY(m_Lon), loc->Lon());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lon), "");
	if (loc->Lat())
		gtk_entry_set_text(GTK_ENTRY(m_Lat), loc->Lat());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lat), "");

	// Other parameters
	const gchar *observer = file.Observer();
	if (observer)
		gtk_entry_set_text(GTK_ENTRY(m_Observer), observer);
	else
		gtk_entry_set_text(GTK_ENTRY(m_Observer), "");
	const gchar *filter = file.Filter();
	if (filter)
		gtk_entry_set_text(GTK_ENTRY(m_Filter), filter);
	else
		gtk_entry_set_text(GTK_ENTRY(m_Filter), "");
	const gchar *telescope = file.Telescope();
	if (telescope)
		gtk_entry_set_text(GTK_ENTRY(m_Telescope), telescope);
	else
		gtk_entry_set_text(GTK_ENTRY(m_Telescope), "");
	const gchar *camera = file.Camera();
	if (camera)
		gtk_entry_set_text(GTK_ENTRY(m_Camera), camera);
	else
		gtk_entry_set_text(GTK_ENTRY(m_Camera), "");
	const gchar *fov = file.FieldOfView();
	if (fov)
		gtk_entry_set_text(GTK_ENTRY(m_FOV), fov);
	else
		gtk_entry_set_text(GTK_ENTRY(m_FOV), "");
	const gchar *orientation = file.Orientation();
	if (orientation)
		gtk_entry_set_text(GTK_ENTRY(m_Orientation), orientation);
	else
		gtk_entry_set_text(GTK_ENTRY(m_Orientation), "");

	// Comments
	const gchar *notes = file.Notes();
	if (notes)
		gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(m_Notes)), notes, -1);
	else
		gtk_text_buffer_set_text(gtk_text_view_get_buffer(GTK_TEXT_VIEW(m_Notes)), "", -1);
}

void CCatEditHeaderDlg::GetData(CCatalog &file) const
{
	char *notes;
	GtkTextBuffer *textbuf;
	GtkTextIter start, end;
	CObjectCoords obj;
	CLocation loc;

	// Object
	obj.SetName(gtk_entry_get_text(GTK_ENTRY(m_ObjName)));
	obj.SetRA(gtk_entry_get_text(GTK_ENTRY(m_RA)));
	obj.SetDec(gtk_entry_get_text(GTK_ENTRY(m_Dec)));
	file.SetObject(obj);
	// Location
	loc.SetName(gtk_entry_get_text(GTK_ENTRY(m_LocName)));
	loc.SetLon(gtk_entry_get_text(GTK_ENTRY(m_Lon)));
	loc.SetLat(gtk_entry_get_text(GTK_ENTRY(m_Lat)));
	file.SetLocation(loc);
	// Other fields
	file.SetObserver(gtk_entry_get_text(GTK_ENTRY(m_Observer)));
	file.SetFilter(gtk_entry_get_text(GTK_ENTRY(m_Filter)));
	file.SetTelescope(gtk_entry_get_text(GTK_ENTRY(m_Telescope)));
	file.SetCamera(gtk_entry_get_text(GTK_ENTRY(m_Camera)));
	file.SetFieldOfView(gtk_entry_get_text(GTK_ENTRY(m_FOV)));
	file.SetOrientation(gtk_entry_get_text(GTK_ENTRY(m_Orientation)));
	// Comments
	textbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(m_Notes));
	gtk_text_buffer_get_start_iter(textbuf, &start);
	gtk_text_buffer_get_end_iter(textbuf, &end);
	notes = gtk_text_buffer_get_text(textbuf, &start, &end, FALSE);
	file.SetNotes(notes);
	g_free(notes);
}

void CCatEditHeaderDlg::button_clicked(GtkWidget *pButton, CCatEditHeaderDlg *pMe)
{
	pMe->OnButtonClicked(pButton);
}

void CCatEditHeaderDlg::OnButtonClicked(GtkWidget *pButton)
{
	if (pButton == m_ObjBtn) 
		EditObjectCoords();
	else if (pButton == m_LocBtn) 
		EditLocation();
}

void CCatEditHeaderDlg::EditObjectCoords(void)
{
	CObjectCoords obj;
	CObjectDlg dlg(GTK_WINDOW(m_pDlg));

	obj.SetName(gtk_entry_get_text(GTK_ENTRY(m_ObjName)));
	obj.SetRA(gtk_entry_get_text(GTK_ENTRY(m_RA)));
	obj.SetDec(gtk_entry_get_text(GTK_ENTRY(m_Dec)));
	if (dlg.Execute(&obj)) {
		gtk_entry_set_text(GTK_ENTRY(m_ObjName), obj.Name());
		gtk_entry_set_text(GTK_ENTRY(m_RA), obj.RA());
		gtk_entry_set_text(GTK_ENTRY(m_Dec), obj.Dec());
	}
}

void CCatEditHeaderDlg::EditLocation(void)
{
	CLocation obs;
	CLocationDlg dlg(GTK_WINDOW(m_pDlg));

	obs.SetName(gtk_entry_get_text(GTK_ENTRY(m_LocName)));
	obs.SetLon(gtk_entry_get_text(GTK_ENTRY(m_Lon)));
	obs.SetLat(gtk_entry_get_text(GTK_ENTRY(m_Lat)));
	if (dlg.Execute(&obs)) {
		gtk_entry_set_text(GTK_ENTRY(m_LocName), obs.Name());
		gtk_entry_set_text(GTK_ENTRY(m_Lon), obs.Lon());
		gtk_entry_set_text(GTK_ENTRY(m_Lat), obs.Lat());
	}
}
