/**************************************************************

file_classes.h (C-Munipack project)
Catalogs of variable stars
Copyright (C) 2010 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <glib.h>

#include "file_classes.h"

#define BUFSIZE		4096
#define MAX_LINE	65535
#define ALLOC_BY	64

//----------------------   HELPER CLASSES AND FUNCTIONS   -------------------------------

//
// File reader class
//
// This class can read a lines from a text file
// The length of a single line is limited to MAX_LINE characters (excluding line delimiter and null terminator)
// When a longer line occurs in a file, it is truncated.
// The file can contain the BOM at the beginning.
//
class CFileReader
{
public:
	// Constructor
	CFileReader(FILE *f):m_BufPtr(NULL), m_BufEnd(NULL), m_File(f), m_Eof(false)
	{
		size_t bytes = fread(m_Buf, 1, 3, m_File);
		m_BufEnd = m_Buf + bytes;
		if (bytes==3 && m_Buf[0]==0xEF && m_Buf[1]==0xBB && m_Buf[2]==0xBF) 
			m_BufPtr = m_BufEnd;
		else 
			m_BufPtr = m_Buf;
	}

	// Get next line (store the line to buffer)
	// If the line is longer than the buffer, it is truncated
	// Returns number of characters stored to the buffer without trailing nul character
	// or -1 if the end of file was reached
	int NextLine(char *buf, size_t bufsize)
	{
		int state = 0;
		unsigned char ch;
		char *lineptr = buf;
		size_t length;

		length = 0;
		while (true) {
			if (m_BufPtr >= m_BufEnd) {
				size_t bytes = fread(m_Buf, 1, BUFSIZE, m_File);
				if (bytes==0) {
					m_Eof = true;
					break;
				}
				m_BufPtr = m_Buf;
				m_BufEnd = m_Buf + bytes;
			}
			ch = *m_BufPtr;
			if (state==0) {
				if (ch=='\xD') {
					state = 1;
				} else 
				if (ch=='\xA')  {
					m_BufPtr++;
					break;
				} else {
					if (length+1 < bufsize) {
						*lineptr++ = ch;
						length++;
					}
				}
			} else {
				if (ch=='\xA')
					m_BufPtr++;
				break;
			}
			m_BufPtr++;
		}
		if (!m_Eof || length>0) {
			*lineptr = '\0';
			return length;
		}
		return -1;
	}

private:
	unsigned char	m_Buf[BUFSIZE], *m_BufPtr, *m_BufEnd;
	FILE			*m_File;
	bool			m_Eof;
};

// 
// Make copy of a string
//
static char *StrDup(const char *str)
{
	if (str) {
		size_t len = strlen(str);
		char *buf = (char*)malloc((len+1)*sizeof(char));
		if (buf)
			strcpy(buf, str);
		return buf;
	}
	return NULL;
}

//----------------------   dBase FILE READER   -------------------------------

// Constructor
CDBFReader::CDBFReader(FILE *f):m_File(f), m_Cols(NULL), m_Record(NULL), 
	m_ColCount(0)
{
	char fielddef[32];
	int i, j, start, count;

	memset(&m_Header, 0, sizeof(tDBFHeader));
	
	// Read file header
	if (fread(&m_Header, sizeof(tDBFHeader), 1, f)==1) {
		// Read field definitions
		count = (m_Header.head_len - sizeof(tDBFHeader))/32;
		if (count>0 && count<256) {
			m_Cols = (tDBFField*)g_malloc(count*sizeof(tDBFField));
			start = 1;
			m_ColCount = 0;
			for (i=0; i<count; i++) {
				if (fread(fielddef, sizeof(fielddef), 1, f)==1) {
					for (j=0; j<11 && fielddef[j]!='\0'; j++)
						m_Cols[m_ColCount].name[j] = fielddef[j];
					m_Cols[m_ColCount].name[j] = '\0';
					m_Cols[m_ColCount].length = fielddef[16];
					m_Cols[m_ColCount].start = start;
					start += fielddef[16];
					m_ColCount++;
				}
			}
			fseek(m_File, m_Header.head_len, SEEK_SET);
			m_Record = (char*)g_malloc(m_Header.rec_length);
		}
	}
}

// Destructor
CDBFReader::~CDBFReader()
{
	g_free(m_Cols);
	g_free(m_Record);
}

// Get next m_Record
bool CDBFReader::Next()
{
	if (m_File && m_Record) 
		return fread(m_Record, m_Header.rec_length, 1, m_File)==1;
	return false;
}

// Get field name
const char *CDBFReader::FieldName(int col) const
{
	if (col>=0 && col<m_ColCount)
		return m_Cols[col].name;
	return NULL;
}

// Get field value
bool CDBFReader::GetField(int col, char *m_Buf, size_t buflen)
{
	size_t	start, length, pos;

	if (m_Record && col>=0 && col<m_ColCount) {
		start = m_Cols[col].start;
		length = m_Cols[col].length;
		pos = strspn(m_Record + start, " ");
		if (pos >= length) 
			length = 0;
		else
			length -= pos;
		if (length>0) {
			if (length+1 > buflen) {
				memcpy(m_Buf, m_Record+start+pos, buflen-1);
				length = buflen-1;
			} else {
				memcpy(m_Buf, m_Record+start+pos, length);
				while (length>0 && m_Buf[length-1]==' ')
					length--;
			}
		}
		m_Buf[length] = '\0';
		return true;
	} else {
		m_Buf[0] = '\0';
		return false;
	}
}

//--------------------   TEXT FILE READER   -----------------------

// Constructor
CTextReader::CTextReader(FILE *f):m_Line(NULL), m_LineLength(0)
{
	m_Reader = new CFileReader(f);
	m_Line = (char*)g_malloc(MAX_LINE+1);
}

// Destructor
CTextReader::~CTextReader(void)
{
	delete m_Reader;
	g_free(m_Line);
}

// Read next line
bool CTextReader::ReadLine(void)
{
	int length = m_Reader->NextLine(m_Line, MAX_LINE+1);
	while (length>=0) {
		if (length>0) {
			m_LineLength = length;
			return true;
		}
		length = m_Reader->NextLine(m_Line, MAX_LINE+1);
	}
	m_LineLength = 0;
	return false;
}

// Get field
bool CTextReader::GetField(int start, int length, char *m_Buf, int buflen) const
{
	int pos, i;

	if (start < (int)m_LineLength) {
		if (start+length > (int)m_LineLength)
			length = m_LineLength - start;
		if (length >= buflen)
			return false;
		pos = strspn(m_Line + start, " ");
		if (pos >= length) {
			m_Buf[0] = '\0';
		} else {
			memcpy(m_Buf, m_Line+start+pos, length-pos);
			i = length-pos;
			while (i>0 && m_Buf[i-1]==' ')
				i--;
			m_Buf[i] = '\0';
		}
		return true;
	}
	m_Buf[0] = '\0';
	return false;
}

//----------------------------   CSV FILE READER   ---------------------------

//
// Quote a string if necessary
//
char *CCSVFileBase::StrQuote(const char *str, bool quote_always)
{
	size_t length;
	bool quote;
	const char *ptr;
	char *buf, *dptr;

	if (!str)
		return NULL;

	if (quote_always) {
		quote = false;
		for (ptr=str; *ptr!='\0'; ptr++) {
			unsigned char ch = (unsigned char)(*ptr);
			if (ch == m_FieldSep || ch == m_QuoteChar) {
				quote = true;
				break;
			}
		}
	} else {
		quote = true;
	}

	// Compute length required for 
	if (quote) {
		length = 2;
		for (ptr=str; *ptr!='\0'; ptr++) {
			unsigned char ch = (unsigned char)(*ptr);
			if (ch == m_QuoteChar) 
				length++;
			length++;
		}
	} else {
		length = strlen(str);
	}

	dptr = buf = (char*)malloc((length+1)*sizeof(char));
	if (quote) {
		for (ptr=str; *ptr!='\0'; ptr++) {
			unsigned char ch = (unsigned char)(*ptr);
			if (ch == m_QuoteChar) 
				*dptr++ = m_QuoteChar;
			*dptr++ = ch;
		}
		*dptr = '\0';
	} else {
		strcpy(buf, str);
	}
	return buf;
}

char *CCSVFileBase::StrUnquote(const char *str)
{
	int state, length;
	const char *ptr;
	char *buf, *dptr;

	if (!str)
		return NULL;

	// Compute length required for an unquoted string
	length = state = 0;
	for (ptr=str; *ptr!='\0'; ptr++) {
		unsigned char ch = (unsigned char)(*ptr);
		switch (state)
		{
		case 0:
			if (ch==m_QuoteChar) {
				state = 1;
			} else if (ch>' ') {
				state = 3;
				length = 1;
			}
			break;

		case 1:
			if (ch==m_QuoteChar)
				state = 2;
			else
				length++;
			break;

		case 2:
			if (ch==m_QuoteChar) {
				length++;
				state = 1;
			} else {
				state = 4;
			}
			break;

		case 3:
			length++;
			break;

		case 4:
			break;
		}
	}
	
	// Unquote string
	dptr = buf = (char*)malloc((length+1)*sizeof(char));
	length = state = 0;
	for (ptr=str; *ptr!='\0'; ptr++) {
		unsigned char ch = (unsigned char)(*ptr);
		switch (state)
		{
		case 0:
			if (ch==m_QuoteChar) {
				state = 1;
			} else if (ch>' ') {
				state = 3;
				*dptr++ = ch;
				length++;
			}
			break;

		case 1:
			if (ch==m_QuoteChar)
				state = 2;
			else {
				*dptr++ = ch;
				length++;
			}
			break;

		case 2:
			if (ch==m_QuoteChar) {
				*dptr++ = m_QuoteChar;
				length++;
				state = 1;
			} else {
				state = 4;
			}
			break;

		case 3:
			*dptr++ = ch;
			length++;
			break;

		case 4:
			break;
		}
	}
	// For unquoted strings, trim trailing white spaces
	if (state==3) {
		dptr--;
		while (dptr>=buf && *dptr<=' ') {
			dptr--;
			length--;
		}
		dptr++;
	}
	*dptr = '\0';
	return buf;
}

char *CCSVFileBase::Implode(const tCSVRecord *data)
{
	int i, count;
	char *ptr, *buf;

	char **list = (char**)malloc(data->count*sizeof(char*));
	count = 0;
	for (i=0; i<data->count; i++) {
		if (data->list[i]) {
			list[i] = StrQuote(data->list[i], false);
			count = i+1;
		} else {
			list[i] = NULL;
		}
	}
	size_t length = 1;
	for (i=0; i<count; i++) {
		if (length>0)
			length++;
		if (list[i])
			length += strlen(list[i]);
	}
	ptr = buf = (char*)malloc((length+1)*sizeof(char));
	for (i=0; i<count; i++) {
		if (ptr>buf) 
			*ptr++ = m_FieldSep;
		if (list[i]) {
			strcpy(ptr, list[i]);
			ptr += strlen(ptr);
			free(list[i]);
		}
	}
	*ptr = '\0';
	free(list);

	return buf;
}

bool CCSVFileBase::IsValid(const char *buf) const
{
	const char *ptr;

	for (ptr=buf; *ptr!='\0'; ptr++) {
		unsigned char ch = (unsigned char)(*ptr);
		if (ch=='#')
			return false;
		else if (ch==m_FieldSep || ch==m_QuoteChar || ch>' ')
			return true;
	}
	return false;
}

void CCSVFileBase::Explode(const char *str, tCSVRecord *data)
{
	int state, column;
	char *buf, *start, *ptr;

	data->count = 0;
	if (str) {
		buf = (char*)malloc((strlen(str)+1)*sizeof(char));
		strcpy(buf, str);
		state = column = 0;
		for (ptr=start=buf; *ptr!='\0'; ptr++) {
			unsigned char ch = (unsigned char)(*ptr);
			switch (state)
			{
			case 0:
				if (ch=='#') {
					// Comment line
					state = 9;
					break;
				} else if (ch==m_FieldSep) {
					*ptr = '\0';
					SetData(data, column++, StrUnquote(start));
					start = ptr+1;
					state = 0;
				} else if (ch==m_QuoteChar) 
					state = 1;
				else if (ch>' ')
					state = 3;
				break;

			case 1:
				if (ch==m_QuoteChar)
					state = 2;
				break;

			case 2:
				if (ch==m_FieldSep) {
					*ptr = '\0';
					SetData(data, column++, StrUnquote(start));
					start = ptr+1;
					state = 0;
				} else if (ch==m_QuoteChar) {
					state = 1;
				} else
					state = 4;
				break;

			case 3:
				if (ch==m_FieldSep) {
					*ptr = '\0';
					SetData(data, column++, StrUnquote(start));
					start = ptr+1;
					state = 0;
				}
				break;
				
			case 4:
				if (ch==m_FieldSep) {
					*ptr = '\0';
					SetData(data, column++, StrUnquote(start));
					start = ptr+1;
					state = 0;
				}
				break;
			}
		}
		// Last field
		if (state==2 || state==3) 
			SetData(data, column, StrUnquote(start));
		free(buf);
	}
}

void CCSVFileBase::SetData(CCSVFile::tCSVRecord *data, int column, char *buf)
{
	if (column>=0) {
		if (column >= data->capacity) {
			data->capacity = column+1;
			data->list = (char**)realloc(data->list, data->capacity*sizeof(char*));
		}
		if (column > data->count)
			memset(data->list+data->count, 0, (column-data->count)*sizeof(char*));
		if (column >= data->count) {
			data->list[column] = buf;
			data->count = column+1;
		} else {
			free(data->list[column]);
			data->list[column] = buf;
		}
	}
}

void CCSVFileBase::ClearData(CCSVFile::tCSVRecord *data)
{
	int i;

	for (i=0; i<data->count; i++)
		free(data->list[i]);
	free(data->list);
	data->count = data->capacity = 0;
	data->list = NULL;
}

//----------------------------   CSV FILE READER   ---------------------------

// Constructor
CCSVReader::CCSVReader(FILE *f, char field_sep, char quote_char):CCSVFileBase(field_sep, quote_char),
m_ColCount(0), m_Cols(NULL)
{
	int i;
	char buf[MAX_LINE+1];
	tCSVRecord hdr;

	m_Reader = new CFileReader(f);
	memset(&m_Rec, 0, sizeof(tCSVRecord));

	// First line is a header
	while (m_Reader->NextLine(buf, MAX_LINE+1)>=0) {
		if (IsValid(buf)) {
			memset(&hdr, 0, sizeof(tCSVRecord));
			Explode(buf, &hdr);
			if (hdr.count>0) {
				m_Cols = (char**)malloc(hdr.count*sizeof(char*));
				memset(m_Cols, 0, hdr.count*sizeof(char*));
				for (i=0; i<hdr.count; i++) 
					m_Cols[i] = hdr.list[i];
				free(hdr.list);
			}
			m_ColCount = hdr.count;
			break;
		}
	}
}

// Destructor
CCSVReader::~CCSVReader()
{
	int i;

	for (i=0; i<m_ColCount; i++)
		free(m_Cols[i]);
	free(m_Cols);

	delete m_Reader;
}

// Read next line
bool CCSVReader::ReadLine(void)
{
	char buf[MAX_LINE+1];

	ClearData(&m_Rec);
	while (m_Reader->NextLine(buf, MAX_LINE+1)>=0) {
		if (IsValid(buf)) {
			Explode(buf, &m_Rec);
			return true;
		}
	}
	return false;
}

const char *CCSVReader::ColumnName(int column) const
{
	if (column>=0 && column<m_ColCount)
		return m_Cols[column];
	return NULL;
}

const char *CCSVReader::GetStr(int column, const char *defval) const
{
	if (column>=0 && column<m_ColCount && column<m_Rec.count) 
		return m_Rec.list[column];
	return NULL;
}

bool CCSVReader::GetInt(int column, int *value) const
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		int val = strtol(str, &endptr, 10);
		if (endptr!=str) {
			*value = val;
			return true;
		}
	}
	*value = 0;
	return false;
}

int CCSVReader::GetInt(int column, int defval) const
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		int val = strtol(str, &endptr, 10);
		if (endptr!=str) 
			return val;
	}
	return defval;
}

bool CCSVReader::GetDbl(int column, double *value) const
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		double val = strtod(str, &endptr);
		if (endptr!=str) {
			*value = val;
			return true;
		}
	}
	*value = 0;
	return false;
}

double CCSVReader::GetDbl(int column, double defval) const
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		double val = strtod(str, &endptr);
		if (str!=endptr)
			return val;
	}
	return defval;
}

bool CCSVReader::GetBool(int column, bool *value) const
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		bool val = strtol(str, &endptr, 10)!=0;
		if (endptr!=str) {
			*value = val;
			return true;
		}
	}
	*value = false;
	return false;
}

bool CCSVReader::GetBool(int column, bool defval) const
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		bool val = strtol(str, &endptr, 10)!=0;
		if (str!=endptr)
			return val;
	}
	return defval;
}


// Constructor
CCSVFile::CCSVFile(void):CCSVFileBase(',', '"'), m_RecIndex(0), m_FreeID(0), 
	m_Changed(false), m_Ready(false)
{
	memset(&m_Header, 0, sizeof(tCSVHeader));
	memset(&m_Lines, 0, sizeof(tCSVLines));
	memset(&m_Rec, 0, sizeof(tCSVRecord));
}

// Destructor
CCSVFile::~CCSVFile()
{
	Clear();
}

// Clear all data and header
void CCSVFile::Clear(void)
{
	int i;

	ClearData(&m_Rec);
	m_Changed = m_Ready = false;
	m_RecIndex = 0;

	for (i=0; i<m_Header.count; i++) 
		free(m_Header.fields[i]);
	free(m_Header.fields);
	m_Header.fields = NULL;
	m_Header.count = m_Header.capacity = 0;

	for (i=0; i<m_Lines.count; i++) {
		if (m_Lines.lines[i])
			free(m_Lines.lines[i]->line);
		free(m_Lines.lines[i]);
	}
	free(m_Lines.lines);
	m_Lines.lines = NULL;
	m_Lines.count = m_Lines.capacity = 0;
}

void CCSVFile::Load(FILE *f, char field_sep, char quote_char)
{
	bool	header;
	char	buf[MAX_LINE+1];

	m_FreeID = 0;
	header = true;
	m_FieldSep = field_sep;
	m_QuoteChar = quote_char;

	CFileReader reader(f);
	while (reader.NextLine(buf, MAX_LINE+1)>=0) {
		tCSVLine *ptr = (tCSVLine*)malloc(sizeof(tCSVLine));
		memset(ptr, 0, sizeof(tCSVLine));
		ptr->line = StrDup(buf);
		ptr->id = -1;
		if (IsValid(buf)) {
			if (header) {
				// Header line
				if (ParseHeader(buf)) {
					ptr->header = true;
					header = false;
				}
			} else {
				// Data line
				ptr->id = m_FreeID++;
			}
		}
		if (m_Lines.count >= m_Lines.capacity) {
			m_Lines.capacity += ALLOC_BY;
			m_Lines.lines = (tCSVLine**)realloc(m_Lines.lines, m_Lines.capacity*sizeof(tCSVLine*));
		}
		m_Lines.lines[m_Lines.count++] = ptr;
	}
	m_RecIndex = m_Lines.count;
	m_Changed = m_Ready = false;
}

void CCSVFile::Save(FILE *f)
{
	int i;

	Flush();
	for (i=0; i<m_Lines.count; i++) {
		if (m_Lines.lines[i] && m_Lines.lines[i]->line && !m_Lines.lines[i]->deleted)
			fprintf(f, "%s\n", m_Lines.lines[i]->line);
	}
}

const char *CCSVFile::ColumnName(int column) const
{
	if (column>=0 && column<m_Header.count)
		return m_Header.fields[column];
	return NULL;
}

int CCSVFile::FindColumn(const char *name) const
{
	int i;

	if (name) {
		for (i=0; i<m_Header.count; i++) {
			if (strcmp(m_Header.fields[i], name)==0)
				return i;
		}
	}
	return -1;
}

int CCSVFile::AddColumn(const char *name)
{
	int index = -1;

	if (name) {
		size_t len = strlen(name);
		if (len>0 && FindColumn(name)<0) {
			if (m_Header.count >= m_Header.capacity) {
				m_Header.capacity += ALLOC_BY;
				m_Header.fields = (char**)realloc(m_Header.fields, m_Header.capacity*sizeof(char*));
			}
			index = m_Header.count;
			m_Header.fields[index] = (char*)malloc((len+1)*sizeof(char));
			strcpy(m_Header.fields[index], name);
			m_Header.count++;
			m_Header.changed = true;
		}
	}
	return index;
}

int CCSVFile::RecordID(void) const
{
	if (m_RecIndex>=0 && m_RecIndex<m_Lines.count) {
		if (m_Lines.lines[m_RecIndex])
			return m_Lines.lines[m_RecIndex]->id;
	}
	return -1;
}

bool CCSVFile::First(void)
{
	int i;

	Flush();
	for (i=0; i<m_Lines.count; i++) {
		if (m_Lines.lines[i]->id>=0 && !m_Lines.lines[i]->deleted) {
			m_RecIndex = i;
			m_Ready = false;
			return true;
		}
	}
	m_RecIndex = m_Lines.count;
	m_Changed = m_Ready = false;
	ClearData(&m_Rec);
	return false;
}

bool CCSVFile::Next(void)
{
	int i;

	Flush();
	for (i=m_RecIndex+1; i<m_Lines.count; i++) {
		if (m_Lines.lines[i]->id>=0 && !m_Lines.lines[i]->deleted) {
			m_RecIndex = i;
			m_Ready = false;
			return true;
		}
	}
	m_RecIndex = m_Lines.count;
	m_Changed = m_Ready = false;
	ClearData(&m_Rec);
	return false;
}

bool CCSVFile::FindRecord(int id)
{
	int i;

	Flush();
	if (id>=0) {
		for (i=0; i<m_Lines.count; i++) {
			if (m_Lines.lines[i]->id == id) {
				m_RecIndex = i;
				m_Ready = false;
				return true;
			}
		}
	}
	m_RecIndex = m_Lines.count;
	m_Ready = false;
	return false;
}

int CCSVFile::Append(void)
{
	int index, retval;

	Flush();
	ClearData(&m_Rec);

	if (m_Lines.count >= m_Lines.capacity) {
		m_Lines.capacity += ALLOC_BY;
		m_Lines.lines = (tCSVLine**)realloc(m_Lines.lines, m_Lines.capacity*sizeof(tCSVLine*));
	}
	retval = m_FreeID++;
	index = m_Lines.count;
	m_Lines.lines[index] = (tCSVLine*)malloc(sizeof(tCSVLine));
	memset(m_Lines.lines[index], 0, sizeof(tCSVLine));
	m_Lines.lines[index]->id = retval;
	m_RecIndex = index;
	m_Lines.count++;
	m_Changed = m_Ready = true;
	return retval;
}

int CCSVFile::Insert(void)
{
	int index, retval;

	if (m_RecIndex>=0 && m_RecIndex<m_Lines.count) {
		Flush();
		ClearData(&m_Rec);
		if (m_Lines.count >= m_Lines.capacity) {
			m_Lines.capacity += ALLOC_BY;
			m_Lines.lines = (tCSVLine**)realloc(m_Lines.lines, m_Lines.capacity*sizeof(tCSVLine*));
		}
		retval = m_FreeID++;
		index = m_Lines.count;
		memmove(m_Lines.lines+(index+1), m_Lines.lines+index, (m_Lines.count-index)*sizeof(tCSVLine*));
		m_Lines.lines[index] = (tCSVLine*)malloc(sizeof(tCSVLine));
		memset(m_Lines.lines[index], 0, sizeof(tCSVLine));
		m_Lines.lines[index]->id = retval;
		m_Lines.count++;
		m_Changed = m_Ready = true;
	} else {
		m_RecIndex = m_Lines.count;
		m_Ready = false;
		retval = -1;
	}
	return retval;
}

void CCSVFile::Delete(void)
{
	if (m_RecIndex>=0 && m_RecIndex<m_Lines.count) 
		m_Lines.lines[m_RecIndex]->deleted = true;
}

void CCSVFile::DeleteAll(void)
{
	int i;

	Flush();
	for (i=0; i<m_Lines.count; i++) {
		if (m_Lines.lines[i]->id>=0)
			m_Lines.lines[i]->deleted = true;
	}
	m_RecIndex = m_Lines.count;
	m_Changed = m_Ready = false;
	ClearData(&m_Rec);
}

bool CCSVFile::IsDeleted(void) const
{
	if (m_RecIndex>=0 && m_RecIndex<m_Lines.count) 
		return m_Lines.lines[m_RecIndex]->deleted;
	return false;
}

bool CCSVFile::ParseHeader(const char *buf)
{
	int i;
	tCSVRecord hdr;

	memset(&hdr, 0, sizeof(tCSVRecord));
	Explode(buf, &hdr);
	if (hdr.count>0) {
		m_Header.count = m_Header.capacity = hdr.count;
		m_Header.fields = (char**)malloc(m_Header.capacity*sizeof(char*));
		memset(m_Header.fields, 0, m_Header.capacity*sizeof(char*));
		for (i=0; i<hdr.count; i++) 
			m_Header.fields[i] = hdr.list[i];
		free(hdr.list);
		return true;
	}
	return false;
}

char *CCSVFile::MakeHeader(void)
{
	tCSVRecord hdr;

	if (m_Header.count>0) {
		memset(&hdr, 0, sizeof(tCSVRecord));
		hdr.count = m_Header.count;
		hdr.list = m_Header.fields;
		return Implode(&hdr);
	}
	return NULL;
}

void CCSVFile::Flush(void)
{
	int i, index;
	bool ok = false;

	if (m_Header.changed) {
		// Update existing header line
		for (i=0; i<m_Lines.count; i++) {
			if (m_Lines.lines[i]->header) {
				free(m_Lines.lines[i]->line);
				m_Lines.lines[i]->line = MakeHeader();
				ok = true;
				break;
			}
		}
		if (!ok) {
			// Make new header line, insert it before the first data record
			index = m_Lines.count;
			for (i=0; i<m_Lines.count; i++) {
				if (m_Lines.lines[i]->id>=0) {
					index = i;
					break;
				}
			}
			if (m_Lines.count >= m_Lines.capacity) {
				m_Lines.capacity += ALLOC_BY;
				m_Lines.lines = (tCSVLine**)realloc(m_Lines.lines, m_Lines.capacity*sizeof(tCSVLine*));
			}
			if (index<m_Lines.count) 
				memmove(m_Lines.lines+(index+1), m_Lines.lines+index, (m_Lines.count-index)*sizeof(tCSVLine*));
			m_Lines.lines[index] = (tCSVLine*)malloc(sizeof(tCSVLine));
			memset(m_Lines.lines[index], 0, sizeof(tCSVLine));
			m_Lines.lines[index]->header = true;
			m_Lines.lines[index]->id = -1;
			m_Lines.lines[index]->line = MakeHeader();
			m_Lines.count++;
			if (index<=m_RecIndex)
				m_RecIndex++;
		}
		m_Header.changed = false;
	}
	if (m_Changed && m_Ready) {
		if (m_RecIndex>=0 && m_RecIndex<m_Lines.count) {
			free(m_Lines.lines[m_RecIndex]->line);
			m_Lines.lines[m_RecIndex]->line = Implode(&m_Rec);
		}
		m_Changed = false;
	}
}

void CCSVFile::Prepare(void)
{
	if (!m_Ready) {
		ClearData(&m_Rec);
		if (m_RecIndex>=0 && m_RecIndex<m_Lines.count) {
			Explode(m_Lines.lines[m_RecIndex]->line, &m_Rec);
			m_Ready = true;
		}
		m_Changed = false;
	}
}

const char *CCSVFile::GetStr(int column)
{
	Prepare();
	if (column>=0 && column<m_Rec.count)
		return m_Rec.list[column];
	return NULL;
}

void CCSVFile::SetStr(int column, const char *data)
{
	Prepare();
	if (column>=0 && column<m_Header.count) {
		SetData(&m_Rec, column, StrDup(data));
		m_Changed = true;
	}
}

bool CCSVFile::GetInt(int column, int *value)
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		int val = strtol(str, &endptr, 10);
		if (endptr!=str) {
			*value = val;
			return true;
		}
	}
	*value = 0;
	return false;
}

void CCSVFile::SetInt(int column, int value)
{
	char buf[256];

	sprintf(buf, "%d", value);
	SetStr(column, buf);
}

int CCSVFile::GetInt(int column, int defval)
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		int val = strtol(str, &endptr, 10);
		if (endptr!=str) 
			return val;
	}
	return defval;
}

void CCSVFile::SetDbl(int column, double value, int prec)
{
	char buf[256];

	sprintf(buf, "%.*f", prec, value);
	SetStr(column, buf);
}

bool CCSVFile::GetDbl(int column, double *value)
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		double val = strtod(str, &endptr);
		if (endptr!=str) {
			*value = val;
			return true;
		}
	}
	*value = 0;
	return false;
}

double CCSVFile::GetDbl(int column, double defval)
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		double val = strtod(str, &endptr);
		if (endptr!=str) {
			return val;
		}
	}
	return defval;
}

void CCSVFile::SetBool(int column, bool value)
{
	SetStr(column, (value ? "1" : "0"));
}

bool CCSVFile::GetBool(int column, bool *value)
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		bool val = strtol(str, &endptr, 10)!=0;
		if (endptr!=str) {
			*value = val;
			return true;
		}
	}
	*value = false;
	return false;
}

bool CCSVFile::GetBool(int column, bool defval)
{
	char *endptr;
	const char *str = GetStr(column);
	if (str) {
		bool val = strtol(str, &endptr, 10)!=0;
		if (endptr!=str) 
			return val;
	}
	return defval;
}

//--------------------------   BINARY STREAM   --------------------------

struct tBinHeader
{
	int			id;
	unsigned	bytes;
};

//
// Constructor
//
CBinReader::CBinReader(FILE *f):m_PartLen(0), m_PartPos(0), m_File(f)
{
}

//
// Destructor
//
CBinReader::~CBinReader()
{
}

//
// Get next line
//
bool CBinReader::Next(int *part_id)
{
	tBinHeader header;

	if (m_File) {
		if (m_PartLen > m_PartPos) 
			fseek(m_File, m_PartLen-m_PartPos, SEEK_CUR);
		if (fread(&header, sizeof(tBinHeader), 1, m_File)==1) {
			if (part_id)
				*part_id = header.id;
			m_PartLen = header.bytes;
			m_PartPos = 0;
			return true;
		}
	}
	return false;
}

//
// Read data from file
//
int CBinReader::Read(char *buffer, unsigned count, unsigned size)
{
	size_t retval, bytes = count*size;
	if (bytes > m_PartLen-m_PartPos)
		count = (m_PartLen-m_PartPos)/size;
	retval = fread(buffer, size, count, m_File);
	if (retval>0)
		m_PartPos += retval*size;
	return retval;
}


//
// Constructor
//
CBinWriter::CBinWriter(FILE *f):m_InPart(false), m_PartHdr(0), m_PartLen(0), 
	m_File(f)
{
}

//
// Destructor
//
CBinWriter::~CBinWriter()
{
	if (m_InPart)
		EndPart();
}

//
// Start a new object
//
void CBinWriter::BeginPart(int id)
{
	tBinHeader header;

	if (m_InPart)
		EndPart();
	m_InPart = true;
	m_PartHdr = ftell(m_File);

	header.id = id;
	header.bytes = 0;
	fwrite(&header, sizeof(header), 1, m_File);
}

//
// Write data to a file
//
int CBinWriter::Write(const char *buffer, unsigned count, unsigned size)
{
	if (m_InPart && m_File) 
		return fwrite(buffer, count, size, m_File);
	return -1;
}


//
// End of part
//
void CBinWriter::EndPart(void)
{
	tBinHeader header;

	if (m_InPart) {
		size_t pos = ftell(m_File);
		fseek(m_File, m_PartHdr, SEEK_SET);
		fread(&header, sizeof(header), 1, m_File);
		header.bytes = pos - m_PartHdr - sizeof(header);
		fseek(m_File, m_PartHdr, SEEK_SET);
		fwrite(&header, sizeof(header), 1, m_File);
		fseek(m_File, pos, SEEK_SET);
	}
	m_InPart = false;
}
