/**************************************************************

file_classes.h (C-Munipack project)
Classes for various file formats
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef MUNIWIN_FILE_CLASSES_H
#define MUNIWIN_FILE_CLASSES_H

#include <stdint.h>

//
// dBase file parser
//
class CDBFReader
{
public:
	// Constructor
	CDBFReader(FILE *f);

	// Destructor
	virtual ~CDBFReader();

	// Get number of header columns
	int Columns(void) const
	{ return m_ColCount; }

	// Read next record
	bool Next();

	// Get field name
	const char *FieldName(int col) const;

	// Get field data
	bool GetField(int index, char *buf, size_t buflen);

private:
	// File header
	struct tDBFHeader
	{
		uint8_t		version, year, month, day;
		uint32_t	count;
		uint16_t	head_len, rec_length;
		int8_t		reserved[20];
	};

	// Field descriptor
	struct tDBFField
	{
		char		name[16];
		int16_t		start, length;
	};

	FILE		*m_File;
	tDBFField	*m_Cols;
	char		*m_Record;
	int			m_ColCount;
	tDBFHeader	m_Header;
};

//
// Text file parser (fixed positions)
//
class CTextReader
{
public:
	// Constructor
	CTextReader(FILE *f);

	// Destructor
	virtual ~CTextReader();

	// Get next line
	bool ReadLine();

	// Get field value
	bool GetField(int start, int length, char *buf, int buflen) const;

	// Get line length
	size_t Length(void) const
	{ return m_LineLength; }

	// Get character at specified position
	char Col(int index) const
	{ return m_Line[index]; }

private:
	class CFileReader *m_Reader;
	char *m_Line;
	size_t m_LineLength;
};

//
// CSV File base class (RFC 4180 compliant)
//
class CCSVFileBase
{
public:
	// Constructor
	CCSVFileBase(char field_sep, char quote_char):m_FieldSep(field_sep), m_QuoteChar(quote_char) {}

protected:
	char m_FieldSep, m_QuoteChar;

	struct tCSVRecord {
		int count, capacity;
		char **list;
	};

	// Returns true if the buffer contains valid header or data line
	bool IsValid(const char *buf) const;

	// Split line in buffer into fields. The separator characters
	// are replaced by nul terminators. Returns number of fields found.
	void Explode(const char *buf, tCSVRecord *data);

	// Make record line in the m_Line buffer
	char *Implode(const tCSVRecord *data);

	// Set value in data record
	void SetData(tCSVRecord *data, int column, char *buf);

	// Free allocated memory
	void ClearData(tCSVRecord *data);

	// Quote a string if necessary (it contains a quote character or a field separator)
	// Quotes inside a quoted string are replaced by two subsequent quotes
	char *StrQuote(const char *str, bool quote_always);

	// If the string is quoted (first non-ws character is quote), remove quotes
	// Two subsequent quotes in a quoted string are replaced by a single quote
	char *StrUnquote(const char *str);
};

//
// CSV file reader
//
class CCSVReader:public CCSVFileBase
{
public:
	// Constructor
	CCSVReader(FILE *f, char field_sep = ',', char quote_char = '"');

	// Destructor
	virtual ~CCSVReader();

	// Get next line
	bool ReadLine(void);

	// Get number of header columns
	int Columns(void) const
	{ return m_ColCount; }

	// Get column name
	const char *ColumnName(int column) const;

	// Get field value
	const char *GetStr(int column, const char *defval = NULL) const;
	bool GetInt(int column, int *value) const;
	int GetInt(int column, int defval = 0) const;
	bool GetDbl(int column, double *value) const;
	double GetDbl(int column, double defval = 0.0) const;
	bool GetBool(int column, bool *value) const;
	bool GetBool(int column, bool defval = false) const;

private:
	class CFileReader *m_Reader;
	int			m_ColCount;
	char		**m_Cols;
	tCSVRecord	m_Rec;
};

class CCSVFile:public CCSVFileBase
{
public:
	// Constructor
	CCSVFile(void);

	// Destructor
	~CCSVFile();

	// Clear all data and header
	void Clear(void);

	// Load table from a file
	void Load(FILE *f, char field_sep = ',', char quote_char = '"');

	// Save table to a file
	void Save(FILE *f);

	// Get number of header columns
	int Columns(void) const
	{ return m_Header.count; }

	// Get column name
	const char *ColumnName(int column) const;

	// Find column by name
	int FindColumn(const char *name) const;

	// Append a new column
	int AddColumn(const char *name);

	// Returns true if the pointer is beyond the last record
	bool Eof(void) const
	{ return (m_RecIndex==m_Lines.count); }

	// Get number of records
	int Records(void) const
	{ return m_Lines.count; }

	// Returns index of current record
	int RecordID(void) const;

	// Record browsing
	bool First(void);
	bool Next(void);

	// Find record by its id and make it active
	bool FindRecord(int id);

	// Add a new record at the end of the file
	// Returns record identifier or negative number on failure
	int Append(void);

	// Add a new record before the active one
	// Returns record identifier or negative number on failure
	int Insert(void);

	// Mark active record as deleted
	void Delete(void);

	// Mark all records as deleted
	void DeleteAll(void);

	// Is active record marked as deleted?
	bool IsDeleted(void) const;

	// Get field value
	const char *GetStr(int column);
	bool GetInt(int column, int *value);
	int GetInt(int column, int defval = 0);
	bool GetDbl(int column, double *value);
	double GetDbl(int column, double defval = 0.0);
	bool GetBool(int column, bool *value);
	bool GetBool(int column, bool defval = false);

	// Set field value
	void SetStr(int column, const char *value);
	void SetInt(int column, int value);
	void SetDbl(int column, double value, int prec = 5);
	void SetBool(int column, bool value);

private:
	struct tCSVHeader {
		int count, capacity;
		char **fields;
		bool changed;
	};

	struct tCSVLine {
		int id;
		bool header, deleted;
		char *line;
	};

	struct tCSVLines {
		int count, capacity;
		tCSVLine **lines;
	};

	tCSVHeader	m_Header;
	tCSVLines	m_Lines;
	int			m_RecIndex, m_FreeID;
	bool		m_Changed, m_Ready;
	tCSVRecord	m_Rec;

	// Parse given buffer and fill the m_Header structure
	bool ParseHeader(const char *buf);

	// Make a header line from the m_Header structure
	char *MakeHeader();

	// Parse actual line into m_Rec
	void Prepare();

	// Save changes from m_Rec to actual line
	void Flush();
};

//
// Multipart binary data file
//
class CBinReader
{
public:
	// Constructor
	CBinReader(FILE *f);

	// Destructor
	virtual ~CBinReader();

	// Get next line
	bool Next(int *id);

	// Get length of actual object in bytes
	unsigned Length(void) const
	{ return m_PartLen; }

	// Read data from file
	// params:
	//	count				- [in] number of items to read
	//	size				- [in] size of item in bytes
	// returns positive number of items that were read into buffer
	// or negative error code on failure.
	int Read(char *buffer, unsigned count, unsigned size);

private:
	unsigned	m_PartLen, m_PartPos;
	FILE		*m_File;
};

class CBinWriter
{
public:
	// Constructor
	CBinWriter(FILE *f);

	// Destructor
	virtual ~CBinWriter();

	// Start a new object
	void BeginPart(int id);

	// Finish current object
	void EndPart(void);

	// Write data to a file
	// params:
	//	count				- [in] number of items to read
	//	size				- [in] size of item in bytes
	// returns positive number of items that were read into buffer
	// or negative error code on failure.
	int Write(const char *buffer, unsigned count, unsigned size);

private:
	bool		m_InPart;
	unsigned	m_PartHdr, m_PartLen;
	FILE		*m_File;
};

//
// XML parser
//
class CXMLParser
{
public:
	// Callback prototypes
	typedef void StartElementHandler(const char *element, const char **attr, void *user_data);
	typedef void EndElementHandler(const char *element, void *user_data);
	typedef void CharacterDataHandler(const char *data, int len, void *user_data);

public:
	// Constructor
	CXMLParser(FILE *f);

	// Destructor
	virtual ~CXMLParser();

	// Parse
	void Parse(StartElementHandler *start_proc, EndElementHandler *end_proc,
		CharacterDataHandler *cdata_proc, void *user_data);

private:
	StartElementHandler		*m_StartCB;
	EndElementHandler		*m_EndCB;
	CharacterDataHandler	*m_CDataCB;
	void	*m_UserData;
	FILE	*m_File;
};

#endif
