/**************************************************************

lightcurve_dlg.cpp (C-Munipack project)
The 'Light curve' window
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "project.h"
#include "configuration.h"
#include "progress_dlg.h"
#include "lightcurve_dlg.h"
#include "makelightcurve_dlg.h"
#include "choosestars_dlg.h"
#include "chooseaperture_dlg.h"
#include "main.h"
#include "utils.h"
#include "proc_classes.h"
#include "ctxhelp.h"
#include "chart_dlg.h"
#include "frameinfo_dlg.h"
#include "object_dlg.h"
#include "observer_dlg.h"
#include "chartexport_dlg.h"

//-------------------------   LOCAL MENU   --------------------------------

enum tMenuId
{
	MENU_FILE = 1,
	MENU_EDIT,
	MENU_VIEW,
	MENU_TOOLS,
	MENU_HELP
};

enum tCommandId
{
	CMD_SAVE = 100,
	CMD_EXPORT,
	CMD_REBUILD,
	CMD_PRINT,
	CMD_CLOSE,
	CMD_PROPERTIES,
	CMD_STARS,
	CMD_APERTURE,
	CMD_ERRORS,
	CMD_GRID,
	CMD_CHART,
	CMD_HELP,
	CMD_HIDE,
	CMD_DELETE,
	CMD_PREVIEW,
	CMD_FRAMEINFO,
	CMD_STATISTICS,
	CMD_MEASUREMENT
};

static const CMenuBar::tMenuItem FileMenu[] = {
	{ CMenuBar::MB_ITEM,		CMD_SAVE,		"_Save" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,		CMD_EXPORT,		"_Export as image" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,		CMD_REBUILD,	"_Rebuild" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,		CMD_CLOSE,		"_Close" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem EditMenu[] = {
	{ CMenuBar::MB_ITEM,		CMD_PROPERTIES,	"Edit _properties" },
	{ CMenuBar::MB_ITEM,		CMD_STARS,		"Change selected _stars" },
	{ CMenuBar::MB_ITEM,		CMD_APERTURE,	"Change _aperture" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ViewMenu[] = {
	{ CMenuBar::MB_CHECKBTN,	CMD_ERRORS,		"_Error bars" },
	{ CMenuBar::MB_CHECKBTN,	CMD_GRID,		"_Grid" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,		CMD_CHART,		"_Chart" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ToolsMenu[] = {
	{ CMenuBar::MB_CHECKBTN,	CMD_STATISTICS,	"_Statistics" },
	{ CMenuBar::MB_CHECKBTN,	CMD_MEASUREMENT,	"_Measurement" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem HelpMenu[] = {
	{ CMenuBar::MB_ITEM,		CMD_HELP,		"_Show help" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenu MainMenu[] = {
	{ "_Files", MENU_FILE,	FileMenu },
	{ "_Edit",	MENU_EDIT,	EditMenu },
	{ "_View",	MENU_VIEW,	ViewMenu },
	{ "_Tools", MENU_TOOLS,	ToolsMenu },
	{ "_Help",	MENU_HELP,	HelpMenu, CMenuBar::MBF_RIGHT },
	{ NULL }
};

static const CPopupMenu::tPopupMenuItem GraphContextMenu[] = {
	{ CPopupMenu::MB_ITEM, CMD_PREVIEW,		"_Show frame" },
	{ CPopupMenu::MB_ITEM, CMD_FRAMEINFO,	"_Show properties" },
	{ CPopupMenu::MB_SEPARATOR },
	{ CPopupMenu::MB_ITEM, CMD_HIDE,		"_Delete from data set" },
	{ CPopupMenu::MB_ITEM, CMD_DELETE,		"_Remove from project" },
	{ CPopupMenu::MB_END }
};

//-------------------------   FILE FORMATS   ------------------------------------

static const struct {
	const gchar *Id, *Caption, *Extension;
	gboolean DiffMag, InstMag;
} FileFormats[] = {
	{ "C-Munipack/1.0", "C-Munipack (default)", "txt", true, true },
	{ "AVE", "AVE compatible", "djm", true, false },
	{ "MCV", "MCV compatible", "dat", false, true },
	{ NULL }
};

static const struct {
	const gchar *Id, *Caption;
} DateFormats[] = {
	{ "JDGEO", "Julian date geocentric" },
	{ "JDHEL", "Julian date heliocentric" },
	{ NULL }
};

static const gchar *FileExtension(gint type)
{
	if (type>=0)
		return FileFormats[type].Extension;
	return "";
}

static void SelectItem(GtkComboBox *cb, gint id)
{
	gboolean ok;
	gint t;
	GtkTreeIter iter;

	GtkTreeModel *model = gtk_combo_box_get_model(cb);
	ok = gtk_tree_model_get_iter_first(model, &iter);
	while (ok) {
		gtk_tree_model_get(model, &iter, 0, &t, -1);
		if (t == id) {
			gtk_combo_box_set_active_iter(cb, &iter);
			return;
		}	
		ok = gtk_tree_model_iter_next(model, &iter);
	}
	gtk_combo_box_set_active(cb, -1);
}

//------------------------  TOOLBAR HELPERS   ---------------------------------

static GtkToolItem *toolbar_new_button_from_stock(GtkWidget *toolbar, const gchar *stock_id,
	const gchar *tooltip)
{
	GtkToolItem *tbtn = gtk_tool_button_new_from_stock(stock_id);
	gtk_widget_set_tooltip_text(GTK_WIDGET(tbtn), tooltip);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkToolItem *toolbar_new_widget(GtkWidget *toolbar, GtkWidget *widget)
{
	GtkToolItem *item = gtk_tool_item_new();
	gtk_container_add(GTK_CONTAINER(item), widget);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
	return item;
}

static GtkWidget *toolbar_new_combo(GtkWidget *toolbar, const gchar *tooltip)
{
	GtkWidget *entry = gtk_combo_box_new();
	gtk_widget_set_tooltip_text(entry, tooltip);
	gtk_widget_set_size_request(entry, 120, -1);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

static GtkWidget *toolbar_new_label(GtkWidget *toolbar, const char *caption)
{
	GtkWidget *entry = gtk_label_new(caption);
	gtk_misc_set_padding(GTK_MISC(entry), 8, 0);
	gtk_misc_set_alignment(GTK_MISC(entry), 0, 0.5);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

//-------------------------   CHOOSE STARS DIALOG   --------------------------------

CLightCurveDlg::CLightCurveDlg(void):m_UpdatePos(true), m_LastPosValid(false), 
	m_ApertureIndex(-1), m_ChannelX(-1), m_ChannelY(-1), m_LastPosX(-1), 
	m_LastPosY(-1), m_LastFocus(-1), m_TimerId(0), m_InfoMode(INFO_NONE)
{
	gchar buf[512];
	GtkWidget *scrwnd, *tbox;
	GtkCellRenderer *renderer;

	m_ShowErrors = CConfig::GetBool("LightCurveDlg", "Errors", true);
	m_ShowGrid   = CConfig::GetBool("LightCurveDlg", "Grid", false);

	// Dialog caption
	sprintf(buf, "%s - %s", "Light curve", g_AppTitle);
	gtk_window_set_title(GTK_WINDOW(m_pDlg), buf);

	// Menu bar
	m_Menu.Create(MainMenu);
	m_Menu.RegisterCallback(MenuCallback, this);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_Menu.Handle(), FALSE, FALSE, 0);

	// Toolbar
	tbox = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(tbox), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(tbox), GTK_ORIENTATION_HORIZONTAL);
	gtk_box_pack_start(GTK_BOX(m_MainBox), tbox, false, false, 0);

	m_XLabel = toolbar_new_label(tbox, "X axis");
	m_XChannels = gtk_list_store_new(2, GTK_TYPE_INT, GTK_TYPE_STRING);
	m_XCombo = toolbar_new_combo(tbox, "Column shown on the horizontal axis of the graph");
	g_signal_connect(G_OBJECT(m_XCombo), "changed", G_CALLBACK(changed), this);
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(m_XCombo), renderer, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(m_XCombo), renderer, "text", 1);

	m_YLabel = toolbar_new_label(tbox, "Y axis");
	m_YChannels = gtk_list_store_new(2, GTK_TYPE_INT, GTK_TYPE_STRING);
	m_YCombo = toolbar_new_combo(tbox, "Column shown on the vertical axis of the graph");
	g_signal_connect(G_OBJECT(m_YCombo), "changed", G_CALLBACK(changed), this);
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(m_YCombo), renderer, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(m_YCombo), renderer, "text", 1);

	m_ALabel = toolbar_new_label(tbox, "Aperture");
	m_Apertures = gtk_list_store_new(2, GTK_TYPE_INT, GTK_TYPE_STRING);
	m_ACombo = toolbar_new_combo(tbox, "Aperture used to get brightness of the objects");
	g_signal_connect(G_OBJECT(m_ACombo), "changed", G_CALLBACK(changed), this);
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(m_ACombo), renderer, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(m_ACombo), renderer, "text", 1);

	toolbar_new_label(tbox, "Zoom");
	m_ZoomFit = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_FIT, "Zoom to all points");
	g_signal_connect(G_OBJECT(m_ZoomFit), "clicked", G_CALLBACK(button_clicked), this);
	m_ZoomOut = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_OUT, "Zoom out");
	g_signal_connect(G_OBJECT(m_ZoomOut), "clicked", G_CALLBACK(button_clicked), this);
	m_ZoomIn = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_IN, "Zoom in");
	g_signal_connect(G_OBJECT(m_ZoomIn), "clicked", G_CALLBACK(button_clicked), this);

	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(m_MainBox), hbox, TRUE, TRUE, 0);

	// Graph
	m_GraphData = cmpack_graph_data_new();
	m_Graph = cmpack_graph_view_new_with_model(m_GraphData);
	cmpack_graph_view_set_mouse_control(CMPACK_GRAPH_VIEW(m_Graph), TRUE);
	cmpack_graph_view_set_selection_mode(CMPACK_GRAPH_VIEW(m_Graph), GTK_SELECTION_MULTIPLE);
	cmpack_graph_view_set_scales(CMPACK_GRAPH_VIEW(m_Graph), TRUE, TRUE);
	cmpack_graph_view_set_grid(CMPACK_GRAPH_VIEW(m_Graph), TRUE, TRUE);
	g_signal_connect(G_OBJECT(m_Graph), "mouse-moved", G_CALLBACK(mouse_moved), this);
	g_signal_connect(G_OBJECT(m_Graph), "button_press_event", G_CALLBACK(button_press_event), this);
	g_signal_connect(G_OBJECT(m_Graph), "selection-changed", G_CALLBACK(selection_changed), this);
	gtk_widget_set_size_request(m_Graph, 300, 200);
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_Graph);
	gtk_box_pack_start(GTK_BOX(hbox), scrwnd, TRUE, TRUE, 0);

	// Statistics tool
	m_InfoBox.SetCaption("Statistics");
	m_InfoBox.RegisterCallback(InfoBoxCallback, this);
	gtk_box_pack_start(GTK_BOX(hbox), m_InfoBox.Handle(), FALSE, FALSE, 0);

	// Measurement tool
	m_MeasBox.RegisterCallback(InfoBoxCallback, this);
	gtk_box_pack_start(GTK_BOX(hbox), m_MeasBox.Handle(), FALSE, FALSE, 0);
	m_MeasBox.SetGraphView(CMPACK_GRAPH_VIEW(m_Graph));

	// Timers
	m_TimerId = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 100, GSourceFunc(update_status_timer), this, NULL);

	// Popup menu
	m_GraphMenu.Create(GraphContextMenu);

	// Show the dialog
	gtk_widget_show_all(m_MainBox);
	m_InfoBox.Show(false);
	m_MeasBox.Show(false);
}

CLightCurveDlg::~CLightCurveDlg()
{
	// Disconnect graph signals
	g_signal_handlers_disconnect_by_func(G_OBJECT(m_Graph), (gpointer)mouse_moved, this);
	g_signal_handlers_disconnect_by_func(G_OBJECT(m_Graph), (gpointer)button_press_event, this);

	// Release objects, free allocated memory
	g_source_remove(m_TimerId);
	g_object_unref(m_XChannels);
	g_object_unref(m_YChannels);
	g_object_unref(m_Apertures);
	g_object_unref(m_GraphData);
}

bool CLightCurveDlg::Make(GtkWindow *parent, bool selected_files, const tParamsRec &rec, 
	const CObjectCoords &obj, const CLocation &loc)
{
	m_MeasBox.SetTable(NULL);
	m_Table.Clear();
	m_FrameSet.Clear();
	m_AperList.Clear();
	m_ChannelX = m_ChannelY = -1;
	m_DataSaved = false;
	m_Params = rec;
	m_ObjCoords = obj;
	m_Location = loc;

	// Create list of files
	if (!InitFileList(parent, selected_files))
		return false;
	
	// Apertures
	m_AperList = *g_Project->RefApertures();
	if (m_AperList.Count()==0) {
		ShowError(parent, "No aperture available.\nCheck the configuration and execute the photometry and the matching.");
		return false;
	}

	// Choose stars
	if (g_Project->Selection()->Count()>0) {
		m_Selection = *g_Project->Selection();
		m_Tags = *g_Project->Tags();
	} else if (g_Project->RefSelection()->Count()>0) {
		m_Selection = *g_Project->RefSelection();
		m_Tags = *g_Project->RefTags();
	} else {
		m_Selection.Clear();
		m_Tags.Clear();
	}
	CChooseStarsDlg dlg(parent);
	if (!dlg.Execute(m_Selection, m_Tags, m_Params.instmag))
		return false;
	g_Project->SetSelection(m_Selection);
	g_Project->SetTags(m_Tags);
	RebuildData(parent);

	// Choose aperture
	m_ApertureIndex = m_AperList.Find(g_Project->GetInt("LightCurve", "Aperture", 0));
	if (m_ApertureIndex<0)
		m_ApertureIndex = (m_AperList.Count()>0 ? 0 : -1);
	if (m_Selection.CountStars(CMPACK_SELECT_CHECK)>0 && m_Selection.CountStars(CMPACK_SELECT_COMP)>0) {
		CChooseApertureDlg dlg(parent);
		if (!dlg.Execute(m_FrameSet, m_AperList, m_Selection, &m_ApertureIndex))
			return false;
	} else {
		CChooseApertureSimpleDlg dlg(parent);
		if (!dlg.Execute(m_FrameSet, m_AperList, m_Selection, &m_ApertureIndex))
			return false;
	}
	g_Project->SetInt("LightCurve", "Aperture", m_AperList.GetId(m_ApertureIndex));

	UpdateApertures();
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_ACombo), m_ApertureIndex);
	UpdateLightCurve();
	UpdateChannels();
	UpdateGraph(TRUE, TRUE);
	SetInfoMode(INFO_NONE);
	SetStatus(NULL);
	UpdateControls();
	return true;
}

void CLightCurveDlg::OnFrameSetChanged(void)
{
	UpdateLightCurve();
	UpdateGraph(TRUE, TRUE);
	UpdateTools();
	UpdateStatus();
}

void CLightCurveDlg::UpdateControls(void)
{
	m_Menu.Check(CMD_ERRORS, m_ShowErrors);
	m_Menu.Check(CMD_GRID, m_ShowGrid);
	m_Menu.Check(CMD_STATISTICS, m_InfoMode == INFO_STATISTICS);
	m_Menu.Check(CMD_MEASUREMENT, m_InfoMode == INFO_MEASUREMENT);
}

void CLightCurveDlg::UpdateLightCurve(void)
{
	int res = 0;

	m_MeasBox.SetTable(NULL);

	if (m_Params.amass) 
		res = CmpackAirMass(NULL, m_FrameSet, m_ObjCoords, m_Location);

	if (res==0 && m_Params.helcor)
		res = CmpackHelCorr(NULL, m_FrameSet, m_ObjCoords);

	if (res==0) {
		res = CmpackLightCurve(NULL, m_Table, m_Selection, m_FrameSet, 
			m_AperList.GetId(m_ApertureIndex), (CmpackLCurveFlags)(
			CMPACK_LCURVE_FRAME_IDS |
			(m_Params.helcor ? CMPACK_LCURVE_HELCORR | CMPACK_LCURVE_HJD : 0) |
			(m_Params.amass ? CMPACK_LCURVE_AIRMASS | CMPACK_LCURVE_ALTITUDE : 0) |
			(m_Params.instmag ? CMPACK_LCURVE_INSTMAG : 0)));
	}

	if (res==0)
		m_MeasBox.SetTable(&m_Table);
}

void CLightCurveDlg::UpdateGraph(gboolean autozoom_x, gboolean autozoom_y)
{
	cmpack_graph_view_set_model(CMPACK_GRAPH_VIEW(m_Graph), NULL);
	cmpack_graph_data_clear(m_GraphData);
	m_Table.Render(m_GraphData, m_ChannelX, m_ChannelY, !m_ShowErrors);
	m_Table.SetView(CMPACK_GRAPH_VIEW(m_Graph), m_ChannelX, m_ChannelY, !m_ShowErrors);
	cmpack_graph_view_set_grid(CMPACK_GRAPH_VIEW(m_Graph), m_ShowGrid, m_ShowGrid);
	cmpack_graph_view_set_model(CMPACK_GRAPH_VIEW(m_Graph), m_GraphData);
	cmpack_graph_view_reset_zoom(CMPACK_GRAPH_VIEW(m_Graph), autozoom_x, autozoom_y);
}

void CLightCurveDlg::UpdateChannels(void)
{
	int i;
	GtkTreeIter iter;
	CChannels *cx = m_Table.ChannelsX(), *cy = m_Table.ChannelsY();

	// Update list of independent channels
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_XCombo), NULL);
	gtk_list_store_clear(m_XChannels);
	for (i=0; i<cx->Count(); i++) {
		gtk_list_store_append(m_XChannels, &iter);
		gtk_list_store_set(m_XChannels, &iter, 0, i, 1, cx->GetName(i), -1);
	}
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_XCombo), GTK_TREE_MODEL(m_XChannels));
	if (m_ChannelX<0 || m_ChannelX>=cx->Count()) {
		if (cx->Count()>=2 && m_Params.helcor)
			m_ChannelX = 1;
		else 
			m_ChannelX = (cx->Count()>0 ? 0 : -1);
	}
	if (m_ChannelX>=0) {
		m_MeasBox.SetChannel(CMPACK_AXIS_X, m_ChannelX);
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_XCombo), m_ChannelX);
	}
	gtk_widget_set_sensitive(m_XCombo, cx->Count()>1);
	gtk_widget_set_sensitive(m_XLabel, cx->Count()>1);

	// Update list of independent channels
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_YCombo), NULL);
	gtk_list_store_clear(m_YChannels);
	for (i=0; i<cy->Count(); i++) {
		gtk_list_store_append(m_YChannels, &iter);
		gtk_list_store_set(m_YChannels, &iter, 0, i, 1, cy->GetName(i), -1);
	}
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_YCombo), GTK_TREE_MODEL(m_YChannels));
	if (m_ChannelY<0 || m_ChannelY>=cy->Count()) 
		m_ChannelY = (cy->Count()>0 ? 0 : -1);
	if (m_ChannelY>=0) {
		m_MeasBox.SetChannel(CMPACK_AXIS_Y, m_ChannelY);
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_YCombo), m_ChannelY);
	}
	gtk_widget_set_sensitive(m_YCombo, cy->Count()>1);
	gtk_widget_set_sensitive(m_YLabel, cy->Count()>1);
}

void CLightCurveDlg::UpdateApertures(void)
{
	int i;
	char txt[256];
	GtkTreeIter iter;

	// Update list of independent channels
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_ACombo), NULL);
	gtk_list_store_clear(m_Apertures);
	for (i=0; i<m_AperList.Count(); i++) {
		const CAperture *aper = m_AperList.Get(i);
		sprintf(txt, "#%d (%.2f)", aper->Id(), aper->Radius());
		gtk_list_store_append(m_Apertures, &iter);
		gtk_list_store_set(m_Apertures, &iter, 0, aper->Id(), 1, txt, -1);
	}
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_ACombo), GTK_TREE_MODEL(m_Apertures));
	if (m_ApertureIndex<0 || m_ApertureIndex>=m_AperList.Count()) 
		m_ApertureIndex = (m_AperList.Count()>0 ? 0 : -1);
	if (m_ApertureIndex>=0) 
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_ACombo), m_ApertureIndex);
	gtk_widget_set_sensitive(m_ACombo, m_AperList.Count()>1);
	gtk_widget_set_sensitive(m_ALabel, m_AperList.Count()>1);
}

void CLightCurveDlg::MenuCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data)
{
	CLightCurveDlg *pMe = (CLightCurveDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		// Menu bar command
		pMe->OnCommand(wparam);
		break;
	}
}

void CLightCurveDlg::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	// File menu
	case CMD_CLOSE:
		Close();
		break;
	case CMD_SAVE:
		SaveLightCurve();
		break;
	case CMD_EXPORT:
		ExportAsImage();
		break;
	case CMD_REBUILD:
		RebuildData(GTK_WINDOW(m_pDlg));
		break;

	// Edit menu
	case CMD_PROPERTIES:
		EditProperties();
		break;
	case CMD_STARS:
		EditSelection();
		break;
	case CMD_APERTURE:
		ChangeAperture();
		break;

	// View menu
	case CMD_ERRORS:
		m_ShowErrors = m_Menu.IsChecked(CMD_ERRORS);
		UpdateGraph(FALSE, FALSE);
		CConfig::SetBool("LightCurveDlg", "Errors", m_ShowErrors);
		break;
	case CMD_GRID:
		m_ShowGrid = m_Menu.IsChecked(CMD_GRID);
		UpdateGraph(FALSE, FALSE);
		CConfig::SetBool("LightCurveDlg", "Grid", m_ShowGrid);
		break;
	case CMD_CHART:
		ShowChart();
		break;

	// Tools menu
	case CMD_STATISTICS:
		if (m_Menu.IsChecked(CMD_STATISTICS)) 
			SetInfoMode(INFO_STATISTICS);
		else if (m_InfoMode == INFO_STATISTICS)
			SetInfoMode(INFO_NONE);
		break;
	case CMD_MEASUREMENT:
		if (m_Menu.IsChecked(CMD_MEASUREMENT)) 
			SetInfoMode(INFO_MEASUREMENT);
		else if (m_InfoMode == INFO_MEASUREMENT)
			SetInfoMode(INFO_NONE);
		break;

	// Help menu
	case CMD_HELP:
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_LIGHT_CURVE);
		break;
	}
}

void CLightCurveDlg::changed(GtkWidget *button, CLightCurveDlg *pDlg)
{
	pDlg->OnChanged(button);
}

void CLightCurveDlg::OnChanged(GtkWidget *pBtn)
{
	if (pBtn==m_XCombo) {
		int ch = gtk_combo_box_get_active(GTK_COMBO_BOX(m_XCombo));
		if (ch!=m_ChannelX) {
			m_ChannelX = ch;
			UpdateGraph(TRUE, FALSE);
			UpdateTools();
		}
	} else if (pBtn==m_YCombo) {
		int ch = gtk_combo_box_get_active(GTK_COMBO_BOX(m_YCombo));
		if (ch!=m_ChannelY) {
			m_ChannelY = ch;
			UpdateGraph(FALSE, TRUE);
			UpdateTools();
		}
	} else if (pBtn==m_ACombo) {
		int index = gtk_combo_box_get_active(GTK_COMBO_BOX(m_ACombo));
		if (index!=m_ApertureIndex) {
			m_ApertureIndex = index;
			g_Project->SetInt("LightCurve", "Aperture", m_AperList.GetId(m_ApertureIndex));
			m_DataSaved = false;
			UpdateLightCurve();
			UpdateGraph(FALSE, FALSE);
			UpdateTools();
		}
	}
}

void CLightCurveDlg::button_clicked(GtkWidget *button, CLightCurveDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CLightCurveDlg::OnButtonClicked(GtkWidget *pBtn)
{
	double zoom;

	if (pBtn==GTK_WIDGET(m_ZoomIn)) {
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X, zoom + 5.0);
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y, zoom + 5.0);
	} else 
	if (pBtn==GTK_WIDGET(m_ZoomOut)) {
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X, zoom - 5.0);
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y, zoom - 5.0);
	} else 
	if (pBtn==GTK_WIDGET(m_ZoomFit)) {
		cmpack_graph_view_reset_zoom(CMPACK_GRAPH_VIEW(m_Graph), TRUE, TRUE);
	}
}

void CLightCurveDlg::mouse_moved(GtkWidget *button, CLightCurveDlg *pDlg)
{
	pDlg->m_UpdatePos = true;
}

//
// Mouse button handler
//
gint CLightCurveDlg::button_press_event(GtkWidget *widget, GdkEventButton *event, CLightCurveDlg *pMe)
{
	int focused;

	if (event->type==GDK_BUTTON_PRESS && event->button==3) {
		gtk_widget_grab_focus(widget);
		if (widget==pMe->m_Graph) {
			focused = cmpack_graph_view_get_focused(CMPACK_GRAPH_VIEW(widget));
			if (focused>=0 && !cmpack_graph_view_is_selected(CMPACK_GRAPH_VIEW(widget), focused)) {
				cmpack_graph_view_unselect_all(CMPACK_GRAPH_VIEW(widget));
				cmpack_graph_view_select(CMPACK_GRAPH_VIEW(widget), focused);
			}
		}
		pMe->OnContextMenu(widget, event);
		return TRUE;
	}
	return FALSE;
}

void CLightCurveDlg::selection_changed(GtkWidget *pChart, CLightCurveDlg *pDlg)
{
	pDlg->OnSelectionChanged();
}

//
// Left button click
//
void CLightCurveDlg::OnSelectionChanged(void)
{
	UpdateTools();
}

//
// Show context menu
//
void CLightCurveDlg::OnContextMenu(GtkWidget *widget, GdkEventButton *event)
{
	int res, selected;

	if (widget==m_Graph) {
		selected = cmpack_graph_view_get_selected_count(CMPACK_GRAPH_VIEW(m_Graph));
		m_GraphMenu.Enable(CMD_DELETE, selected>0);
		m_GraphMenu.Enable(CMD_HIDE, selected>0);
		m_GraphMenu.Enable(CMD_FRAMEINFO, selected==1);
		m_GraphMenu.Enable(CMD_PREVIEW, selected==1);
		res = m_GraphMenu.Execute(event);
		switch (res) 
		{
		case CMD_PREVIEW:
			// Show frame preview
			ShowFramePreview();
			break;
		case CMD_FRAMEINFO:
			// Show frame properties
			ShowFrameInfo();
			break;
		case CMD_HIDE:
			// Remove selected points from data set
			RemoveFromDataSet();
			break;
		case CMD_DELETE:
			// Delete selected points from project
			DeleteFromProject();
			break;
		}
	}
}

// 100 ms timer
gboolean CLightCurveDlg::update_status_timer(CLightCurveDlg *pDlg)
{
	gdk_threads_lock();
	if (pDlg->m_UpdatePos) {
		pDlg->m_UpdatePos = false;
		pDlg->UpdateStatus();
	}
	gdk_threads_unlock();
	return TRUE;
}

void CLightCurveDlg::UpdateStatus(void)
{
	char	msg[256], buf1[256], buf2[256];
	gdouble	dx, dy, sdev;

	int item = cmpack_graph_view_get_focused(CMPACK_GRAPH_VIEW(m_Graph));
	if (item>=0) {
		if (m_LastFocus!=item) {
			m_LastFocus = item;
			int frame_id = (int)cmpack_graph_data_get_param(m_GraphData, item),
				xcol = m_Table.ChannelsX()->GetColumn(m_ChannelX),
				ycol = m_Table.ChannelsY()->GetColumn(m_ChannelY),
				ucol = m_Table.ChannelsY()->GetColumnU(m_ChannelY);
			if (m_Table.Find(frame_id) && m_Table.GetDbl(xcol, &dx) && m_Table.GetDbl(ycol, &dy)) {
				PrintKeyValue(buf1, dx, m_Table.ChannelsX()->Get(m_ChannelX));
				if (ucol>=0 && m_Table.GetDbl(ucol, &sdev))
					PrintKeyValueU(buf2, dy, sdev, m_Table.ChannelsY()->Get(m_ChannelY));
				else 
					PrintKeyValue(buf2, dy, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "Frame #%d: %s, %s", frame_id, buf1, buf2);
			} else {
				sprintf(msg, "Frame #%d", frame_id);
			}
			SetStatus(msg);
		}
	} else {
		m_LastFocus = -1;
		if (m_ChannelX>=0 && m_ChannelY>=0 && cmpack_graph_view_mouse_pos(CMPACK_GRAPH_VIEW(m_Graph), &dx, &dy)) {
			if (!m_LastPosValid || dx!=m_LastPosX || dy!=m_LastPosY) {
				m_LastPosValid = true;
				m_LastPosX = dx;
				m_LastPosY = dy;
				PrintKeyValue(buf1, dx, m_Table.ChannelsX()->Get(m_ChannelX));
				PrintKeyValue(buf2, dy, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "Cursor: %s, %s", buf1, buf2);
				SetStatus(msg);
			}
		} else {
			if (m_LastPosValid) {
				m_LastPosValid = false;
				SetStatus(NULL);
			}
		}
	}
}

void CLightCurveDlg::PrintValue(char *buf, double value, const CChannel *channel)
{
	const gchar *unit = channel->Unit();
	int prec = channel->Precision();

	if (unit)
		sprintf(buf, "%.*f %s", prec, value, unit);
	else
		sprintf(buf, "%.*f", prec, value);
}

void CLightCurveDlg::PrintKeyValue(char *buf, double value, const CChannel *channel)
{
	const gchar *name = channel->Name(), *unit = channel->Unit();
	int prec = channel->Precision();

	if (unit)
		sprintf(buf, "%s = %.*f %s", name, prec, value, unit);
	else
		sprintf(buf, "%s = %.*f", name, prec, value);
}

void CLightCurveDlg::PrintKeyValueU(char *buf, double value, double sdev, const CChannel *channel)
{
	const gchar *name = channel->Name(), *unit = channel->Unit();
	int prec = channel->Precision();

	if (unit)
		sprintf(buf, "%s = %.*f +- %.*f %s", name, prec, value, prec, sdev, unit);
	else
		sprintf(buf, "%s = %.*f +- %.*f", name, prec, value, prec, sdev);
}

void CLightCurveDlg::UpdateTools(void)
{
	char	buf[256], msg[256];
	double	dx, dy, dmin, dmax, mean, sdev;
	int		xcol, ycol, ucol;

	if (m_InfoMode == INFO_MEASUREMENT) {
		// Update measurement tool
		m_MeasBox.SetChannel(CMPACK_AXIS_X, m_ChannelX);
		m_MeasBox.SetChannel(CMPACK_AXIS_Y, m_ChannelY);
	} else 
	if (m_InfoMode == INFO_STATISTICS) {
		m_InfoBox.BeginUpdate();
		m_InfoBox.Clear();
		// Show information about selected star
		xcol = m_Table.ChannelsX()->GetColumn(m_ChannelX);
		ycol = m_Table.ChannelsY()->GetColumn(m_ChannelY);
		ucol = m_Table.ChannelsY()->GetColumnU(m_ChannelY);
		int selected_count = cmpack_graph_view_get_selected_count(CMPACK_GRAPH_VIEW(m_Graph));
		if (selected_count==1) {
			int row = cmpack_graph_view_get_selected(CMPACK_GRAPH_VIEW(m_Graph));
			int frame_id = (int)cmpack_graph_data_get_param(m_GraphData, row);
			if (frame_id>=0) {
				sprintf(buf, "Frame #%d", frame_id);
				m_InfoBox.AddTitle(1, buf);
				if (m_Table.Find(frame_id) && m_Table.GetDbl(xcol, &dx) && m_Table.GetDbl(ycol, &dy)) {
					PrintKeyValue(buf, dx, m_Table.ChannelsX()->Get(m_ChannelX));
					m_InfoBox.AddText(buf);
					if (ucol>=0 && m_Table.GetDbl(ucol, &sdev)) 
						PrintKeyValueU(buf, dy, sdev, m_Table.ChannelsY()->Get(m_ChannelY));
					else
						PrintKeyValue(buf, dy, m_Table.ChannelsY()->Get(m_ChannelY));
					m_InfoBox.AddText(buf);
				}
			}
		} else {
			// Compute statistics
			int count = 0, length = (selected_count>0 ? selected_count : m_Table.Rows());
			double *x = new double[length], *y = new double[length];
			if (selected_count>0) {
				GList *list = cmpack_graph_view_get_selected_rows(CMPACK_GRAPH_VIEW(m_Graph));
				for (GList *ptr=list; ptr!=NULL; ptr=ptr->next) {
					int frame_id = (int)cmpack_graph_data_get_param(m_GraphData, (gint)(intptr_t)ptr->data);
					if (m_Table.Find(frame_id) && m_Table.GetDbl(xcol, x+count) && m_Table.GetDbl(ycol, y+count))
						count++;
				}
				g_list_free(list);
			} else {
				bool ok = m_Table.Rewind();
				while (ok) {
					if (m_Table.GetDbl(xcol, x+count) && m_Table.GetDbl(ycol, y+count))
						count++;
					ok = m_Table.Next();
				}
			}
			if (selected_count>0)
				sprintf(buf, "There are %d selected points.", count);
			else
				sprintf(buf, "There are %d points in total.", count);
			m_InfoBox.AddText(buf);
			if (ComputeMinMax(count, x, &dmin, &dmax)) {
				m_InfoBox.AddText("");
				sprintf(msg, "Data %s:", m_Table.ChannelsX()->GetName(m_ChannelX));
				m_InfoBox.AddTitle(1, msg);
				PrintValue(buf, dmin, m_Table.ChannelsX()->Get(m_ChannelX));
				sprintf(msg, "%s: %s", "Min.", buf);
				m_InfoBox.AddText(msg);
				PrintValue(buf, dmax, m_Table.ChannelsX()->Get(m_ChannelX));
				sprintf(msg, "%s: %s", "Max.", buf);
				m_InfoBox.AddText(msg);
			}
			if (ComputeMinMax(count, y, &dmin, &dmax)) {
				cmpack_robustmean(count, y, &mean, &sdev);
				m_InfoBox.AddText("");
				sprintf(msg, "Data %s:", m_Table.ChannelsY()->GetName(m_ChannelY));
				m_InfoBox.AddTitle(1, msg);
				PrintValue(buf, dmin, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "%s: %s", "Min.", buf);
				m_InfoBox.AddText(msg);
				PrintValue(buf, dmax, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "%s: %s", "Max.", buf);
				m_InfoBox.AddText(msg);
				PrintValue(buf, mean, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "%s: %s", "Mean", buf);
				m_InfoBox.AddText(msg);
				PrintValue(buf, sdev, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "%s: %s", "St.dev.", buf);
				m_InfoBox.AddText(msg);
			}
			delete[] x;
			delete[] y;
		}
		m_InfoBox.EndUpdate();
	} else {
		// No tool is open
		cmpack_graph_view_set_cursors(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X, 0);
		cmpack_graph_view_set_cursors(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y, 0);
	}
}

void CLightCurveDlg::SetInfoMode(tInfoMode mode)
{
	m_InfoMode = mode;
	switch (m_InfoMode) 
	{
	case INFO_STATISTICS:
		m_MeasBox.Show(false);
		m_InfoBox.Show(true);
		break;
	case INFO_MEASUREMENT:
		m_InfoBox.Show(false);
		m_MeasBox.Show(true);
		break;
	default:
		m_MeasBox.Show(false);
		m_InfoBox.Show(false);
		break;
	}
	UpdateTools();
	UpdateControls();
}

void CLightCurveDlg::InfoBoxCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data)
{
	CLightCurveDlg *pMe = (CLightCurveDlg*)cb_data;

	switch (message)
	{
	case CInfoBox::CB_CLOSED:
		pMe->OnInfoBoxClosed();
		break;
	}
}

void CLightCurveDlg::OnInfoBoxClosed(void)
{
	m_InfoMode = INFO_NONE;
	UpdateControls();
}

void CLightCurveDlg::SaveLightCurve(void)
{
	bool jdhel = strcmp(m_Table.ChannelsX()->GetName(m_ChannelX), "JDHEL")==0;

	CSaveLightCurveDlg pDlg(GTK_WINDOW(m_pDlg), m_Params.instmag);
	if (pDlg.Execute(m_Table, jdhel)) {
		m_DataSaved = true;
		UpdateControls();
	}
}

void CLightCurveDlg::ExportAsImage(void)
{
	CGraphExportDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute(CMPACK_GRAPH_VIEW(m_Graph));
}

void CLightCurveDlg::EditProperties(void)
{
	CLightCurveOptionsDlg dlg(GTK_WINDOW(m_pDlg));
	if (dlg.Execute(m_Params, m_ObjCoords, m_Location)) {
		g_Project->SetBool("LightCurve", "HelCorr", m_Params.helcor);
		g_Project->SetBool("LightCurve", "AirMass", m_Params.amass);
		g_Project->SetBool("LightCurve", "InstMag", m_Params.instmag);
		g_Project->SetObjectCoords(m_ObjCoords);
		CConfig::SetLocation(m_Location);
		m_DataSaved = false;
		UpdateLightCurve();
		UpdateChannels();
		UpdateGraph(TRUE, TRUE);
		UpdateTools();
		SetStatus(NULL);
		UpdateControls();
	}
}

void CLightCurveDlg::EditSelection(void)
{
	CChooseStarsDlg dlg(GTK_WINDOW(m_pDlg));
	if (dlg.Execute(m_Selection, m_Tags, m_Params.instmag)) {
		g_Project->SetSelection(m_Selection);
		g_Project->SetTags(m_Tags);
		m_DataSaved = false;
		RebuildData(GTK_WINDOW(m_pDlg));
		UpdateLightCurve();
		UpdateChannels();
		UpdateGraph(TRUE, TRUE);
		UpdateTools();
		SetStatus(NULL);
		UpdateControls();
	}
}

void CLightCurveDlg::ChangeAperture(void)
{
	bool ok;

	int index = m_ApertureIndex;
	if (m_Selection.CountStars(CMPACK_SELECT_CHECK)>0 && m_Selection.CountStars(CMPACK_SELECT_COMP)>0) {
		CChooseApertureDlg dlg(GTK_WINDOW(m_pDlg));
		ok = dlg.Execute(m_FrameSet, m_AperList, m_Selection, &index);
	} else {
		CChooseApertureSimpleDlg dlg(GTK_WINDOW(m_pDlg));
		ok = dlg.Execute(m_FrameSet, m_AperList, m_Selection, &index);
	}
	if (ok) {
		if (index!=m_ApertureIndex) {
			m_ApertureIndex = index;
			gtk_combo_box_set_active(GTK_COMBO_BOX(m_ACombo), m_ApertureIndex);
			g_Project->SetInt("LightCurve", "Aperture", m_AperList.GetId(m_ApertureIndex));
			m_DataSaved = false;
			UpdateLightCurve();
			UpdateGraph(TRUE, TRUE);
			UpdateTools();
			SetStatus(NULL);
			UpdateControls();
		}
	}
}

void CLightCurveDlg::ShowChart(void)
{
	CChartDlg dlg(GTK_WINDOW(m_pDlg));
	dlg.Execute(m_Selection, m_Tags, m_AperList.GetId(m_ApertureIndex));
}

GList *CLightCurveDlg::GetSelectedFrames(void)
{
	GList *frames = NULL;

	GList *rows = cmpack_graph_view_get_selected_rows(CMPACK_GRAPH_VIEW(m_Graph));
	if (rows) {
		for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
			int frame = cmpack_graph_data_get_param(m_GraphData, (intptr_t)ptr->data);
			GtkTreePath *path = g_Project->GetFilePath(frame);
			if (path)
				frames = g_list_prepend(frames, path);
		}
		g_list_free(rows);
	}
	return g_list_reverse(frames);
}

GtkTreePath *CLightCurveDlg::GetSelectedFrame(void)
{
	int row = cmpack_graph_view_get_selected(CMPACK_GRAPH_VIEW(m_Graph));
	if (row>=0) {
		int frame = cmpack_graph_data_get_param(m_GraphData, row);
		return g_Project->GetFilePath(frame);
	}
	return NULL;
}

//----------------------------   LIGHT CURVE OPTIONS ---------------------------------

CLightCurveOptionsDlg::CLightCurveOptionsDlg(GtkWindow *pParent)
{
	GtkWidget *tbox;

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Light curve options", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_APPLY, GTK_RESPONSE_ACCEPT, 
		GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_dialog_widget_standard_tooltips(GTK_DIALOG(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);

	// Dialog icon
	gchar *icon = get_icon_file("lightcurve");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog layout
	tbox = gtk_table_new(12, 4, FALSE);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(m_pDlg)->vbox), tbox);
	gtk_container_set_border_width(GTK_CONTAINER(tbox), 4);
	gtk_table_set_row_spacings(GTK_TABLE(tbox), 4);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 2, 12);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 3, 12);
	gtk_table_set_row_spacing(GTK_TABLE(tbox), 6, 12);
	gtk_table_set_col_spacings(GTK_TABLE(tbox), 4);

	// Light curve options
	m_HelCor = gtk_check_button_new_with_label("Compute heliocentric correction");
	g_signal_connect(G_OBJECT(m_HelCor), "toggled", G_CALLBACK(button_toggled), this);
	gtk_table_attach(GTK_TABLE(tbox), m_HelCor, 0, 3, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	m_AirMass = gtk_check_button_new_with_label("Compute air mass coefficients");
	g_signal_connect(G_OBJECT(m_AirMass), "toggled", G_CALLBACK(button_toggled), this);
	gtk_table_attach(GTK_TABLE(tbox), m_AirMass, 0, 3, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	m_InstMag = gtk_check_button_new_with_label("Show raw instrumental magnitudes");
	g_signal_connect(G_OBJECT(m_InstMag), "toggled", G_CALLBACK(button_toggled), this);
	gtk_table_attach(GTK_TABLE(tbox), m_InstMag, 0, 3, 2, 3, GTK_FILL, GTK_FILL, 0, 0);

	// Object coordinates
	m_ObjLabel = gtk_label_new("Object - designation");
	gtk_misc_set_alignment(GTK_MISC(m_ObjLabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjLabel, 0, 1, 4, 5);
	m_ObjName = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(m_ObjName), MAX_OBJECT_LEN);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjName, 1, 2, 4, 5);
	m_ObjBtn = gtk_button_new_with_label("More");
	g_signal_connect(G_OBJECT(m_ObjBtn), "clicked", G_CALLBACK(button_clicked), this);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_ObjBtn, 2, 3, 4, 5);
	m_RALabel = gtk_label_new("- right ascension");
	gtk_misc_set_alignment(GTK_MISC(m_RALabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RALabel, 0, 1, 5, 6);
	m_RA = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(m_RA), 64);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RA, 1, 2, 5, 6);
	m_RAUnit = gtk_label_new("[h m s]");
	gtk_misc_set_alignment(GTK_MISC(m_RAUnit), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_RAUnit, 2, 3, 5, 6);
	m_DecLabel = gtk_label_new("- declination");
	gtk_misc_set_alignment(GTK_MISC(m_DecLabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_DecLabel, 0, 1, 6, 7);
	m_Dec = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(m_Dec), 64);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Dec, 1, 2, 6, 7);
	m_DecUnit = gtk_label_new("[\xC2\xB1""d m s]");
	gtk_misc_set_alignment(GTK_MISC(m_DecUnit), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_DecUnit, 2, 3, 6, 7);

	// Location
	m_LocLabel = gtk_label_new("Location - name");
	gtk_misc_set_alignment(GTK_MISC(m_LocLabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LocLabel, 0, 1, 7, 8);
	m_LocName = gtk_entry_new();
	gtk_entry_set_max_length(GTK_ENTRY(m_LocName), MAX_LOCATION_LEN);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LocName, 1, 2, 7, 8);
	m_LocBtn = gtk_button_new_with_label("More");
	g_signal_connect(G_OBJECT(m_LocBtn), "clicked", G_CALLBACK(button_clicked), this);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LocBtn, 2, 3, 7, 8);
	m_LonLabel = gtk_label_new("- longitude");
	gtk_misc_set_alignment(GTK_MISC(m_LonLabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LonLabel, 0, 1, 8, 9);
	m_Lon = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Lon, 1, 2, 8, 9);
	m_LonUnit = gtk_label_new("[d m s E/W]");
	gtk_misc_set_alignment(GTK_MISC(m_LonUnit), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LonUnit, 2, 3, 8, 9);
	m_LatLabel = gtk_label_new("- latitude");
	gtk_misc_set_alignment(GTK_MISC(m_LatLabel), 1.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LatLabel, 0, 1, 9, 10);
	m_Lat = gtk_entry_new_with_max_length(32);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_Lat, 1, 2, 9, 10);
	m_LatUnit = gtk_label_new("[ d m s N/S]");
	gtk_misc_set_alignment(GTK_MISC(m_LatUnit), 0.0, 0.5);
	gtk_table_attach_defaults(GTK_TABLE(tbox), m_LatUnit, 2, 3, 9, 10);

	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);
}

CLightCurveOptionsDlg::~CLightCurveOptionsDlg()
{
	gtk_widget_destroy(m_pDlg);
}

void CLightCurveOptionsDlg::response_dialog(GtkDialog *pDlg, gint response_id, CLightCurveOptionsDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id))
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CLightCurveOptionsDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Commit changes
		return OnCloseQuery();

	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_MAKE_LIGHT_CURVE);
		return false;
	}
	return true;
}

bool CLightCurveOptionsDlg::Execute(CLightCurveDlg::tParamsRec &params, 
	CObjectCoords &obj, CLocation &loc)
{
	m_ComputeHC = params.helcor;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_HelCor), m_ComputeHC);
	m_ComputeAM = params.amass;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_AirMass), m_ComputeAM);
	m_ShowInstMag = params.instmag;
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_InstMag), m_ShowInstMag);
	m_ObjCoords = obj;
	m_Location = loc;

	UpdateObjectCoords();
	UpdateLocation();
	UpdateControls();
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg)) != GTK_RESPONSE_ACCEPT) 
		return false;

	params.amass = m_ComputeAM;
	params.helcor = m_ComputeHC;
	params.instmag = m_ShowInstMag;
	obj = m_ObjCoords;
	loc = m_Location;
	return true;
}

//
// Set parameters
//
void CLightCurveOptionsDlg::UpdateObjectCoords(void)
{
	const CObjectCoords *obj = &m_ObjCoords;
	if (obj->Name())
		gtk_entry_set_text(GTK_ENTRY(m_ObjName), obj->Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_ObjName), "");
	if (obj->RA())
		gtk_entry_set_text(GTK_ENTRY(m_RA), obj->RA());
	else
		gtk_entry_set_text(GTK_ENTRY(m_RA), "");
	if (obj->Dec())
		gtk_entry_set_text(GTK_ENTRY(m_Dec), obj->Dec());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Dec), "");
}

void CLightCurveOptionsDlg::UpdateLocation(void)
{
	const CLocation *obs = &m_Location;
	if (obs->Name())
		gtk_entry_set_text(GTK_ENTRY(m_LocName), obs->Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_LocName), "");
	if (obs->Lon())
		gtk_entry_set_text(GTK_ENTRY(m_Lon), obs->Lon());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lon), "");
	if (obs->Lat())
		gtk_entry_set_text(GTK_ENTRY(m_Lat), obs->Lat());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lat), "");
}

void CLightCurveOptionsDlg::UpdateControls()
{
	gboolean ok;

	ok = m_ComputeHC || m_ComputeAM;
	gtk_widget_set_sensitive(m_ObjBtn, ok);
	gtk_widget_set_sensitive(m_ObjName, ok);
	gtk_widget_set_sensitive(m_ObjLabel, ok);
	gtk_widget_set_sensitive(m_RA, ok);
	gtk_widget_set_sensitive(m_RALabel, ok);
	gtk_widget_set_sensitive(m_RAUnit, ok);
	gtk_widget_set_sensitive(m_Dec, ok);
	gtk_widget_set_sensitive(m_DecLabel, ok);
	gtk_widget_set_sensitive(m_DecUnit, ok);

	ok = m_ComputeAM;
	gtk_widget_set_sensitive(m_LocBtn, ok);
	gtk_widget_set_sensitive(m_LocName, ok);
	gtk_widget_set_sensitive(m_LocLabel, ok);
	gtk_widget_set_sensitive(m_Lat, ok);
	gtk_widget_set_sensitive(m_LatLabel, ok);
	gtk_widget_set_sensitive(m_LatUnit, ok);
	gtk_widget_set_sensitive(m_Lon, ok);
	gtk_widget_set_sensitive(m_LonLabel, ok);
	gtk_widget_set_sensitive(m_LonUnit, ok);
}

bool CLightCurveOptionsDlg::OnCloseQuery()
{
	const gchar *ra, *dec, *lon, *lat;
	gchar buf[64];
	gdouble x, y;

	m_ComputeHC = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_HelCor))!=0;
	m_ComputeAM = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_AirMass))!=0;
	m_ShowInstMag = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_InstMag))!=0;
	
	if (m_ComputeHC || m_ComputeAM) {
		ra = gtk_entry_get_text(GTK_ENTRY(m_RA));
		dec = gtk_entry_get_text(GTK_ENTRY(m_Dec));
		if (*ra=='\0' || *dec=='\0') {
			ShowError(GTK_WINDOW(m_pDlg), "Please, enter object coordinates.");
			return false;
		}
		if (cmpack_strtora(ra, &x)!=0) {
			ShowError(GTK_WINDOW(m_pDlg), "Invalid value of the right ascension.");
			return false;
		}
		cmpack_ratostr(x, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_RA), buf);
		
		if (cmpack_strtodec(dec, &y)!=0) {
			ShowError(GTK_WINDOW(m_pDlg), "Invalid value of the declination.");
			return false;
		}
		cmpack_dectostr(y, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_Dec), buf);
	}
	m_ObjCoords.SetName(gtk_entry_get_text(GTK_ENTRY(m_ObjName)));
	m_ObjCoords.SetRA(gtk_entry_get_text(GTK_ENTRY(m_RA)));
	m_ObjCoords.SetDec(gtk_entry_get_text(GTK_ENTRY(m_Dec)));

	if (m_ComputeAM) {
		lon = gtk_entry_get_text(GTK_ENTRY(m_Lon));
		lat = gtk_entry_get_text(GTK_ENTRY(m_Lat));
		if (*lon=='\0' || *lat=='\0') {
			ShowError(GTK_WINDOW(m_pDlg), "Please, enter the geographic coordinates.");
			return false;
		}
		if (cmpack_strtolon(lon, &x)!=0) {
			ShowError(GTK_WINDOW(m_pDlg), "Invalid value of the longitude.");
			return false;
		}
		cmpack_lontostr(x, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_Lon), buf);
		
		if (cmpack_strtolat(lat, &y)!=0) {
			ShowError(GTK_WINDOW(m_pDlg), "Invalid value of the latitude.");
			return false;
		}
		cmpack_lattostr(y, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_Lat), buf);
	}
	m_Location.SetName(gtk_entry_get_text(GTK_ENTRY(m_LocName)));
	m_Location.SetLon(gtk_entry_get_text(GTK_ENTRY(m_Lon)));
	m_Location.SetLat(gtk_entry_get_text(GTK_ENTRY(m_Lat)));

	return true;
}

void CLightCurveOptionsDlg::button_clicked(GtkWidget *pButton, CLightCurveOptionsDlg *pMe)
{
	pMe->OnButtonClicked(pButton);
}

void CLightCurveOptionsDlg::OnButtonClicked(GtkWidget *pButton)
{
	if (pButton == m_ObjBtn) 
		EditObjectCoords();
	else if (pButton == m_LocBtn) 
		EditLocation();
	UpdateControls();
}

void CLightCurveOptionsDlg::button_toggled(GtkWidget *pButton, CLightCurveOptionsDlg *pMe)
{
	pMe->OnButtonToggled(pButton);
}

void CLightCurveOptionsDlg::OnButtonToggled(GtkWidget *pButton)
{
	m_ComputeHC = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_HelCor))!=0;
	m_ComputeAM = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_AirMass))!=0;
	UpdateControls();
}

void CLightCurveOptionsDlg::EditObjectCoords(void)
{
	CObjectDlg dlg(GTK_WINDOW(m_pDlg));

	m_ObjCoords.SetName(gtk_entry_get_text(GTK_ENTRY(m_ObjName)));
	m_ObjCoords.SetRA(gtk_entry_get_text(GTK_ENTRY(m_RA)));
	m_ObjCoords.SetDec(gtk_entry_get_text(GTK_ENTRY(m_Dec)));
	if (dlg.Execute(&m_ObjCoords)) {
		UpdateObjectCoords();
		UpdateControls();
	}
}

void CLightCurveOptionsDlg::EditLocation(void)
{
	CLocationDlg dlg(GTK_WINDOW(m_pDlg));

	m_Location.SetName(gtk_entry_get_text(GTK_ENTRY(m_LocName)));
	m_Location.SetLon(gtk_entry_get_text(GTK_ENTRY(m_Lon)));
	m_Location.SetLat(gtk_entry_get_text(GTK_ENTRY(m_Lat)));
	if (dlg.Execute(&m_Location)) {
		UpdateLocation();
		UpdateControls();
	}
}

//-------------------------   SAVE LIGHT CURVE   --------------------------------

CSaveLightCurveDlg::CSaveLightCurveDlg(GtkWindow *pParent, bool instmag):m_Updating(false), 
	m_InstMag(instmag), m_FileType(TYPE_MUNIPACK), m_JDType(DATE_JD_GEOCENTRIC), m_ExportPath(NULL)
{
	GtkFileFilter *filters[3];
	GtkWidget *frame, *hbox, *label;
	GtkTreeIter iter;
	GtkCellRenderer *renderer;

	// Dialog with buttons
	m_pDlg = gtk_file_chooser_dialog_new("Save light curve", pParent,
		GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(m_pDlg));
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(m_pDlg), true);

	// Dialog icon
	gchar *icon = get_icon_file("lightcurve");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// File filters
	filters[0] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[0], "*.txt");
	gtk_file_filter_add_pattern(filters[0], "*.dat");
	gtk_file_filter_set_name(filters[0], "Text files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);
	filters[1] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[1], "*.djm");
	gtk_file_filter_set_name(filters[1], "AVE files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[1]);
	filters[2] = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filters[2], "*");
	gtk_file_filter_set_name(filters[2], "All files");
	gtk_file_chooser_add_filter(GTK_FILE_CHOOSER(m_pDlg), filters[2]);
	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(m_pDlg), filters[0]);

	// Options
	frame = gtk_frame_new("Export options");
	hbox = gtk_hbox_new(FALSE, 8);
	gtk_container_add(GTK_CONTAINER(frame), hbox);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), 8);
	m_FileTypes = gtk_list_store_new(2, G_TYPE_INT, G_TYPE_STRING);
	for (int i=0; FileFormats[i].Caption; i++) {
		if ((!m_InstMag && FileFormats[i].DiffMag) || (m_InstMag && FileFormats[i].InstMag)) {
			gtk_list_store_append(m_FileTypes, &iter);
			gtk_list_store_set(m_FileTypes, &iter, 0, i, 1, FileFormats[i].Caption, -1);
		}
	}
	label = gtk_label_new("File type");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
	m_TypeCombo = gtk_combo_box_new_with_model(GTK_TREE_MODEL(m_FileTypes));
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(m_TypeCombo), renderer, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(m_TypeCombo), renderer, "text", 1);
	gtk_box_pack_start(GTK_BOX(hbox), m_TypeCombo, FALSE, TRUE, 0);
	g_signal_connect(G_OBJECT(m_TypeCombo), "changed", G_CALLBACK(selection_changed), this);

	m_JDTypes = gtk_list_store_new(2, G_TYPE_INT, G_TYPE_STRING);
	label = gtk_label_new("Observation time");
	gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, TRUE, 0);
	m_JDCombo = gtk_combo_box_new_with_model(GTK_TREE_MODEL(m_JDTypes));
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(m_JDCombo), renderer, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(m_JDCombo), renderer, "text", 1);
	gtk_box_pack_start(GTK_BOX(hbox), m_JDCombo, FALSE, TRUE, 0);
	g_signal_connect(G_OBJECT(m_JDCombo), "changed", G_CALLBACK(selection_changed), this);
	
	gtk_file_chooser_set_extra_widget(GTK_FILE_CHOOSER(m_pDlg), frame);
	gtk_widget_show_all(frame);
}

CSaveLightCurveDlg::~CSaveLightCurveDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_object_unref(m_FileTypes);
	g_object_unref(m_JDTypes);
	g_free(m_ExportPath);
}

CSaveLightCurveDlg::tFileType CSaveLightCurveDlg::StrToFileType(const gchar *str)
{
	if (str) {
		for (gint i=0; FileFormats[i].Id!=NULL; i++) {
			if (strcmp(FileFormats[i].Id, str)==0)
				return (tFileType)i;
		}
	}
	return TYPE_MUNIPACK;
}

const gchar *CSaveLightCurveDlg::FileTypeToStr(tFileType type)
{
	if (type>=0 && type<TYPE_N_ITEMS)
		return FileFormats[type].Id;
	return "";
}

bool CSaveLightCurveDlg::Execute(const CTable &table, bool jdhel)
{
	GtkTreeIter iter;
	char *filename, *folder, *aux;

	m_Table.MakeCopy(table);
	aux = g_Project->GetStr("LightCurve", (!m_InstMag ? "FileType" : "FileType2"));
	m_FileType = StrToFileType(aux);
	g_free(aux);

	m_JDType = (jdhel ? DATE_JD_HELIOCENTRIC : DATE_JD_GEOCENTRIC);

	m_Updating = true;

	// Restore last folder and file name
	folder = g_Project->GetStr("Output", "Folder", NULL);
	if (!folder)
		folder = CConfig::GetStr("Output", "Folder", NULL);
	if (!folder)
		folder = g_path_get_dirname(g_Project->Path());
	if (folder && g_file_test(folder, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(m_pDlg), folder);
	g_free(folder);
	filename = g_Project->GetStr("LightCurve", "File", "lightcurve.txt");
	const gchar *defext = FileExtension(m_FileType);
	if (defext) {
		gchar *newname = SetFileExtension(filename, defext);
		g_free(filename);
		filename = newname;
	}
	gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_pDlg), g_basename(filename));
	g_free(filename);

	// File types
	SelectItem(GTK_COMBO_BOX(m_TypeCombo), m_FileType);
	gtk_widget_set_sensitive(m_TypeCombo, 
		gtk_tree_model_iter_n_children(GTK_TREE_MODEL(m_FileTypes), NULL)>1);
	// JD columns
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_JDCombo), NULL);
	gtk_list_store_clear(m_JDTypes);
	for (int i=0; DateFormats[i].Caption; i++) {
		if (i==DATE_JD_GEOCENTRIC || m_Table.ChannelsX()->FindFirst("JDHEL")>=0) {
			gtk_list_store_append(m_JDTypes, &iter);
			gtk_list_store_set(m_JDTypes, &iter, 0, i, 1, DateFormats[i].Caption, -1);
		}
	}
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_JDCombo), GTK_TREE_MODEL(m_JDTypes));
	SelectItem(GTK_COMBO_BOX(m_JDCombo), m_JDType);
	gtk_widget_set_sensitive(m_JDCombo, 
		gtk_tree_model_iter_n_children(GTK_TREE_MODEL(m_JDTypes), NULL)>1);

	m_Updating = false;

	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))!=GTK_RESPONSE_ACCEPT)
		return false;

	// Save last folder and file name
	folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(m_pDlg));
	g_Project->SetStr("Output", "Folder", folder);
	CConfig::GetStr("Output", "Folder", folder);
	g_free(folder);
	m_ExportPath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
	g_Project->SetStr("LightCurve", "File", g_basename(m_ExportPath));
	g_Project->SetStr("LightCurve", (!m_InstMag ? "FileType" : "FileType2"), FileTypeToStr(m_FileType));

	CProgressDlg pdlg(GTK_WINDOW(m_pDlg), "Saving light curve");
	pdlg.SetFileName(g_basename(m_ExportPath));
	pdlg.Execute(ExportLCurveProc, this);
	g_free(m_ExportPath);
	m_ExportPath = NULL;
	return true;
}

void CSaveLightCurveDlg::selection_changed(GtkComboBox *pWidget, CSaveLightCurveDlg *pMe)
{
	pMe->OnSelectionChanged(pWidget);
}

void CSaveLightCurveDlg::OnSelectionChanged(GtkComboBox *pWidget)
{
	int id;
	GtkTreeIter iter;

	if (!m_Updating && gtk_combo_box_get_active_iter(pWidget, &iter)) {
		if (GTK_WIDGET(pWidget) == m_TypeCombo) {
			gtk_tree_model_get(GTK_TREE_MODEL(m_FileTypes), &iter, 0, &id, -1);
			if (id != m_FileType) {
				m_FileType = (tFileType)id;
				OnTypeChanged();
			}
		} else
		if (GTK_WIDGET(pWidget) == m_JDCombo) {
			gtk_tree_model_get(GTK_TREE_MODEL(m_JDTypes), &iter, 0, &id, -1);
			m_JDType = (tDateType)id;
		}
	}
}

void CSaveLightCurveDlg::OnTypeChanged(void)
{
	const gchar *ext = FileExtension(m_FileType);
	if (ext) {
		gchar *oldname = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(m_pDlg));
		if (oldname) {
			gchar *newname = SetFileExtension(oldname, ext);
			gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(m_pDlg), g_basename(newname));
			g_free(newname);
			g_free(oldname);
		}
	}
}

int CSaveLightCurveDlg::ExportLCurveProc(CProgressDlg *sender, void *user_data)
{
	return ((CSaveLightCurveDlg*)user_data)->OnExportCurve(sender);
}

int CSaveLightCurveDlg::OnExportCurve(CProgressDlg *sender)
{
	int i, magcol = -1, res = 0;
	double mag;
	bool first;

	CChannels *cx = m_Table.ChannelsX();
	for (i=0; i<cx->Count(); i++) {
		bool jdhel = strcmp(m_Table.ChannelsX()->GetName(i), "JDHEL")==0;
		if (m_JDType==DATE_JD_GEOCENTRIC) 
			cx->Get(i)->SetExported(!jdhel);
		else
			cx->Get(i)->SetExported(jdhel);
	}

	CChannels *cy = m_Table.ChannelsY();
	first = true;
	for (i=0; i<cy->Count(); i++) {
		int column = m_Table.ChannelsY()->GetColumn(i);
		if (first && m_Table.ChannelsY()->GetInfo(i) == DATA_MAGNITUDE) {
			magcol = column;
			first = false;
		}
		if (m_FileType==TYPE_AVE) 
			cy->Get(i)->SetExported(column==magcol);
		else
			cy->Get(i)->SetExported(true);
		if (m_FileType==TYPE_MCV)
			m_Table.SetNulValue(column, 0);
	}

	bool ok = m_Table.Rewind();
	while (ok) {
		if (m_FileType==TYPE_AVE && magcol>=0) 
			m_Table.Show(m_Table.GetDbl(magcol, &mag));
		else
			m_Table.Show(true);
		ok = m_Table.Next();
	}

	if (m_JDType==DATE_JD_GEOCENTRIC)
		m_Table.SetParam("JD", "geocentric");
	else
		m_Table.SetParam("JD", "heliocentric");
	if (m_FileType==TYPE_AVE || m_FileType==TYPE_MCV)
		m_Table.Export(m_ExportPath, true, true, true, &res);
	else
		m_Table.Export(m_ExportPath, true, false, false, &res);
	return res;
}
