/**************************************************************

phot_class.cpp (C-Munipack project)
Photometry file class interface
Copyright (C) 2011 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>

#include "phot_class.h"

//--------------------------   PHOTOMETRY FILE  ----------------------------------

//
// Default constructor
//
CPhot::CPhot():m_Handle(NULL), m_CacheFlags(0), m_RefCount(0), m_RefMag(NULL)
{
}

//
// Destructor
//
CPhot::~CPhot()
{
	if (m_Handle)
		cmpack_pht_destroy(m_Handle);
	g_free(m_RefMag);
}

//
// Clear data
//
bool CPhot::Close(int *result)
{
	if (m_Handle) {
		int res = cmpack_pht_close(m_Handle);
		if (res!=0) {
			if (result)
				*result = res;
			return false;
		}
		m_Handle = NULL;
		m_Apertures.Clear();
		g_free(m_RefMag);
		m_RefMag = NULL;
		m_RefCount = 0;
		m_CacheFlags = 0;
	}
	if (result)
		*result = 0;
	return true;
}

//
// Set image info from a file
//
bool CPhot::Open(const gchar *fpath, CmpackOpenMode mode, unsigned flags, int *result)
{
	int res;

	Close();

	if (mode != CMPACK_OPEN_CREATE) {
		if (!fpath || !g_file_test(fpath, G_FILE_TEST_IS_REGULAR)) {
			if (result)
				*result = CMPACK_ERR_OPEN_ERROR;
			return false;
		}
	} else {
		if (!fpath) {
			if (result)
				*result = CMPACK_ERR_OPEN_ERROR;
			return false;
		}
	}
	
	char *f = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);
	res = cmpack_pht_open(&m_Handle, f, mode, flags);
	g_free(f);
	if (res!=0) {
		if (result)
			*result = res;
		return false;
	}

	InvalidateCache();
	if (result)
		*result = CMPACK_ERR_OK;
	return true;
}

//
// Get chart width in pixels
//
void CPhot::InvalidateCache(void)
{
	m_CacheFlags = 0;
	free(m_RefMag);
	m_RefMag = NULL;
	m_RefCount = 0;
}

//
// Get number of stars
//
int CPhot::ObjectCount(void) const
{
	if (m_Handle)
		return cmpack_pht_object_count(m_Handle);
	return 0;
}

//
// Get object identifier
//
int CPhot::GetObjectID(int index)
{
	CmpackPhtObject obj;

	if (m_Handle && cmpack_pht_get_object(m_Handle, index, CMPACK_PO_ID, &obj)==0)
		return obj.id;
	return -1;
}

//
// Get object reference identifier
//
int CPhot::GetObjectRefID(int index)
{
	CmpackPhtObject obj;

	if (m_Handle && cmpack_pht_get_object(m_Handle, index, CMPACK_PO_REF_ID, &obj)==0)
		return obj.ref_id;
	return -1;
}

//
// Get object position
//
bool CPhot::GetObjectPos(int index, double *x, double *y)
{
	CmpackPhtObject obj;

	if (m_Handle && cmpack_pht_get_object(m_Handle, index, CMPACK_PO_CENTER, &obj)==0) {
		if (x)
			*x = obj.x;
		if (y)
			*y = obj.y;
		return true;
	}	
	return false;
}

//
// Get object measurement
//
bool CPhot::GetMagnitude(int star_index, int aperture_index, double *mag, double *err)
{
	CmpackPhtData data;

	if (m_Handle) {
		data.mag_valid = 0;
		cmpack_pht_get_data(m_Handle, star_index, aperture_index, &data);
		if (data.mag_valid) {
			if (mag)
				*mag = data.magnitude;
			if (err)
				*err = data.mag_error;
			return true;
		}
	}
	return false;
}

//
// Update reference magnitudes
//
double CPhot::GetRefMag(int index)
{
	int i, count, nstars;
	double *maglist, mag;

	if (m_Handle) {
		if (!m_RefMag) {
			m_RefCount = cmpack_pht_aperture_count(m_Handle);
			m_RefMag = (tRefMag*)g_malloc0(m_RefCount*sizeof(tRefMag));
		}
		if (m_RefMag && index>=0 && index<m_RefCount) {
			if (!m_RefMag[index].valid) {
				m_RefMag[index].mag = 0.0;
				nstars = cmpack_pht_object_count(m_Handle);
				if (nstars>0) {
					maglist = (double*)g_malloc(nstars*sizeof(double));
					count = 0;
					for (i=0; i<nstars; i++) {
						if (GetMagnitude(i, index, &mag, NULL)) 
							maglist[count++] = mag;
					}
					if (count>0) 
						cmpack_robustmean(count, maglist, &m_RefMag[index].mag, NULL);
					g_free(maglist);
				}
				m_RefMag[index].valid = true;
			}
			return m_RefMag[index].mag;
		}
	}
	return 0.0;
}

//
// Number of apertures
//
int CPhot::ApertureCount(void) const
{
	if (m_Handle)
		return cmpack_pht_aperture_count(m_Handle);
	return 0;
}

//
// Update reference magnitudes
//
const CApertures *CPhot::Apertures(void)
{
	int i, count;
	CmpackPhtAperture aper;

	if (!(m_CacheFlags & CF_APERTURES)) {
		m_Apertures.Clear();
		if (m_Handle) {
			count = cmpack_pht_aperture_count(m_Handle);
			for (i=0; i<count; i++) {
				cmpack_pht_get_aperture(m_Handle, i, CMPACK_PA_ID | CMPACK_PA_RADIUS, &aper);
				m_Apertures.Add(CAperture(aper.id, aper.radius));
			}
		}
		m_CacheFlags |= CF_APERTURES;
	}
	return &m_Apertures;
}

//
// Update reference magnitudes
//
bool CPhot::GetParams(unsigned mask, CmpackPhtInfo &info) const
{
	if (m_Handle)
		return cmpack_pht_get_info(m_Handle, mask, &info)==0;
	return false;
}

//
// Has the file been matched?
//
bool CPhot::Matched(void) const
{
	CmpackPhtInfo info;

	if (GetParams(CMPACK_PI_MATCH_PARAMS, info))
		return info.matched!=0;
	return false;
}

//
// Find aperture
//
int CPhot::FindAperture(int id) const
{
	if (m_Handle)
		return cmpack_pht_find_aperture(m_Handle, id);
	return -1;
}

//
// Render to chart data
//
void CPhot::Render(CmpackChartData *data, int aperture_index, bool gray_not_matched,
	bool transparent)
{
	bool gray;
	int i, count;
	double ref;
	CmpackPhtInfo info;
	CmpackPhtObject obj;
	CmpackPhtData mag;
	CmpackChartItem item;

	memset(&item, 0, sizeof(CmpackChartItem));

	cmpack_chart_data_clear(data);
	if (m_Handle) {
		cmpack_pht_get_info(m_Handle, CMPACK_PI_FRAME_PARAMS, &info);
		cmpack_chart_data_set_dimensions(data, info.width, info.height);
		count = cmpack_pht_object_count(m_Handle);
		cmpack_chart_data_alloc(data, count);
		ref = GetRefMag(aperture_index);
		for (i=0; i<count; i++) {
			cmpack_pht_get_object(m_Handle, i, CMPACK_PO_ID | CMPACK_PO_REF_ID | CMPACK_PO_CENTER, &obj);
			mag.mag_valid = 0;
			cmpack_pht_get_data(m_Handle, i, aperture_index, &mag);
			item.x = obj.x;
			item.y = obj.y;
			item.outline = mag.mag_valid==0;
			gray = (gray_not_matched && obj.ref_id<0);
			item.color = (gray ? CMPACK_COLOR_GRAY : CMPACK_COLOR_DEFAULT);
			item.param = obj.id;
			if (!transparent)
				item.d = 2.0 + MAX(0, (mag.mag_valid ? 1.0 + 0.75*(ref - mag.magnitude) : 1.0));
			else
				item.d = 0;
			cmpack_chart_data_add(data, &item, sizeof(item));
		}
	}
}

int CPhot::FindObject(int obj_id)
{
	if (m_Handle)
		return cmpack_pht_find_object(m_Handle, obj_id, 0);
	return -1;
}

int CPhot::FindObjectRefID(int ref_id)
{
	if (m_Handle)
		return cmpack_pht_find_object(m_Handle, ref_id, 1);
	return -1;
}

//
// Deep copy from another photometry file
//
void CPhot::MakeCopy(const CPhot &phot)
{
	Close();
	if (phot.Handle()) {
		m_Handle = cmpack_pht_init();
		cmpack_pht_copy(m_Handle, phot.Handle());
		InvalidateCache();
	}
}
