/**************************************************************

project_dlg.cpp (C-Munipack project)
Project settings dialog
Copyright (C) 2012 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>
#include <glib/gstdio.h>

#include "utils.h"
#include "configuration.h"
#include "profile.h"
#include "project_dlg.h"
#include "main.h"
#include "project.h"
#include "ctxhelp.h"
#include "loadprofile_dlg.h"
#include "opensave_dlgs.h"
#include "observer_dlg.h"

struct tStringList
{
	const char *str;			// Caption (NULL = terminator)
	int id;						// Value, bitmask
};

struct tStackItem
{
	int level;
	GtkTreePath *item;
	tStackItem(int l, GtkTreePath *i):level(l), item(i) {}
	~tStackItem(void) { if (item) gtk_tree_path_free(item); }
	static void Release(tStackItem *pMe) { delete pMe; }
};

struct tFindData
{
	int page_id;
	GtkTreePath *path;
	tFindData(int page):page_id(page), path(NULL) {}
};

// Options for image rotation
static const tStringList BitpixStrings[] = {
	{ "Autodetection",					CMPACK_BITPIX_AUTO },
	{ "Signed short int (2 bytes)",		CMPACK_BITPIX_SSHORT },
	{ "Unsigned short int (2 bytes)",	CMPACK_BITPIX_USHORT },
	{ "Signed int (4 bytes)",			CMPACK_BITPIX_SLONG },
	{ "Unsigned int (4 bytes)",			CMPACK_BITPIX_ULONG },
	{ "Single precision FP (4 bytes)",	CMPACK_BITPIX_FLOAT },
	{ "Double precision FP (8 bytes)",	CMPACK_BITPIX_DOUBLE },
	{ NULL }
};

// Binning
static const tStringList BinningStrings[] = {
	{ "No binning",			1 },
	{ "2x2",				2 },
	{ "3x3",				3 },
	{ "4x4",				4 },
	{ NULL }
};

static gboolean FindPage(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	int id;
	gtk_tree_model_get(model, iter, 0, &id, -1);
	if (id == ((tFindData*)data)->page_id) {
		((tFindData*)data)->path = gtk_tree_path_copy(path);
		return TRUE;
	}
	return FALSE;
}

static GtkWidget *new_label(const gchar *text, gfloat xalign = 0)
{
	GtkWidget *label = gtk_label_new(NULL);
	gtk_label_set_markup(GTK_LABEL(label), text);
	gtk_misc_set_alignment(GTK_MISC(label), xalign, 0.5);
	return label;
}

static GtkWidget *add_label(GtkTable *tbox, int row, const gchar *text)
{
	GtkWidget *label = new_label(text, 0);
	gtk_table_attach(tbox, label, 0, 3, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
	return label;
}

static GtkWidget *add_entry(GtkTable *tbox, const gchar *caption, gint row, gint maxlen, const gchar *tooltip)
{
	GtkWidget *entry = gtk_entry_new();
	gtk_widget_set_tooltip_text(entry, tooltip);
	gtk_entry_set_max_length(GTK_ENTRY(entry), maxlen);
	gtk_table_attach(tbox, new_label(caption), 0, 1, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach(tbox, entry, 1, 2, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
	return entry;
}

static GtkWidget *add_push_button(GtkTable *tbox, gint row, const gchar *label, const gchar *tooltip)
{
	GtkWidget *button = gtk_button_new_with_label(label);
	gtk_widget_set_tooltip_text(button, tooltip);
	gtk_table_attach(GTK_TABLE(tbox), button, 2, 3, row, row+1, 
		GTK_FILL, GTK_FILL, 0, 0);
	return button;
}

static GtkWidget *add_static(GtkTable *tbox, const gchar *caption, gint row, 
	PangoEllipsizeMode ellipsize, const gchar *tooltip)
{
	GtkWidget *entry;

	gtk_table_attach(tbox, new_label(caption), 0, 1, row, row+1, 
		GTK_FILL, GTK_FILL, 0, 0);
	entry = new_label(NULL);
	gtk_widget_set_tooltip_text(entry, tooltip);
	gtk_label_set_ellipsize(GTK_LABEL(entry), ellipsize);
	gtk_table_attach(tbox, entry, 1, 2, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
	return entry;
}

static GtkWidget *add_spin_button(GtkTable *tbox, int left, int top, 
	double dmin, double dmax, double step, int digits, const gchar *tooltip)
{
	GtkObject *adj = gtk_adjustment_new(0, dmin, dmax, step, 10*step, 0);
	GtkWidget *spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0, digits);
	gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spin), TRUE);
	gtk_widget_set_tooltip_text(spin, tooltip);
	gtk_table_attach(tbox, spin, left, left+1, top, top+1, 
		GTK_FILL, GTK_FILL, 0, 0);
	return spin;
}

static GtkWidget *add_spin_button_dbl(GtkTable *tbox, int top, const gchar *caption, 
	CProfile::tParameter p, double step, int digits, const gchar *tooltip)
{
	double dmin, dmax, defval;
	gchar buf[512];
	GtkWidget *widget;

	gtk_table_attach(tbox, new_label(caption), 0, 1, top, top+1, 
		GTK_FILL, GTK_FILL, 0, 0);
	CProfile::GetLimitsDbl(p, &dmin, &dmax);
	widget = add_spin_button(tbox, 1, top, dmin, dmax, step, digits, tooltip);
	defval = CProfile::GetDefaultDbl(p);
	sprintf(buf, "Default: %.*f", digits, defval);
	gtk_table_attach(tbox, new_label(buf), 2, 3, top, top+1, 
		GtkAttachOptions(GTK_FILL), GTK_FILL, 0, 0);
	return widget;
}

static GtkWidget *add_spin_button_int(GtkTable *tbox, int top, const gchar *caption,
	CProfile::tParameter p, double step, int digits, const gchar *tooltip)
{
	int imin, imax, defval;
	gchar buf[512];
	GtkWidget *widget;

	gtk_table_attach(tbox, new_label(caption), 0, 1, top, top+1, 
		GTK_FILL, GTK_FILL, 0, 0);
	CProfile::GetLimitsInt(p, &imin, &imax);
	widget = add_spin_button(tbox, 1, top, imin, imax, step, digits, tooltip);
	defval = CProfile::GetDefaultInt(p);
	sprintf(buf, "Default: %d", defval);
	gtk_table_attach(tbox, new_label(buf), 2, 3, top, top+1, 
		GtkAttachOptions(GTK_FILL), GTK_FILL, 0, 0);
	return widget;
}

static GtkWidget *add_combo_box(GtkTable *tbox, int top, const gchar *caption, 
	const tStringList *strings, const gchar *tooltip)
{
	GtkWidget *cbox;

	gtk_table_attach(tbox, new_label(caption), 0, 1, top, top+1, GTK_FILL, GTK_FILL, 0, 0);
	cbox = gtk_combo_box_new_text();
	for (int i=0; strings[i].str!=NULL; i++)
		gtk_combo_box_append_text(GTK_COMBO_BOX(cbox), strings[i].str);
	gtk_widget_set_tooltip_text(cbox, tooltip);
	gtk_table_attach(tbox, cbox, 1, 2, top, top+1, GTK_FILL, GTK_FILL, 0, 0);
	return cbox;
}

typedef const gchar *(*AddComboFn)(int);

static GtkWidget *add_combo_box(GtkTable *tbox, int top, const gchar *caption, 
	AddComboFn fn, int start, int end, const gchar *tooltip)
{
	const gchar *str;
	GtkWidget *cbox;

	gtk_table_attach(tbox, new_label(caption), 0, 1, top, top+1, GTK_FILL, GTK_FILL, 0, 0);
	cbox = gtk_combo_box_new_text();
	for (int i=start; i<end && (str=fn(i))!=NULL; i++) 
		gtk_combo_box_append_text(GTK_COMBO_BOX(cbox), str);
	gtk_widget_set_tooltip_text(cbox, tooltip);
	gtk_table_attach(tbox, cbox, 1, 2, top, top+1, GTK_FILL, GTK_FILL, 0, 0);
	return cbox;
}

static GtkWidget *add_radio_btn(GtkTable *tbox, int top, const gchar *caption, 
	GSList **group, const gchar *tooltip)
{
	GtkWidget *btn = gtk_radio_button_new_with_label(*group, caption);
	gtk_widget_set_tooltip_text(btn, tooltip);
	gtk_table_attach(tbox, btn, 0, 3, top, top+1, GTK_FILL, GTK_FILL, 0, 0);
	if (*group == NULL)
		*group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(btn));
	return btn;
}

static GtkWidget *add_check_btn(GtkTable *tbox, int top, const gchar *caption, const gchar *tooltip)
{
	GtkWidget *btn = gtk_check_button_new_with_label(caption);
	gtk_widget_set_tooltip_text(btn, tooltip);
	gtk_table_attach(tbox, btn, 0, 3, top, top+1, GTK_FILL, GTK_FILL, 0, 0);
	return btn;
}

static GtkWidget *add_button_box(GtkTable *tbox, int top)
{
	GtkWidget *bbox = gtk_hbutton_box_new();
	gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 8);
	gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_START);
	gtk_table_attach(tbox, bbox, 0, 2, top, top+1, GTK_FILL, GTK_FILL, 0, 0);
	return bbox;
}

static int binning_to_index(int binning)
{
	for (int i=0; BinningStrings[i].str!=NULL; i++) {
		if (binning == BinningStrings[i].id)
			return i;
	}
	return -1;
}

static int index_to_binning(int index)
{
	if (index>=0)
		return BinningStrings[index].id;
	return 0;
}

static int bitpix_to_index(int bitpix)
{
	for (int i=0; BitpixStrings[i].str!=NULL; i++) {
		if (bitpix == BitpixStrings[i].id)
			return i;
	}
	return 0;
}

static int index_to_bitpix(int index)
{
	if (index>=0)
		return BitpixStrings[index].id;
	return -1;
}

static int type_to_index(tProjectType type)
{
	return (type - PROJECT_REDUCE);
}

static tProjectType index_to_type(int index)
{
	if (index>=0)
		return (tProjectType)index;
	return PROJECT_INVALID;
}

//-------------------------   CONFIGURATION DIALOG   --------------------------------

CProjectDlg::CProjectDlg(GtkWindow *pParent):m_CurrentPage(NULL), m_Updating(false)
{
	char buf[FILENAME_MAX+32];
	int imin, imax;
	double dmin, dmax;
	GtkWidget *hbox_list_page, *tbox, *list_frame, *tbox2, *bbox;
	GtkWidget *vbox_page;
	GtkTreeViewColumn *col;
	GtkCellRenderer *renderer;
	GtkTreeSelection *selection;
	GtkRequisition sreq;
	GdkRectangle rc;
	GSList *group;
	GSList *stack = g_slist_prepend(NULL, new tStackItem(-1, NULL));

	m_PageId = CConfig::GetInt("ProjectDlg", "LastPage", PAGE_CAMERA);

	// Dialog with buttons
	m_pDlg = gtk_dialog_new_with_buttons("Project settings", pParent, 
		(GtkDialogFlags)(GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR),
		GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT, GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, 
		GTK_STOCK_HELP, GTK_RESPONSE_HELP, NULL);
	gtk_dialog_widget_standard_tooltips(GTK_DIALOG(m_pDlg));
	g_signal_connect(G_OBJECT(m_pDlg), "response", G_CALLBACK(response_dialog), this);

	// Dialog icon
	gchar *icon = get_icon_file("preferences");
	gtk_window_set_icon(GTK_WINDOW(m_pDlg), gdk_pixbuf_new_from_file(icon, NULL));
	g_free(icon);

	// Dialog layout
	hbox_list_page = gtk_hbox_new(FALSE, 4);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(m_pDlg)->vbox), hbox_list_page, TRUE, TRUE, 0);

	// List of categories
	m_PageList = gtk_tree_store_new(4, G_TYPE_INT, G_TYPE_STRING, GTK_TYPE_WIDGET, GDK_TYPE_PIXBUF);
	m_FilteredPageList = gtk_tree_model_filter_new(GTK_TREE_MODEL(m_PageList), NULL);
	gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(m_FilteredPageList), 
		(GtkTreeModelFilterVisibleFunc)page_visible, this, NULL);
	m_PageView = gtk_tree_view_new_with_model(m_FilteredPageList);
	col = gtk_tree_view_column_new();
	gtk_tree_view_column_set_resizable(col, FALSE);
	gtk_tree_view_append_column(GTK_TREE_VIEW(m_PageView), col);
	renderer = gtk_cell_renderer_pixbuf_new();
	gtk_tree_view_column_pack_start(col, renderer, FALSE);
	gtk_tree_view_column_add_attribute(col, renderer, "pixbuf", 3);
	renderer = gtk_cell_renderer_text_new();
	gtk_tree_view_column_pack_start(col, renderer, TRUE);
	gtk_tree_view_column_add_attribute(col, renderer, "text", 1);
	gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(m_PageView)), GTK_SELECTION_BROWSE);
	list_frame = gtk_frame_new(NULL);
	gtk_frame_set_shadow_type(GTK_FRAME(list_frame), GTK_SHADOW_IN);
	gtk_container_add(GTK_CONTAINER(list_frame), m_PageView);
	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(m_PageView), FALSE);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_PageView));
	g_signal_connect(G_OBJECT(selection), "changed", G_CALLBACK(selection_changed), this);
	gtk_box_pack_start(GTK_BOX(hbox_list_page), list_frame, FALSE, TRUE, 0);
	gtk_widget_set_size_request(list_frame, 200, -1);

	m_PageScroller = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(m_PageScroller), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_box_pack_start(GTK_BOX(hbox_list_page), m_PageScroller, TRUE, TRUE, 0);
	vbox_page = gtk_vbox_new(FALSE, 0);
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(m_PageScroller), vbox_page);

    m_TitleFrame = gtk_frame_new(NULL);
    gtk_frame_set_shadow_type(GTK_FRAME(m_TitleFrame), GTK_SHADOW_NONE);
    gtk_box_pack_start(GTK_BOX(vbox_page), m_TitleFrame, FALSE, FALSE, 0);
	m_PageTitle = gtk_label_new(NULL);
    gtk_container_add(GTK_CONTAINER(m_TitleFrame), m_PageTitle);

	GtkWidget *hsep = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(vbox_page), hsep, FALSE, FALSE, 4);
	
	m_PageFrame = gtk_frame_new(NULL);
    gtk_box_pack_start(GTK_BOX(vbox_page), m_PageFrame, TRUE, TRUE, 0);
    gtk_frame_set_shadow_type(GTK_FRAME(m_PageFrame), GTK_SHADOW_NONE);

	// Project
	sprintf(buf, "Project '%s'", g_Project->Name());
	tbox = CreatePage(PAGE_PROJECT, 0, buf, NULL, FALSE, &stack);
	add_label(GTK_TABLE(tbox), 0, "<b>General properties</b>");
	m_Type = add_combo_box(GTK_TABLE(tbox), 1, "Type", (AddComboFn)ProjectTypeCaption, 
		PROJECT_REDUCE, EndOfProjectTypes, "Select a project type according to the type of source frames and a kind of output data");
	g_signal_connect(G_OBJECT(m_Type), "changed", G_CALLBACK(combo_box_changed), this);
	m_Name = add_entry(GTK_TABLE(tbox), "Name", 2, MAX_PROJECT_LEN, "Name of the project, it must consist of characters allowed in a file name only");
	m_Path = add_static(GTK_TABLE(tbox), "Path", 3, PANGO_ELLIPSIZE_MIDDLE, "Path to the directory with the project files");
	gtk_widget_set_size_request(m_Name, 320, -1);

	add_label(GTK_TABLE(tbox), 5, "<b>Import/export project settings</b>");
	bbox = add_button_box(GTK_TABLE(tbox), 6);
	m_LoadProfile = gtk_button_new_with_label("Load from profile");
	gtk_widget_set_tooltip_text(m_LoadProfile, "Restore project settings from a file");
	gtk_container_add(GTK_CONTAINER(bbox), m_LoadProfile);
	g_signal_connect(G_OBJECT(m_LoadProfile), "clicked", G_CALLBACK(button_clicked), this);
	m_LoadDefaults = gtk_button_new_with_label("Load factory defaults");
	gtk_widget_set_tooltip_text(m_LoadDefaults, "Set parameters on all pages to the default values");
	gtk_container_add(GTK_CONTAINER(bbox), m_LoadDefaults);
	g_signal_connect(G_OBJECT(m_LoadDefaults), "clicked", G_CALLBACK(button_clicked), this);
	m_SaveProfile = gtk_button_new_with_label("Save to profile");
	gtk_widget_set_tooltip_text(m_SaveProfile, "Save project settings to a file");
	gtk_container_add(GTK_CONTAINER(bbox), m_SaveProfile);
	g_signal_connect(G_OBJECT(m_SaveProfile), "clicked", G_CALLBACK(button_clicked), this);

	// Camera
	tbox = CreatePage(PAGE_CAMERA, 1, "Camera", NULL, TRUE, &stack);
	m_RNoise = add_spin_button_dbl(GTK_TABLE(tbox), 0, "Readout noise", 
		CProfile::READ_NOISE, 0.1, 2, "Readout noise in electrons");
	m_ADCGain = add_spin_button_dbl(GTK_TABLE(tbox), 1, "ADC gain", 
		CProfile::ADC_GAIN, 0.1, 6, "A/D converter gain in electrons per ADU");
	
	// Source frames
	tbox = CreatePage(PAGE_SOURCE_FILES, 1, "Source frames", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "<b>Pixel value range</b>");
	tbox2 = gtk_table_new(4, 3, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(tbox2), 8);
	gtk_table_set_row_spacings(GTK_TABLE(tbox2), 8);
	gtk_table_attach(GTK_TABLE(tbox), tbox2, 0, 3, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	add_label(GTK_TABLE(tbox2), 0, "Bad pixels have value equal to or less than:");
	m_DataMin = add_spin_button_dbl(GTK_TABLE(tbox2), 1, "Min. pixel value", 
		CProfile::BAD_PIXEL_VALUE, 1, 0, "Pixel value that indicates bad pixel (usually 0)");
	add_label(GTK_TABLE(tbox2), 2, "Overexposed pixels have value equal to or greater than:");
	m_DataMax = add_spin_button_dbl(GTK_TABLE(tbox2), 3, "Max. pixel value", 
		CProfile::OVEREXPOSED_VALUE, 1, 0, "Pixel value that indicates overexposed value");
	add_label(GTK_TABLE(tbox), 3, "<b>Transformations</b>");
	tbox2 = gtk_table_new(4, 3, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(tbox2), 8);
	gtk_table_set_row_spacings(GTK_TABLE(tbox2), 8);
	gtk_table_attach(GTK_TABLE(tbox), tbox2, 0, 3, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
	add_label(GTK_TABLE(tbox2), 0, "Convert frames to the following pixel data format:");
	m_WorkFormat = add_combo_box(GTK_TABLE(tbox2), 1, "Image data format", BitpixStrings,
		"Image data format used for internal data representation and storage.");
	add_label(GTK_TABLE(tbox2), 2, "Sum pixel values from neighboring pixels:");
	m_Binning = add_combo_box(GTK_TABLE(tbox2), 3, "Binning", BinningStrings,
		"Number of columns and rows summed together");

	add_label(GTK_TABLE(tbox), 6, "<b>Borders</b>");
	tbox2 = gtk_table_new(2, 6, FALSE);
	gtk_table_attach(GTK_TABLE(tbox), tbox2, 0, 3, 7, 8, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_set_col_spacings(GTK_TABLE(tbox2), 8);
	gtk_table_set_row_spacings(GTK_TABLE(tbox2), 8);
	gtk_table_attach(GTK_TABLE(tbox2), new_label("Skip pixels that are close to the frame border:"), 0, 7, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach(GTK_TABLE(tbox2), new_label("Border"), 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	CProfile::GetBorderSizeLimits(&imin, &imax);
	gtk_table_attach(GTK_TABLE(tbox2), new_label("- left", 1), 1, 2, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	m_BLeft = add_spin_button(GTK_TABLE(tbox2), 2, 1, imin, imax, 1, 0, "Number of unused pixels on the left side of each frame");
	gtk_table_attach(GTK_TABLE(tbox2), new_label("- right", 1), 1, 2, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	m_BRight = add_spin_button(GTK_TABLE(tbox2), 2, 2, imin, imax, 1, 0, "Number of unused pixels on the right side of each frame");
	gtk_table_attach(GTK_TABLE(tbox2), new_label("- top", 1), 3, 4, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
	m_BTop = add_spin_button(GTK_TABLE(tbox2), 4, 1, imin, imax, 1, 0, "Number of unused pixels at the top of each frame");
	gtk_table_attach(GTK_TABLE(tbox2), new_label("- bottom", 1), 3, 4, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	m_BBottom = add_spin_button(GTK_TABLE(tbox2), 4, 2, imin, imax, 1, 0, "Number of unused pixels at the bottom of each frame");
	gtk_table_attach(GTK_TABLE(tbox2), new_label("", TRUE), 6, 7, 1, 2, 
		GtkAttachOptions(GTK_FILL | GTK_EXPAND), GTK_FILL, 0, 0);
		
	// Calibration
	tbox = CreatePage(PAGE_CALIBRATION, 1, "Calibration", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "<b>Calibration scheme</b>");
	group = NULL;
	m_StdCalibration = add_radio_btn(GTK_TABLE(tbox), 1, "Standard (dark + flat)", &group, 
		"Use standard calibration scheme");
	m_AdvCalibration = add_radio_btn(GTK_TABLE(tbox), 2, "Advanced (bias + scalable dark + flat)", &group,
		"Use advanced calibration scheme");

	// Star detection
	tbox = CreatePage(PAGE_STAR_DETECTION, 1, "Star detection", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "<b>Gaussian filter</b>");
	m_FWHM = add_spin_button_dbl(GTK_TABLE(tbox), 1, "Filter width (FWHM)", 
		CProfile::DETECTION_FWHM, 0.1, 2, "Expected average FWHM of objects.");
	add_label(GTK_TABLE(tbox), 3, "<b>Minimum brightness</b>");
	m_Thresh = add_spin_button_dbl(GTK_TABLE(tbox), 4, "Detection threshold", 
		CProfile::DETECTION_THRESHOLD, 0.1, 2, "Minimum signal from an object, filters background artifacts");
	add_label(GTK_TABLE(tbox), 6, "<b>Sharpness limits</b>");
	m_SharpMin = add_spin_button_dbl(GTK_TABLE(tbox), 7, "Minimum sharpness", 
		CProfile::MIN_SHARPNESS, 0.1, 2, "Minimum sharpness of an object, filters out cosmic particles, for example");
	m_SharpMax = add_spin_button_dbl(GTK_TABLE(tbox), 8, "Maximum sharpness", 
		CProfile::MAX_SHARPNESS, 0.1, 2, "Maximum sharpness of an object, filters out nebulous objects");
	add_label(GTK_TABLE(tbox), 10, "<b>Roundness limits</b>");
	m_RoundMin = add_spin_button_dbl(GTK_TABLE(tbox), 11, "Minimum roundness", 
		CProfile::MIN_ROUNDNESS, 0.1, 2, "Minimum roundness of an object, filters out elongated objects");
	m_RoundMax = add_spin_button_dbl(GTK_TABLE(tbox), 12, "Maximum roundness", 
		CProfile::MAX_ROUNDNESS, 0.1, 2, "Maximum roundness of an object, filters out elongated objects");

	// Photometry
	tbox = CreatePage(PAGE_PHOTOMETRY, 1, "Photometry", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "<b>Apertures</b>");
	add_label(GTK_TABLE(tbox), 1, "Radii of the apertures (1-12) for object brightness measurement (pixels):");
	tbox2 = gtk_table_new(4, 6, FALSE);
	gtk_table_set_col_spacings(GTK_TABLE(tbox2), 8);
	gtk_table_set_row_spacings(GTK_TABLE(tbox2), 8);
	gtk_table_attach(GTK_TABLE(tbox), tbox2, 0, 3, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	gtk_table_attach(GTK_TABLE(tbox2), new_label("Aperture"), 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
	CProfile::GetApertureSizeLimits(&dmin, &dmax);
	for (int i=0; i<MAX_APERTURES; i++) {
		int row = (i % 4), col = (i / 4);
		sprintf(buf, "#%d", i+1);
		gtk_table_attach(GTK_TABLE(tbox2), new_label(buf, 1.0), col*2+1, col*2+2, row, row+1, GTK_FILL, GTK_FILL, 0, 0);
		m_Aperture[i] = add_spin_button(GTK_TABLE(tbox2), col*2+2, row, 0, dmax, 0.1, 2, "Aperture radius in pixels");
	}
	add_label(GTK_TABLE(tbox), 4, "<b>Background</b>");
	add_label(GTK_TABLE(tbox), 5, "Size of the annulus for background level measurement (pixels):");
	m_SkyIn = add_spin_button_dbl(GTK_TABLE(tbox), 6, "Inner radius", 
		CProfile::SKY_INNER_RADIUS, 0.1, 2, "Inner radius of the annulus used to measure local background");
	m_SkyOut = add_spin_button_dbl(GTK_TABLE(tbox), 7, "Outer radius", 
		CProfile::SKY_OUTER_RADIUS, 0.1, 2, "Outer radius of the annulus used to measure local background");
	
	// Matching
	tbox = CreatePage(PAGE_MATCHING, 1, "Matching", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "<b>Matching algorithm</b>");
	group = NULL;
	m_MatchStandard = add_radio_btn(GTK_TABLE(tbox), 1, "Standard algorithm (requires at least 3 stars)", &group,
		"The standard algorithm is robust, each frame must contain at least 3 objects");
	g_signal_connect(G_OBJECT(m_MatchStandard), "toggled", G_CALLBACK(button_toggled), this);
	m_MatchSparse = add_radio_btn(GTK_TABLE(tbox), 2, "Algorithm for sparse fields (2 stars or less)", &group,
		"The algorithm for sparse fields works also for frames that consists a single object only");
	g_signal_connect(G_OBJECT(m_MatchSparse), "toggled", G_CALLBACK(button_toggled), this);
	m_MatchBox1 = gtk_table_new(3, 3, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(m_MatchBox1), 8);
	gtk_table_set_col_spacings(GTK_TABLE(m_MatchBox1), 8);
	gtk_table_attach(GTK_TABLE(tbox), m_MatchBox1, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
	add_label(GTK_TABLE(m_MatchBox1), 0, "<b>Standard matching parameters</b>");
	add_label(GTK_TABLE(m_MatchBox1), 1, "Read from the source file no more than");
	m_MatchRS = add_spin_button_int(GTK_TABLE(m_MatchBox1), 2, "Read stars", 
		CProfile::MATCH_READ_STARS, 1, 0, "Number of objects that are used for finding transformation");
	add_label(GTK_TABLE(m_MatchBox1), 3, "Identify in each iteration at least");
	m_MatchIS = add_spin_button_int(GTK_TABLE(m_MatchBox1), 4, "Identification stars", 
		CProfile::MATCH_IDENT_STARS, 1, 0, "Number of polygon vertices");
	add_label(GTK_TABLE(m_MatchBox1), 5, "Maximum misalignment of objects");
	m_MatchClip = add_spin_button_dbl(GTK_TABLE(m_MatchBox1), 6, "Clipping factor", 
		CProfile::MATCH_CLIP, 0.1, 1, "Sensitivity to the noise in position measurements");
	m_MatchBox2 = gtk_table_new(1, 3, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(m_MatchBox2), 8);
	gtk_table_set_col_spacings(GTK_TABLE(m_MatchBox2), 8);
	gtk_table_attach(GTK_TABLE(tbox), m_MatchBox2, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
	add_label(GTK_TABLE(m_MatchBox2), 0, "<b>Sparse fields matching parameters</b>");
	add_label(GTK_TABLE(m_MatchBox2), 1, "Highest allowed offset of the source and reference frame");
	m_MatchMax = add_spin_button_dbl(GTK_TABLE(m_MatchBox2), 2, "Max. offset (pixels)", 
		CProfile::MAX_OFFSET, 1, 0, "Maximum displacement between the source and the reference frame in pixels");

	// Light curve
	tbox = CreatePage(PAGE_LIGHT_CURVE, 1, "Light curve", NULL, TRUE, &stack);
	m_ArtCompStar = add_check_btn(GTK_TABLE(tbox), 0, "Allow using an artificial comparison star",
		"Combine multiple comparison stars");

	// Master bias
	tbox = CreatePage(PAGE_MASTER_BIAS, 1, "Master bias", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "Save output frame in the following pixel data format:");
	m_MBiasFormat = add_combo_box(GTK_TABLE(tbox), 1, "Output data format", BitpixStrings,
		"Target data format");

	// Master dark
	tbox = CreatePage(PAGE_MASTER_DARK, 1, "Master dark", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "Save output frame in the following pixel data format:");
	m_MDarkFormat = add_combo_box(GTK_TABLE(tbox), 1, "Output data format", BitpixStrings,
		"Target data format");

	// Master flat
	tbox = CreatePage(PAGE_MASTER_FLAT, 1, "Master flat", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "Save output frame in the following pixel data format:");
	m_MFlatFormat = add_combo_box(GTK_TABLE(tbox), 1, "Output data format", BitpixStrings,
		"Target data format");
	add_label(GTK_TABLE(tbox), 2, "Pixel values of output frame are normalized to the mean level:");
	m_MFlatLevel = add_spin_button_dbl(GTK_TABLE(tbox), 3, "Output mean level", 
		CProfile::MFLAT_LEVEL, 1, 0, "A master flat frame is normalized to the specified value in ADU");

	// Kombine
	tbox = CreatePage(PAGE_MERGE_FRAMES, 1, "Merge frames", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "Save output frames in the following pixel data format:");
	m_KombineFormat = add_combo_box(GTK_TABLE(tbox), 1, "Output data format", BitpixStrings,
		"Target data format");

	// Find variables
	tbox = CreatePage(PAGE_FIND_VARIABLES, 1, "Find variables", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "Do not report objects that have share of valid measurements less than:");
	m_VarFindThreshold = add_spin_button_dbl(GTK_TABLE(tbox), 1, "Clipping threshold", 
		CProfile::VARFIND_THRESHOLD, 1, 0, "Filter threshold in %, it rules out objects that don't have enough valid measurements");

	// Observer
	tbox = CreatePage(PAGE_OBSERVER, 1, "Observer", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "Observer's default location:");
	m_LocName = add_entry(GTK_TABLE(tbox), "Location", 1, MAX_LOCATION_LEN, "Observer's location designation, e.g. Brno, Czech Republic");
	m_LocBtn = add_push_button(GTK_TABLE(tbox), 1, "More", "Load observer's coordinates from presets");
	g_signal_connect(G_OBJECT(m_LocBtn), "clicked", G_CALLBACK(button_clicked), this);
	m_Lon = add_entry(GTK_TABLE(tbox), "Longitude", 2, 32, "Observer's longitude in degrees, minutes and seconds, use E or W suffix to indicate hemisphere, e.g. 12 34 56 E");
	gtk_table_attach(GTK_TABLE(tbox), new_label("[d m s E/W]"), 2, 3, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
	m_Lat = add_entry(GTK_TABLE(tbox), "Latitude", 3, 32, "Observer's latitude in degrees, minutes and seconds, use N or S suffix to indicate hemisphere, e.g. 56 34 12 N");
	gtk_table_attach(GTK_TABLE(tbox), new_label("[d m s N/S]"), 2, 3, 3, 4, GTK_FILL, GTK_FILL, 0, 0);

	// Files
	tbox = CreatePage(PAGE_FILES, 1, "Files and directories", NULL, TRUE, &stack);
	add_label(GTK_TABLE(tbox), 0, "<b>Calibration</b>");
	m_Bias = add_static(GTK_TABLE(tbox), "Bias frame", 1, PANGO_ELLIPSIZE_MIDDLE, "Path to the most recently used bias frame");
	m_Dark = add_static(GTK_TABLE(tbox), "Dark frame", 2, PANGO_ELLIPSIZE_MIDDLE, "Path to the most recently used dark frame");
	m_Flat = add_static(GTK_TABLE(tbox), "Flat frame", 3, PANGO_ELLIPSIZE_MIDDLE, "Path to the most recently used flat frame");
	add_label(GTK_TABLE(tbox), 5, "<b>Matching</b>");
	m_Ref = add_static(GTK_TABLE(tbox), "Reference frame", 6, PANGO_ELLIPSIZE_NONE, "Identification of the reference frame used for matching");
	m_Cat = add_static(GTK_TABLE(tbox), "Catalogue file", 7, PANGO_ELLIPSIZE_MIDDLE, "Path to the catalog file used for matching");
	gtk_widget_set_size_request(m_Bias, 280, -1);
	
	// Dialog size
	m_DlgWidth = 320; m_DlgHeight = 240;
    gtk_tree_model_foreach(GTK_TREE_MODEL(m_PageList), SetMaxDialogSize, this);
	gtk_widget_show_all(m_TitleFrame);
	gtk_widget_size_request(m_TitleFrame, &sreq);
	m_DlgHeight += sreq.height;
	gtk_widget_size_request(hsep, &sreq);
	m_DlgHeight += sreq.height;
	gtk_widget_size_request(m_PageView, &sreq);
	m_DlgWidth += sreq.width + 32;
	gtk_widget_size_request(GTK_DIALOG(m_pDlg)->action_area, &sreq);
	m_DlgHeight += sreq.height + 56;
	GdkScreen *scr = gdk_screen_get_default();
	gdk_screen_get_monitor_geometry(scr, 0, &rc);
	if (rc.width>0 && rc.height>0) {
		int maxw = RoundToInt(0.9*rc.width);
		if (m_DlgWidth > maxw)
			m_DlgWidth = maxw;
		int maxh = RoundToInt(0.8*rc.height);
		if (m_DlgHeight > maxh)
			m_DlgHeight = maxh;
	}
	gtk_window_set_default_size(GTK_WINDOW(m_pDlg), m_DlgWidth, m_DlgHeight);

	// Free remaining items on stack
	g_slist_foreach(stack->next, (GFunc)tStackItem::Release, NULL);
	g_slist_free(stack);

	// All nodes expanded
	gtk_tree_view_expand_all(GTK_TREE_VIEW(m_PageView));

	gtk_widget_show_all(GTK_DIALOG(m_pDlg)->vbox);
}

CProjectDlg::~CProjectDlg()
{
	gtk_widget_destroy(m_pDlg);
	g_object_unref(m_PageList);
	g_object_unref(m_FilteredPageList);
}

gboolean CProjectDlg::SetMaxDialogSize(GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer data)
{
	CProjectDlg *pMe = (CProjectDlg*)data;
	GtkWidget *page = NULL;
	GtkRequisition sreq;

	gtk_tree_model_get(model, iter, 2, &page, -1);
	gtk_widget_show_all(page);
    gtk_widget_size_request(page, &sreq);
	if (sreq.width > pMe->m_DlgWidth)
		pMe->m_DlgWidth = sreq.width;
	if (sreq.height > pMe->m_DlgHeight)
		pMe->m_DlgHeight = sreq.height;
    return FALSE;
}

GtkWidget *CProjectDlg::CreatePage(int id, int indent, const gchar *caption, const gchar *icon, 
	bool def_btn, GSList **stack)
{
	GtkTreeIter iter, parent;

	while ((*stack)!=NULL && indent <= ((tStackItem*)(*stack)->data)->level) {
		GSList *next = (*stack)->next;
		delete (tStackItem*)(*stack)->data;
		g_slist_free_1(*stack);
		(*stack) = next;
	}
	g_assert((*stack)!=NULL && (*stack)->data!=NULL);

	GtkWidget *box = gtk_vbox_new(FALSE, 8);
	gtk_container_set_border_width(GTK_CONTAINER(box), 4);
	if (((tStackItem*)(*stack)->data)->item!=NULL) {
		gtk_tree_model_get_iter(GTK_TREE_MODEL(m_PageList), &parent, ((tStackItem*)(*stack)->data)->item);
		gtk_tree_store_append(m_PageList, &iter, &parent);
	} else {
		gtk_tree_store_append(m_PageList, &iter, NULL);
	}

	GtkTreePath *path = gtk_tree_model_get_path(GTK_TREE_MODEL(m_PageList), &iter);
	*stack = g_slist_prepend(*stack, new tStackItem(indent, path));

	gtk_tree_store_set(m_PageList, &iter, 0, id, 1, caption, 2, box, -1);
	if (icon) {
		char *fpath = get_icon_file(icon);
		GdkPixbuf *i = gdk_pixbuf_new_from_file(fpath, NULL);
		gtk_tree_store_set(m_PageList, &iter, 3, i, -1);
		g_free(fpath);
	}
	if (def_btn && !g_Project->isReadOnly()) {
		GtkWidget *btnbox = gtk_hbutton_box_new();
		gtk_button_box_set_layout(GTK_BUTTON_BOX(btnbox), GTK_BUTTONBOX_START);
		gtk_box_pack_end(GTK_BOX(box), btnbox, FALSE, FALSE, 0);
		GtkWidget *btn = gtk_button_new_with_label("Set defaults");
		gtk_widget_set_tooltip_text(btn, "Set parameters on the actual page to the default values");
		g_signal_connect(G_OBJECT(btn), "clicked", G_CALLBACK(setdefaults_clicked), this);
		gtk_container_add(GTK_CONTAINER(btnbox), btn);
	}
	GtkWidget *table = gtk_table_new(0, 0, FALSE);
	gtk_table_set_row_spacings(GTK_TABLE(table), 8);
	gtk_table_set_col_spacings(GTK_TABLE(table), 8);
	gtk_box_pack_start(GTK_BOX(box), table, FALSE, TRUE, 0);
	return table;
}

gboolean CProjectDlg::page_visible(GtkTreeModel *model, GtkTreeIter *iter, CProjectDlg *pMe)
{
	int id;

	gtk_tree_model_get(model, iter, 0, &id, -1);
	switch (pMe->m_ProjectType)
	{
	case PROJECT_REDUCE:
		if (id==PAGE_MASTER_BIAS || id==PAGE_MASTER_DARK || id==PAGE_MASTER_FLAT || 
			id==PAGE_MERGE_FRAMES)
			return false;
		break;

	case PROJECT_COMBINING:
		if (id==PAGE_MASTER_BIAS || id==PAGE_MASTER_DARK || id==PAGE_MASTER_FLAT)
			return FALSE;
		break;

	case PROJECT_MASTER_BIAS:
		if (id==PAGE_STAR_DETECTION || id==PAGE_PHOTOMETRY || id==PAGE_MATCHING || 
			id==PAGE_LIGHT_CURVE || id==PAGE_MERGE_FRAMES || id==PAGE_MASTER_DARK || 
			id==PAGE_MASTER_FLAT || id==PAGE_FIND_VARIABLES	|| id==PAGE_OBSERVER ||
			id==PAGE_CAMERA || id==PAGE_CALIBRATION)
			return FALSE;
		break;

	case PROJECT_MASTER_DARK:
		if (id==PAGE_STAR_DETECTION || id==PAGE_PHOTOMETRY || id==PAGE_MATCHING || 
			id==PAGE_LIGHT_CURVE || id==PAGE_MERGE_FRAMES || id==PAGE_MASTER_BIAS || 
			id==PAGE_MASTER_FLAT || id==PAGE_FIND_VARIABLES	|| id==PAGE_OBSERVER ||
			id==PAGE_CAMERA)
			return FALSE;
		break;

	case PROJECT_MASTER_FLAT:
		if (id==PAGE_STAR_DETECTION || id==PAGE_PHOTOMETRY || id==PAGE_MATCHING || 
			id==PAGE_LIGHT_CURVE || id==PAGE_MERGE_FRAMES || id==PAGE_MASTER_BIAS || 
			id==PAGE_MASTER_DARK || id==PAGE_FIND_VARIABLES	|| id==PAGE_OBSERVER ||
			id==PAGE_CAMERA || id==PAGE_CALIBRATION)
			return FALSE;
		break;

	default:
		break;
	}
	return TRUE;
}

bool CProjectDlg::Execute(tSectionId section)
{
	// Initialize the dialog
	m_Profile = *g_Project->Profile();
	m_ProjectType = m_Profile.ProjectType();
	gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(m_FilteredPageList));
	SetData();
	ShowPage((section==PAGE_DEFAULT ? m_PageId : section));
	UpdateControls();

	// Execute the dialog
	if (gtk_dialog_run(GTK_DIALOG(m_pDlg))!=GTK_RESPONSE_ACCEPT) 
		return false;

	// Apply changes
	g_Project->SetProfile(m_Profile);
	g_Project->Save();
	g_Project->ClearThumbnails();
	return true;
}

void CProjectDlg::response_dialog(GtkWidget *pDlg, gint response_id, CProjectDlg *pMe)
{
	if (!pMe->OnResponseDialog(response_id)) 
		g_signal_stop_emission_by_name(pDlg, "response");
}

bool CProjectDlg::OnResponseDialog(gint response_id)
{
	switch (response_id)
	{
	case GTK_RESPONSE_ACCEPT:
		// Commit changes
		return OnCloseQuery();

	case GTK_RESPONSE_HELP:
		// Show context help
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_PROJECT);
		return false;
	}
	return true;
}

void CProjectDlg::SetData(void)
{
	char buf[256];
	bool editable = !g_Project->isReadOnly();

	// General
	gtk_entry_set_text(GTK_ENTRY(m_Name), g_Project->Name());
	if (g_Project->Path()) {
		gchar *dirname = g_path_get_dirname(g_Project->Path());
		gtk_label_set_text(GTK_LABEL(m_Path), dirname);
		g_free(dirname);
	} else 
		gtk_label_set_text(GTK_LABEL(m_Path), "------");
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_Type), 
		type_to_index(m_ProjectType));
	gtk_widget_set_sensitive(GTK_WIDGET(m_Name), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_Type), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_LoadProfile), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_LoadDefaults), editable);
	
	// Camera
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RNoise), 
		m_Profile.GetDbl(CProfile::READ_NOISE));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_ADCGain), 
		m_Profile.GetDbl(CProfile::ADC_GAIN));
	gtk_widget_set_sensitive(GTK_WIDGET(m_RNoise), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_ADCGain), editable);

	// Source frames
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_DataMin), 
		m_Profile.GetDbl(CProfile::BAD_PIXEL_VALUE));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_DataMax), 
		m_Profile.GetDbl(CProfile::OVEREXPOSED_VALUE));
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_WorkFormat), bitpix_to_index(
		m_Profile.GetInt(CProfile::WORK_FORMAT)));
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_Binning), binning_to_index(
		m_Profile.GetInt(CProfile::BINNING)));
	gtk_widget_set_sensitive(GTK_WIDGET(m_DataMin), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_DataMax), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_WorkFormat), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_Binning), editable);
	CmpackBorder border(m_Profile.GetBorder());
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_BLeft), border.left);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_BRight), border.right);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_BTop), border.top);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_BBottom), border.bottom);
	gtk_widget_set_sensitive(GTK_WIDGET(m_BLeft), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_BRight), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_BTop), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_BBottom), editable);

	// Calibration
	bool adv_calib = m_Profile.GetBool(CProfile::ADVANCED_CALIBRATION);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_StdCalibration), !adv_calib);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_AdvCalibration), adv_calib);
	gtk_widget_set_sensitive(GTK_WIDGET(m_StdCalibration), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_AdvCalibration), editable);

	// Star detection
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_FWHM), 
		m_Profile.GetDbl(CProfile::DETECTION_FWHM));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_Thresh), 
		m_Profile.GetDbl(CProfile::DETECTION_THRESHOLD));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_SharpMin), 
		m_Profile.GetDbl(CProfile::MIN_SHARPNESS));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_SharpMax), 
		m_Profile.GetDbl(CProfile::MAX_SHARPNESS));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RoundMin), 
		m_Profile.GetDbl(CProfile::MIN_ROUNDNESS));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RoundMax), 
		m_Profile.GetDbl(CProfile::MAX_ROUNDNESS));
	gtk_widget_set_sensitive(GTK_WIDGET(m_FWHM), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_Thresh), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_SharpMin), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_SharpMax), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_RoundMin), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_RoundMax), editable);

	// Photometry
	CApertures aper = m_Profile.Apertures();
	for (int i=0; i<MAX_APERTURES; i++) {
		int index = aper.Find(i+1);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_Aperture[i]), aper.GetRadius(index));
		gtk_widget_set_sensitive(GTK_WIDGET(m_Aperture[i]), editable);
	}
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_SkyIn), 
		m_Profile.GetDbl(CProfile::SKY_INNER_RADIUS));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_SkyOut), 
		m_Profile.GetDbl(CProfile::SKY_OUTER_RADIUS));
	gtk_widget_set_sensitive(GTK_WIDGET(m_SkyIn), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_SkyOut), editable);

	// Matching
	bool sparse_fields = m_Profile.GetBool(CProfile::SPARSE_FIELDS);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_MatchStandard), 
		!sparse_fields);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_MatchSparse), 
		sparse_fields);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_MatchRS), 
		m_Profile.GetInt(CProfile::MATCH_READ_STARS));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_MatchIS), 
		m_Profile.GetInt(CProfile::MATCH_IDENT_STARS));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_MatchClip), 
		m_Profile.GetDbl(CProfile::MATCH_CLIP));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_MatchMax), 
		m_Profile.GetDbl(CProfile::MAX_OFFSET));
	gtk_widget_set_sensitive(GTK_WIDGET(m_MatchStandard), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_MatchSparse), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_MatchRS), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_MatchIS), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_MatchClip), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_MatchMax), editable);

	// Light curve
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ArtCompStar), 
		m_Profile.GetBool(CProfile::MULTIPLE_COMP_STARS));
	gtk_widget_set_sensitive(GTK_WIDGET(m_ArtCompStar), editable);

	// Master bias
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_MBiasFormat), bitpix_to_index(
		m_Profile.GetInt(CProfile::MBIAS_FORMAT)));
	gtk_widget_set_sensitive(GTK_WIDGET(m_MBiasFormat), editable);

	// Master dark
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_MDarkFormat), bitpix_to_index(
		m_Profile.GetInt(CProfile::MDARK_FORMAT)));
	gtk_widget_set_sensitive(GTK_WIDGET(m_MDarkFormat), editable);

	// Master flat
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_MFlatLevel), 
		m_Profile.GetDbl(CProfile::MFLAT_LEVEL));
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_MFlatFormat), bitpix_to_index(
		m_Profile.GetInt(CProfile::MFLAT_FORMAT)));
	gtk_widget_set_sensitive(GTK_WIDGET(m_MFlatLevel), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_MFlatFormat), editable);

	// Kombine
	gtk_combo_box_set_active(GTK_COMBO_BOX(m_KombineFormat), bitpix_to_index(
		m_Profile.GetInt(CProfile::KOMBINE_FORMAT)));
	gtk_widget_set_sensitive(GTK_WIDGET(m_KombineFormat), editable);

	// Find variables
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_VarFindThreshold), 
		m_Profile.GetDbl(CProfile::VARFIND_THRESHOLD));
	gtk_widget_set_sensitive(GTK_WIDGET(m_VarFindThreshold), editable);

	// Observer
	const CLocation obs = m_Profile.Location();
	if (obs.Name())
		gtk_entry_set_text(GTK_ENTRY(m_LocName), obs.Name());
	else
		gtk_entry_set_text(GTK_ENTRY(m_LocName), "");
	if (obs.Lon())
		gtk_entry_set_text(GTK_ENTRY(m_Lon), obs.Lon());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lon), "");
	if (obs.Lat())
		gtk_entry_set_text(GTK_ENTRY(m_Lat), obs.Lat());
	else
		gtk_entry_set_text(GTK_ENTRY(m_Lat), "");
	gtk_widget_set_sensitive(GTK_WIDGET(m_LocName), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_LocBtn), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_Lon), editable);
	gtk_widget_set_sensitive(GTK_WIDGET(m_Lat), editable);
	
	// Files and directories
	if (g_Project->GetOrigBiasFile()->Valid()) {
		gtk_label_set_text(GTK_LABEL(m_Bias), 
			g_Project->GetOrigBiasFile()->FullPath());
	} else 
		gtk_label_set_text(GTK_LABEL(m_Bias), "------");
	if (g_Project->GetOrigDarkFile()->Valid()) {
		gtk_label_set_text(GTK_LABEL(m_Dark), 
			g_Project->GetOrigDarkFile()->FullPath());
	} else
		gtk_label_set_text(GTK_LABEL(m_Dark), "------");
	if (g_Project->GetOrigFlatFile()->Valid()) {
		gtk_label_set_text(GTK_LABEL(m_Flat), 
			g_Project->GetOrigFlatFile()->FullPath());
	} else
		gtk_label_set_text(GTK_LABEL(m_Flat), "------");
	if (g_Project->GetReferenceType()==REF_FRAME &&
		g_Project->GetReferenceFrame()>=0) {
			sprintf(buf, "%d", g_Project->GetReferenceFrame());
			gtk_label_set_text(GTK_LABEL(m_Ref), buf);
	} else 
		gtk_label_set_text(GTK_LABEL(m_Ref), "------");
	if (g_Project->GetReferenceType()==REF_CATALOG_FILE && 
		g_Project->GetReferenceCatalog()->Valid()) {
			gtk_label_set_text(GTK_LABEL(m_Cat), 
				g_Project->GetReferenceCatalog()->FullPath());
	} else 
		gtk_label_set_text(GTK_LABEL(m_Cat), "------");
}

void CProjectDlg::GetData(void)
{
	// Camera
	m_Profile.SetDbl(CProfile::READ_NOISE, 
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_RNoise)));
	m_Profile.SetDbl(CProfile::ADC_GAIN,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_ADCGain)));

	// Source frames
	m_Profile.SetDbl(CProfile::BAD_PIXEL_VALUE,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_DataMin)));
	m_Profile.SetDbl(CProfile::OVEREXPOSED_VALUE,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_DataMax)));
	m_Profile.SetInt(CProfile::WORK_FORMAT, index_to_bitpix(
		gtk_combo_box_get_active(GTK_COMBO_BOX(m_WorkFormat))));
	m_Profile.SetInt(CProfile::BINNING, index_to_binning(
		gtk_combo_box_get_active(GTK_COMBO_BOX(m_Binning))));
	CmpackBorder border;
	border.left = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_BLeft));
	border.right = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_BRight));
	border.top = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_BTop));
	border.bottom = (gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_BBottom));
	m_Profile.SetBorder(border);

	// Calibration
	m_Profile.SetBool(CProfile::ADVANCED_CALIBRATION,
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_AdvCalibration))!=FALSE);

	// Star detection
	m_Profile.SetDbl(CProfile::DETECTION_FWHM,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_FWHM)));
	m_Profile.SetDbl(CProfile::DETECTION_THRESHOLD,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_Thresh)));
	m_Profile.SetDbl(CProfile::MIN_SHARPNESS,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_SharpMin)));
	m_Profile.SetDbl(CProfile::MAX_SHARPNESS,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_SharpMax)));
	m_Profile.SetDbl(CProfile::MIN_ROUNDNESS,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_RoundMin)));
	m_Profile.SetDbl(CProfile::MAX_ROUNDNESS,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_RoundMax)));

	// Photometry
	CApertures aper;
	for (int i=0; i<MAX_APERTURES; i++) {
		double radius = gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_Aperture[i]));
		if (radius>0)
			aper.Add(CAperture(i+1, radius));
	}
	m_Profile.SetApertures(aper);
	m_Profile.SetDbl(CProfile::SKY_INNER_RADIUS,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_SkyIn)));
	m_Profile.SetDbl(CProfile::SKY_OUTER_RADIUS,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_SkyOut)));

	// Matching
	m_Profile.SetBool(CProfile::SPARSE_FIELDS,
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_MatchSparse))!=FALSE);
	m_Profile.SetInt(CProfile::MATCH_READ_STARS,
		(gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_MatchRS)));
	m_Profile.SetInt(CProfile::MATCH_IDENT_STARS,
		(gint)gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_MatchIS)));
	m_Profile.SetDbl(CProfile::MATCH_CLIP,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_MatchClip)));
	m_Profile.SetDbl(CProfile::MAX_OFFSET,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_MatchMax)));

	// Light curve
	m_Profile.SetBool(CProfile::MULTIPLE_COMP_STARS,
		gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_ArtCompStar))!=FALSE);

	// Master bias
	m_Profile.SetInt(CProfile::MBIAS_FORMAT, index_to_bitpix(
		gtk_combo_box_get_active(GTK_COMBO_BOX(m_MBiasFormat))));

	// Master dark
	m_Profile.SetInt(CProfile::MDARK_FORMAT, index_to_bitpix(
		gtk_combo_box_get_active(GTK_COMBO_BOX(m_MDarkFormat))));

	// Master flat
	m_Profile.SetDbl(CProfile::MFLAT_LEVEL,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_MFlatLevel)));
	m_Profile.SetInt(CProfile::MFLAT_FORMAT, index_to_bitpix(
		gtk_combo_box_get_active(GTK_COMBO_BOX(m_MFlatFormat))));

	// Kombine
	m_Profile.SetInt(CProfile::KOMBINE_FORMAT, index_to_bitpix(
		gtk_combo_box_get_active(GTK_COMBO_BOX(m_KombineFormat))));

	// Find variables
	m_Profile.SetDbl(CProfile::VARFIND_THRESHOLD,
		gtk_spin_button_get_value(GTK_SPIN_BUTTON(m_VarFindThreshold)));

	// Observer
	CLocation loc = m_Profile.Location();
	loc.SetName(gtk_entry_get_text(GTK_ENTRY(m_LocName)));
	loc.SetLon(gtk_entry_get_text(GTK_ENTRY(m_Lon)));
	loc.SetLat(gtk_entry_get_text(GTK_ENTRY(m_Lat)));
	m_Profile.SetLocation(loc);
}

bool CProjectDlg::OnCloseQuery(void)
{
	const gchar *lon, *lat, *name;
	char buf[64];
	double x, y;

	// Check observer's coordinates
	lon = gtk_entry_get_text(GTK_ENTRY(m_Lon));
	lat = gtk_entry_get_text(GTK_ENTRY(m_Lat));
	if (*lon!='\0' || *lat!='\0') {
		if (cmpack_strtolon(lon, &x)!=0) {
			ShowPage(PAGE_OBSERVER);
			ShowError(GTK_WINDOW(m_pDlg), "Invalid value of the longitude.");
			return false;
		}
		cmpack_lontostr(x, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_Lon), buf);
		if (cmpack_strtolat(lat, &y)!=0) {
			ShowPage(PAGE_OBSERVER);
			ShowError(GTK_WINDOW(m_pDlg), "Invalid value of the latitude.");
			return false;
		}
		cmpack_lattostr(y, buf, 64);
		gtk_entry_set_text(GTK_ENTRY(m_Lat), buf);
	}

	GetData();

	if (m_Profile.GetDbl(CProfile::SKY_INNER_RADIUS) >= m_Profile.GetDbl(CProfile::SKY_OUTER_RADIUS)) {
		ShowPage(PAGE_PHOTOMETRY);
		ShowError(GTK_WINDOW(m_pDlg), "Invalid size of the background annulus.\n"
			"The inner radius must be smaller than the outer radius.");
		return false;
	}
	if (m_Profile.GetInt(CProfile::MATCH_READ_STARS) < m_Profile.GetInt(CProfile::MATCH_IDENT_STARS)) {
		ShowPage(PAGE_PHOTOMETRY);
		ShowError(GTK_WINDOW(m_pDlg), "Invalid parameters in the matching parameters.\n"
			"Number of read stars must be greater than number of identification stars.");
		return false;
	}

	// Project name
	name = gtk_entry_get_text(GTK_ENTRY(m_Name));
	if (StrCmp0(name, g_Project->Name())!=0) {
		GError *error = NULL;
		if (!g_Project->Rename(name, &error)) {
			if (error) {
				ShowError(GTK_WINDOW(m_pDlg), error->message);
				g_error_free(error);
			}
			return false;
		}
	}

	return true;
}

void CProjectDlg::UpdateControls(void)
{
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(m_MatchStandard))) {
		gtk_widget_hide(m_MatchBox2);
		gtk_widget_show(m_MatchBox1);
	} else {
		gtk_widget_hide(m_MatchBox1);
		gtk_widget_show(m_MatchBox2);
	}
}

void CProjectDlg::setdefaults_clicked(GtkWidget *pWidget, CProjectDlg *pDlg)
{
	pDlg->OnSetDefaultsClicked(pWidget);
}

void CProjectDlg::OnSetDefaultsClicked(GtkWidget *pButton)
{
	CApertures aper;

	switch (m_PageId)
	{
	case PAGE_CAMERA:
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RNoise), 
			CProfile::GetDefaultDbl(CProfile::READ_NOISE));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_ADCGain), 
			CProfile::GetDefaultDbl(CProfile::ADC_GAIN));
		break;

	case PAGE_SOURCE_FILES:
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_DataMin), 
			CProfile::GetDefaultDbl(CProfile::BAD_PIXEL_VALUE));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_DataMax), 
			CProfile::GetDefaultDbl(CProfile::OVEREXPOSED_VALUE));
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_WorkFormat), bitpix_to_index(
			CProfile::GetDefaultInt(CProfile::WORK_FORMAT)));
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_Binning), binning_to_index(
			CProfile::GetDefaultInt(CProfile::BINNING)));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_BLeft), 0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_BRight), 0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_BTop), 0);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_BBottom), 0);
		break;

	case PAGE_CALIBRATION:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_StdCalibration), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_AdvCalibration), FALSE);
		break;

	case PAGE_STAR_DETECTION:
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_FWHM), 
			CProfile::GetDefaultDbl(CProfile::DETECTION_FWHM));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_Thresh), 
			CProfile::GetDefaultDbl(CProfile::DETECTION_THRESHOLD));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_SharpMin), 
			CProfile::GetDefaultDbl(CProfile::MIN_SHARPNESS));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_SharpMax), 
			CProfile::GetDefaultDbl(CProfile::MAX_SHARPNESS));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RoundMin), 
			CProfile::GetDefaultDbl(CProfile::MIN_ROUNDNESS));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_RoundMax), 
			CProfile::GetDefaultDbl(CProfile::MAX_ROUNDNESS));
		break;

	case PAGE_PHOTOMETRY:
		aper = CProfile::DefaultApertures();
		for (int i=0; i<MAX_APERTURES; i++) {
			int index = aper.Find(i+1);
			gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_Aperture[i]), aper.GetRadius(index));
		}
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_SkyIn), 
			CProfile::GetDefaultDbl(CProfile::SKY_INNER_RADIUS));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_SkyOut), 
			CProfile::GetDefaultDbl(CProfile::SKY_OUTER_RADIUS));
		break;

	case PAGE_MATCHING:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_MatchStandard), TRUE);
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_MatchSparse), FALSE);
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_MatchRS), 
			CProfile::GetDefaultInt(CProfile::MATCH_READ_STARS));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_MatchIS), 
			CProfile::GetDefaultInt(CProfile::MATCH_IDENT_STARS));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_MatchClip), 
			CProfile::GetDefaultDbl(CProfile::MATCH_CLIP));
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_MatchMax), 
			CProfile::GetDefaultDbl(CProfile::MAX_OFFSET));
		break;

	case PAGE_LIGHT_CURVE:
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(m_ArtCompStar), 
			CProfile::GetDefaultBool(CProfile::MULTIPLE_COMP_STARS));
		break;

	case PAGE_MASTER_BIAS:
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_MBiasFormat), bitpix_to_index(
			CProfile::GetDefaultInt(CProfile::MBIAS_FORMAT)));
		break;

	case PAGE_MASTER_DARK:
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_MDarkFormat), bitpix_to_index(
			CProfile::GetDefaultInt(CProfile::MDARK_FORMAT)));
		break;

	case PAGE_MASTER_FLAT:
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_MFlatLevel), 
			CProfile::GetDefaultDbl(CProfile::MFLAT_LEVEL));
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_MFlatFormat), bitpix_to_index(
			CProfile::GetDefaultInt(CProfile::MFLAT_FORMAT)));
		break;

	case PAGE_MERGE_FRAMES:
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_KombineFormat), bitpix_to_index(
			CProfile::GetDefaultInt(CProfile::KOMBINE_FORMAT)));
		break;

	case PAGE_FIND_VARIABLES:
		gtk_spin_button_set_value(GTK_SPIN_BUTTON(m_VarFindThreshold), 
			CProfile::GetDefaultDbl(CProfile::VARFIND_THRESHOLD));
		break;

	case PAGE_OBSERVER:
		gtk_entry_set_text(GTK_ENTRY(m_LocName), "");
		gtk_entry_set_text(GTK_ENTRY(m_Lat), "");
		gtk_entry_set_text(GTK_ENTRY(m_Lon), "");
		break;

	default:
		break;
	}
}

void CProjectDlg::button_toggled(GtkWidget *togglebutton, CProjectDlg *pMe)
{
	pMe->OnButtonToggled(togglebutton);
}

void CProjectDlg::OnButtonToggled(GtkWidget *togglebutton)
{
	UpdateControls();
}

gint CProjectDlg::selection_changed(GtkTreeSelection *widget, CProjectDlg *pDlg)
{
	pDlg->OnSelectionChanged();
	return FALSE;
}

void CProjectDlg::OnSelectionChanged(void)
{
	int id;
	GtkTreeIter iter;
	GtkWidget *page;
	GtkTreeModel *model;
	gchar *caption, buf[1024];

	if (!m_Updating) {
		m_Updating = true;
		GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_PageView));
		GList* rows = gtk_tree_selection_get_selected_rows(selection, &model);
		if (g_list_length(rows)>0) {
			gtk_tree_model_get_iter(model, &iter, (GtkTreePath*)rows->data);
			gtk_tree_model_get(model, &iter, 0, &id, 1, &caption, 2, &page, -1);
		} else {
			page = NULL;
			caption = NULL;
			id = -1;
		}
		if (page!=m_CurrentPage) {
			if (m_CurrentPage)
				gtk_container_remove(GTK_CONTAINER(m_PageFrame), m_CurrentPage);
			m_CurrentPage = page;
			if (m_CurrentPage)
				gtk_container_add(GTK_CONTAINER(m_PageFrame), m_CurrentPage);
			if (caption)
				sprintf(buf, "<span size='large'><b>%s</b></span>", caption);
			else
				buf[0] = '\0';
			gtk_label_set_markup(GTK_LABEL(m_PageTitle), buf);
		}
		m_PageId = id;
		if (m_PageId>=0)
			CConfig::SetInt("ProjectDlg", "LastPage", m_PageId);
		g_list_foreach(rows, (GFunc)gtk_tree_path_free, NULL);
		g_list_free(rows);
		g_free(caption);
		UpdateControls();
		m_Updating = false;
	}
}

void CProjectDlg::ShowPage(int page_id)
{
	tFindData data(page_id);
	GtkTreeModel *model = gtk_tree_view_get_model(GTK_TREE_VIEW(m_PageView));
	gtk_tree_model_foreach(model, FindPage, &data);
	if (data.path) {
		GtkTreeSelection *selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(m_PageView));
		gtk_tree_selection_select_path(selection, data.path);
		gtk_tree_path_free(data.path);
	}
}

void CProjectDlg::button_clicked(GtkWidget *widget, CProjectDlg *pDlg)
{
	pDlg->OnButtonClicked(widget);
}

void CProjectDlg::OnButtonClicked(GtkWidget *pButton)
{
	if (pButton==m_LoadProfile)
		LoadFromFile();
	else if (pButton==m_SaveProfile)
		SaveToFile();
	else if (pButton==m_LoadDefaults)
		LoadDefaults();
	else if (pButton==m_LocBtn)
		EditLocation();
}

void CProjectDlg::combo_box_changed(GtkWidget *widget, CProjectDlg *pDlg)
{
	pDlg->OnComboBoxChanged(widget);
}

void CProjectDlg::OnComboBoxChanged(GtkWidget *pBox)
{
	if (pBox == m_Type) {
		tProjectType type = index_to_type(gtk_combo_box_get_active(GTK_COMBO_BOX(pBox)));
		if (type>=0 && type<EndOfProjectTypes && type!=m_Profile.ProjectType()) {
			m_Profile.SetProjectType(type);
			m_ProjectType = type;
			gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(m_FilteredPageList));
		}
	}
}


//
// Load project settings (show file browser)
//
void CProjectDlg::LoadFromFile(void)
{
	CProfile profile;

	CLoadProfileDlg dlg(GTK_WINDOW(m_pDlg), m_ProjectType, true);
	if (dlg.Execute(profile)) {
		m_Profile = profile;
		m_ProjectType = m_Profile.ProjectType();
		gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(m_FilteredPageList));
		SetData();
	}

	UpdateControls();
}


//
// Load factory defaults
//
void CProjectDlg::LoadDefaults(void)
{
	if (ShowConfirmation(GTK_WINDOW(m_pDlg), "Do you want to set project settings to factory defaults?")) {
		// Preserves project type
		m_Profile.Clear();
		m_Profile.SetProjectType(m_ProjectType);
		m_ProjectType = m_Profile.ProjectType();
		gtk_tree_model_filter_refilter(GTK_TREE_MODEL_FILTER(m_FilteredPageList));
		SetData();
		ShowInformation(GTK_WINDOW(m_pDlg), "Project settings was set to factory defaults.");
	}
	UpdateControls();
}


//
// Save project settings
//
void CProjectDlg::SaveToFile(void)
{
	CSaveProfileDlg dlg(GTK_WINDOW(m_pDlg));
	if (dlg.Execute()) {
		GError *error = NULL;
		if (m_Profile.Export(dlg.Path(), &error)) {
			CConfig::AddProfileToRecentList(m_Profile.ProjectType(), dlg.Path());
		} else {
			if (error) {
				ShowError(GTK_WINDOW(m_pDlg), error->message);
				g_error_free(error);
			}
		}
	}
}


//
// Edit observer's coordinates
//
void CProjectDlg::EditLocation(void)
{
	CLocationDlg dlg(GTK_WINDOW(m_pDlg));

	CLocation loc = m_Profile.Location();
	loc.SetName(gtk_entry_get_text(GTK_ENTRY(m_LocName)));
	loc.SetLon(gtk_entry_get_text(GTK_ENTRY(m_Lon)));
	loc.SetLat(gtk_entry_get_text(GTK_ENTRY(m_Lat)));
	if (dlg.Execute(&loc, true)) {
		m_Profile.SetLocation(loc);
		SetData();
		UpdateControls();
	}
}
