/**************************************************************

table_class.cpp (C-Munipack project)
Table class interface
Copyright (C) 2011 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>

#include "table_class.h"
#include "utils.h"

#define ALLOC_BY 64

//--------------------------   HELPER FUNCTIONS   ----------------------------------

static gchar *FromLocale(const char *str)
{
	if (str)
		return g_locale_to_utf8(str, -1, NULL, NULL, NULL);
	return NULL;
}

//-----------------------   LIST OF CHANNELS   ---------------------

// Constructor with initialization
CChannel::CChannel(CTable *tab, int ycol, int ucol, tChannelInfo info):m_Table(tab), 
	m_Column(ycol), m_ColumnU(ucol), m_Name(NULL), m_Exported(true), m_Info(info),
	m_DataType(CMPACK_TYPE_VOID), m_Precision(0)
{
	CmpackTabColumn iinfo;

	if (m_Table && m_Column>=0) {
		cmpack_tab_get_column(m_Table->Handle(), m_Column, CMPACK_TM_NAME | CMPACK_TM_TYPE_PREC, &iinfo);
		if (iinfo.name)
			m_Name = FromLocale(iinfo.name);
		else
			m_Name = g_strdup("");
		m_DataType = iinfo.dtype;
		m_Precision = iinfo.prec;
	}
}

// Destructor
CChannel::~CChannel(void)
{
	g_free(m_Name);
}

void CChannel::SetExported(bool enabled)
{
	m_Exported = enabled;
}

const gchar *CChannel::Unit(void) const
{
	switch (m_Info)
	{
	case DATA_MAGNITUDE:
	case DATA_DEVIATION:
		return "mag";
	case DATA_HELCOR:
		return "days";
	case DATA_OFFSET:
		return "px";
	case DATA_ALTITUDE:
		return "deg";
	default:
		return NULL;
	}
}

// Get minimum value
double CChannel::Min(void) const
{
	CmpackTabColumn info;

	if (m_Table && m_Column>=0) {
		cmpack_tab_get_column(m_Table->Handle(), m_Column, CMPACK_TM_RANGE, &info);
		if (info.range_valid)
			return info.range_min;
	}
	return 0.0;
}

// Get minimum value
double CChannel::Max(void) const
{
	CmpackTabColumn info;

	if (m_Table && m_Column>=0) {
		cmpack_tab_get_column(m_Table->Handle(), m_Column, CMPACK_TM_RANGE, &info);
		if (info.range_valid)
			return info.range_max;
	}
	return 0.0;
}

// Get minimum value
double CChannel::MaxU(void) const
{
	CmpackTabColumn info;

	if (m_Table && m_ColumnU>=0) {
		cmpack_tab_get_column(m_Table->Handle(), m_ColumnU, CMPACK_TM_RANGE, &info);
		if (info.range_valid)
			return info.range_max;
	}
	return 0.0;
}

// Default Constructor
CChannels::CChannels(void):m_Count(0), m_Capacity(0), m_List(NULL)
{
}

// Destructor
CChannels::~CChannels()
{
	int i;

	for (i=0; i<m_Count; i++)
		delete m_List[i];
	g_free(m_List);
}

// Clear the table
void CChannels::Clear(void)
{
	int i;

	for (i=0; i<m_Count; i++)
		delete m_List[i];
	m_Count = 0;
}

// Add an aperture, if the aperture with the same id is 
// in the table, it changes its parameters
void CChannels::Add(CChannel *item)
{
	if (m_Count>=m_Capacity) {
		m_Capacity += ALLOC_BY;
		m_List = (CChannel**)g_realloc(m_List, m_Capacity*sizeof(CChannel));
	}
	m_List[m_Count++] = item;
}

// Find aperture by id 
int CChannels::FindColumn(int col) const
{
	int i;

	for (i=0; i<m_Count; i++) {
		if (m_List[i]->Column() == col || m_List[i]->ColumnU() == col)
			return i;
	}
	return -1;
}

// Find aperture by id 
int CChannels::FindFirst(const gchar *name) const
{
	int i;

	for (i=0; i<m_Count; i++) {
		if (strcmp(m_List[i]->Name(), name)==0)
			return i;
	}
	return -1;
}

// Find aperture by id 
int CChannels::FindFirst(tChannelInfo info) const
{
	int i;

	for (i=0; i<m_Count; i++) {
		if (m_List[i]->Info() == info)
			return i;
	}
	return -1;
}

// Get aperture by index
const CChannel *CChannels::Get(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index];
	return NULL;
}
CChannel *CChannels::Get(int index) 
{
	if (index>=0 && index<m_Count)
		return m_List[index];
	return NULL;
}

// Get column by index
int CChannels::GetColumn(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index]->Column();
	return -1;
}

// Get column by index
int CChannels::GetColumnU(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index]->ColumnU();
	return -1;
}

// Get name by index
const gchar *CChannels::GetName(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index]->Name();
	return NULL;
}

// Get name by index
int CChannels::GetPrecision(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index]->Precision();
	return 0;
}

// Get channel info
tChannelInfo CChannels::GetInfo(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index]->Info();
	return DATA_UNDEFINED;
}

// Get channel info
const gchar *CChannels::GetUnit(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index]->Unit();
	return NULL;
}

// Get channel info
double CChannels::GetMin(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index]->Min();
	return DATA_UNDEFINED;
}

// Get channel info
double CChannels::GetMax(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index]->Max();
	return DATA_UNDEFINED;
}

// Get channel info
double CChannels::GetMaxU(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index]->MaxU();
	return DATA_UNDEFINED;
}

// Get channel info
bool CChannels::GetExported(int index) const
{
	if (index>=0 && index<m_Count)
		return m_List[index]->Exported();
	return false;
}

//-----------------------   TABLE   --------------------------------

//
// Default constructor
//
CTable::CTable(void):m_Handle(NULL), m_CacheFlags(0), m_PKColumn(-1)
{
}

//
// Constructor with initialization
//
CTable::CTable(CmpackTable *handle):m_Handle(handle), m_CacheFlags(0), 
	m_PKColumn(-1)
{
}

//
// Constructor with initialization
//
CTable::CTable(CmpackTableType type):m_CacheFlags(0), m_PKColumn(-1)
{
	m_Handle = cmpack_tab_init(type);
}

//
// Destructor
//
CTable::~CTable(void)
{
	if (m_Handle)
		cmpack_tab_destroy(m_Handle);
}

//
// Create a new table
//
void CTable::Init(CmpackTableType type)
{
	Clear();
	m_Handle = cmpack_tab_init(type);
	InvalidateCache();
}

//
// Assign a table
//
void CTable::Assign(CmpackTable *table)
{
	Clear();
	m_Handle = table;
	InvalidateCache();
}

//
// Deep copy
//
void CTable::MakeCopy(const CTable &orig)
{
	Clear();
	if (orig.Handle()) {
		m_Handle = cmpack_tab_init(CMPACK_TABLE_UNSPECIFIED);
		cmpack_tab_copy(m_Handle, orig.Handle());
	}
}

//
// Load table
//
bool CTable::Load(const gchar *fpath, int *result)
{
	int res;

	Clear();

	char *f = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);
	res = cmpack_tab_load(&m_Handle, f, 0);
	g_free(f);
	if (res!=0) {
		if (result)
			*result = res;
		return false;
	}
	InvalidateCache();
	if (result)
		*result = 0;
	return true;
}

//
// Save table
//
bool CTable::Save(const gchar *fpath, int *result)
{
	int res;

	if (!m_Handle) {
		if (result)
			*result = CMPACK_ERR_CLOSED_FILE;
		return false;
	}
	char *f = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);
	res = cmpack_tab_save(m_Handle, f, 0, 0, 0);
	g_free(f);
	if (res!=0) {
		if (result)
			*result = res;
		return false;
	}

	InvalidateCache();
	if (result)
		*result = 0;
	return true;
}


//
// Close handle
//
void CTable::Clear(void)
{
	if (m_Handle) {
		cmpack_tab_destroy(m_Handle);
		m_Handle = NULL;
	}
	m_ChannelsX.Clear();
	m_ChannelsY.Clear();
	m_PKColumn = -1;
	InvalidateCache();
}


//
// Get type identifier
//
CmpackTableType CTable::Type(void) const
{
	if (m_Handle)
		return cmpack_tab_get_type(m_Handle);
	return CMPACK_TABLE_UNSPECIFIED;
}

//
// Get header field by index
//
bool CTable::GetParam(int index, char **key, char **val) const
{
	const char *keyword, *value;

	if (m_Handle && cmpack_tab_gkyn(m_Handle, index, &keyword, &value)) {
		if (key)
			*key = (keyword ? g_strdup(keyword) : NULL);
		if (val)
			*val = (value ? g_strdup(value) : NULL);
		return true;
	}
	return false;
}


// Set header field
void CTable::SetParam(const char *key, const char *value)
{
	if (m_Handle)
		cmpack_tab_pkys(m_Handle, key, value);
}

//
// Get number of rows
//
int CTable::Rows(void) const
{
	if (m_Handle)
		return cmpack_tab_nrows(m_Handle);
	return 0;
}

//
// Browsing
//
bool CTable::Rewind(void)
{
	return (m_Handle && cmpack_tab_rewind(m_Handle)==0);
}
bool CTable::Next(void)
{
	return (m_Handle && cmpack_tab_next(m_Handle)==0);
}
bool CTable::EndOfTable(void)
{
	return (m_Handle && cmpack_tab_eof(m_Handle)!=0);
}

//
// Get color filter name
//
const char *CTable::Filter(void) const
{
	if (m_Handle)
		return cmpack_tab_gkys(m_Handle, "Filter");
	return NULL;
}

//
// Get aperture index
//
int CTable::Aperture(void) const
{
	int value;
	if (m_Handle) {
		if (cmpack_tab_gkyi(m_Handle, "Aperture", &value)==0)
			return value;
	}
	return 0;
}

//
// Render to graph
//
void CTable::Render(CmpackGraphData *data, int xch, int ych, bool no_errors)
{
	int		row_count, param, xcol, ycol, ucol;
	double	x, y, error;
	CmpackGraphItem item;

	memset(&item, 0, sizeof(CmpackGraphItem));

	cmpack_graph_data_clear(data);
	if (m_Handle) {
		const CChannel *ch_x = ChannelsX()->Get(xch), *ch_y = ChannelsY()->Get(ych);
		row_count = cmpack_tab_nrows(m_Handle);
		if (ch_x && ch_y && row_count>0) {
			cmpack_graph_data_alloc(data, row_count);
			xcol = ch_x->Column();
			ycol = ch_y->Column();
			ucol = ch_y->ColumnU();
			int index = 0;
			cmpack_tab_rewind(m_Handle);
			while (!cmpack_tab_eof(m_Handle)) {
				if (cmpack_tab_gtdd(m_Handle, xcol, &x)==0 && 
					cmpack_tab_gtdd(m_Handle, ycol, &y)==0) {
						if (ucol<0 || no_errors || cmpack_tab_gtdd(m_Handle, ucol, &error)!=0)
							error = 0.0;
						item.x = x;
						item.y = y;
						item.error = error;
						if (m_PKColumn>=0)
							cmpack_tab_gtdi(m_Handle, m_PKColumn, &param);
						else
							param = index;
						item.param = param;
						cmpack_graph_data_add(data, &item, sizeof(item));
				}
				cmpack_tab_next(m_Handle);
				index++;
			}
		}
	}
}

//
// Invalidate cache
//
void CTable::InvalidateCache(void)
{
	m_CacheFlags = 0;
}

//
// Get double value from actual row
//
bool CTable::GetDbl(int column, double *val) const
{
	if (m_Handle)
		return cmpack_tab_gtdd(m_Handle, column, val)==0;
	return false;
}

// Set nul value to given column
void CTable::SetNulValue(int column, double nulval)
{
	CmpackTabColumn info;
	if (m_Handle) {
		info.nul_value = nulval;
		cmpack_tab_set_column(m_Handle, column, CMPACK_TM_NULVAL, &info);
	}
}

/*double CTable::GetDbl(int column, double defval) const
{
	double val;

	if (m_Handle && cmpack_tab_gtdd(m_Handle, column, &val)==0)
		return val;
	return defval;
}*/

//
// Get column with primary keys
//
int CTable::PKColumn(void)
{
	if ((m_CacheFlags & CF_CHANNELS)==0)
		UpdateChannels();
	return m_PKColumn;
}

//
// Update list of idependent channels
//
CChannels *CTable::ChannelsX(void)
{
	if ((m_CacheFlags & CF_CHANNELS)==0)
		UpdateChannels();
	return &m_ChannelsX;
}

//
// Update list of idependent channels
//
CChannels *CTable::ChannelsY(void)
{
	if ((m_CacheFlags & CF_CHANNELS)==0)
		UpdateChannels();
	return &m_ChannelsY;
}

//
// Update channels
//
void CTable::UpdateChannels(void)
{
	const static int mask = CMPACK_TM_NAME | CMPACK_TM_TYPE_PREC;
	int i, ncols;
	CmpackTableType ttype;
	CmpackTabColumn col, col2;

	m_ChannelsX.Clear();
	m_ChannelsY.Clear();
	m_PKColumn = -1;
	if (!m_Handle)
		return;

	ncols = cmpack_tab_ncolumns(m_Handle);
	ttype = cmpack_tab_get_type(m_Handle);
	for (i=0; i<ncols; i++) {
		cmpack_tab_get_column(m_Handle, i, mask, &col);
		switch (ttype)
		{
		case CMPACK_TABLE_LIGHTCURVE:
		case CMPACK_TABLE_TRACKLIST:
		case CMPACK_TABLE_AIRMASS:
			if (col.dtype == CMPACK_TYPE_INT && strcmp(col.name, "FRAME")==0) {
				m_PKColumn = i;
			} else
			if (col.dtype==CMPACK_TYPE_DBL && (strcmp(col.name, "JD")==0 || strcmp(col.name, "HJD")==0 || 
				strcmp(col.name, "GJD")==0 || strcmp(col.name, "JDHEL")==0 || 
				strcmp(col.name, "JDGEO")==0)) {
					m_ChannelsX.Add(new CChannel(this, i, -1, DATA_JD));
			} else
			if (col.dtype==CMPACK_TYPE_DBL && strcmp(col.name, "AIRMASS")==0) {
					m_ChannelsY.Add(new CChannel(this, i, -1, DATA_AIRMASS));
			} else
			if (col.dtype==CMPACK_TYPE_DBL && strcmp(col.name, "ALTITUDE")==0) {
					m_ChannelsY.Add(new CChannel(this, i, -1, DATA_ALTITUDE));
			} else
			if (col.dtype==CMPACK_TYPE_DBL && (strcmp(col.name, "OFFSETX")==0 || strcmp(col.name, "OFFSETY")==0)) {
					m_ChannelsY.Add(new CChannel(this, i, -1, DATA_OFFSET));
			} else
			if (col.dtype==CMPACK_TYPE_DBL && strcmp(col.name, "HELCOR")==0) {
					m_ChannelsY.Add(new CChannel(this, i, -1, DATA_HELCOR));
			} else
			if (col.dtype==CMPACK_TYPE_DBL && (col.name[0]=='V' || col.name[0]=='C' || col.name[0]=='K')) {
				int err = -1;
				if (cmpack_tab_get_column(m_Handle, i+1, mask, &col2)==0 && col2.name[0]=='s') 
					err = i+1;
				m_ChannelsY.Add(new CChannel(this, i, err, DATA_MAGNITUDE));
				if (err>=0)
					i++;
			} else
			if (col.dtype==CMPACK_TYPE_DBL && strstr(col.name,"MAG")==col.name) {
				int err = -1;
				if (cmpack_tab_get_column(m_Handle, i+1, mask, &col2)==0 && strstr(col2.name, "ERR")==col2.name)
					err = i+1;
				m_ChannelsY.Add(new CChannel(this, i, err, DATA_MAGNITUDE));
				if (err>=0)
					i++;
			} else
			if (col.dtype==CMPACK_TYPE_DBL || col.dtype==CMPACK_TYPE_INT) {
				m_ChannelsY.Add(new CChannel(this, i, -1, DATA_ANY));
			}
			break;

		case CMPACK_TABLE_MUNIFIND:
			if (col.dtype==CMPACK_TYPE_INT && strcmp(col.name, "INDEX")==0) {
				m_PKColumn = i;
			} else
			if (col.dtype==CMPACK_TYPE_DBL && strcmp(col.name, "MEAN_MAG")==0) {
				m_ChannelsX.Add(new CChannel(this, i, -1, DATA_MAGNITUDE));
			} else
			if (col.dtype==CMPACK_TYPE_DBL && strcmp(col.name, "STDEV")==0) {
				m_ChannelsY.Add(new CChannel(this, i, -1, DATA_DEVIATION));
			}
			break;

		case CMPACK_TABLE_APERTURES:
			if (strcmp(col.name, "APERTURE")==0) {
				m_PKColumn = i;
				m_ChannelsX.Add(new CChannel(this, i, -1, DATA_ANY));
			} else
			if (col.name[0]=='C' || col.name[0]=='K') {
				m_ChannelsY.Add(new CChannel(this, i, -1, DATA_DEVIATION));
			} else
			if (col.dtype==CMPACK_TYPE_DBL || col.dtype==CMPACK_TYPE_INT) {
				m_ChannelsY.Add(new CChannel(this, i, -1, DATA_ANY));
			}
			break;

		default:
			if (col.dtype==CMPACK_TYPE_DBL || col.dtype==CMPACK_TYPE_INT) {
				if (m_ChannelsX.Count()==0) 
					m_ChannelsX.Add(new CChannel(this, i, -1, DATA_ANY));
				else
					m_ChannelsY.Add(new CChannel(this, i, -1, DATA_ANY));
			}
			break;
		}
	}

	m_CacheFlags |= CF_CHANNELS;
}

void CTable::CheckLimits(double *a, double *b, double eps, double min, double max)
{
	double z, g;

	*a = LimitValue(*a, min, max);
	*b = LimitValue(*b, min, max);
	if ((*b-*a)<eps && (max-min)>=eps) {
		z = 0.5 * (*a + *b);
		g = z - (z - min)/(max - min)*eps;
		g = LimitValue(g, MAX(min, *b-eps), MIN(*a, max-eps));
		*a = g;
		*b = g + eps;
	}
}

//
// Set graph view parameters
//
void CTable::SetView(CmpackGraphView *view, int xch, int ych, bool no_errors,
	const gchar *user_name_x, const gchar *user_name_y)
{
	gdouble min, max;
	const gchar *name;

	CChannel *ch_x = ChannelsX()->Get(xch);
	if (ch_x) {
		min = ch_x->Min();
		max = ch_x->Max();
		name = (user_name_x ? user_name_x : ch_x->Name());
		switch (ch_x->Info())
		{
		case DATA_MAGNITUDE:
			CheckLimits(&min, &max, 0.05, -99.0, +99.0);
			cmpack_graph_view_set_x_axis(view, FALSE, FALSE, min, max, 0.05,
				GRAPH_FIXED, 1, 3, name);
			break;
		case DATA_JD:
			CheckLimits(&min, &max, 0.05, 0.0, 1e7);
			cmpack_graph_view_set_x_axis(view, FALSE, FALSE, min, max, 0.005,
				GRAPH_FIXED, 0, 3, name);
			break;
		default:
			CheckLimits(&min, &max, 0.005, DBL_MIN, DBL_MAX);
			cmpack_graph_view_set_x_axis(view, FALSE, FALSE, min, max, 0.005,
				GRAPH_FIXED, 0, 3, name);
			break;
		}
	}

	CChannel *ch_y = ChannelsY()->Get(ych);
	if (ch_y) {
		min = ch_y->Min();
		max = ch_y->Max();
		if (!no_errors && ch_y->ColumnU()>0) {
			min -= ch_y->MaxU();
			max += ch_y->MaxU();
		}
		name = (user_name_y ? user_name_y : ch_y->Name());
		switch (ch_y->Info())
		{
		case DATA_MAGNITUDE:
			CheckLimits(&min, &max, 0.05, -99.0, +99.0);
			cmpack_graph_view_set_y_axis(view, FALSE, TRUE, min, max, 0.05,
				GRAPH_FIXED, 1, 3, name);
			break;
		case DATA_DEVIATION:
			CheckLimits(&min, &max, 0.05, 0.0, 99.0);
			cmpack_graph_view_set_y_axis(view, FALSE, FALSE, min, max, 0.05,
				GRAPH_FIXED, 1, 3, name);
			break;
		case DATA_HELCOR:
			CheckLimits(&min, &max, 0.0001, -1.0, 1.0);
			cmpack_graph_view_set_y_axis(view, FALSE, FALSE, min, max, 0.0001,
				GRAPH_FIXED, 1, 4, name);
			break;
		case DATA_AIRMASS:
			CheckLimits(&min, &max, 0.01, 1.0, 99.0);
			cmpack_graph_view_set_y_axis(view, FALSE, FALSE, min, max, 0.01,
				GRAPH_FIXED, 1, 2, name);
			break;
		case DATA_OFFSET:
			CheckLimits(&min, &max, 0.1, -1e6, 1e6);
			cmpack_graph_view_set_y_axis(view, FALSE, FALSE, min, max, 0.1,
				GRAPH_FIXED, 0, 1, name);
			break;
		case DATA_ALTITUDE:
			CheckLimits(&min, &max, 1.0, -90.0, +90.0);
			cmpack_graph_view_set_y_axis(view, FALSE, FALSE, min, max, 1.0,
				GRAPH_FIXED, 0, 0, name);
			break;
		default:
			CheckLimits(&min, &max, 0.005, -DBL_MAX, DBL_MAX);
			cmpack_graph_view_set_y_axis(view, FALSE, FALSE, min, max, 0.005,
				GRAPH_FIXED, 0, 3, name);
			break;
		}
	}
}

bool CTable::Export(const gchar *fpath, bool ignore_ids, bool no_header, 
	bool no_errors, int *result)
{
	int res, i, index, count, ncols, *cols;

	if ((m_CacheFlags & CF_CHANNELS)==0)
		UpdateChannels();

	if (!fpath) {
		if (result)
			*result = CMPACK_ERR_OPEN_ERROR;
		return false;
	}
	if (!m_Handle) {
		if (result)
			*result = CMPACK_ERR_INVALID_CONTEXT;
		return false;
	}

	char *f = g_locale_from_utf8(fpath, -1, NULL, NULL, NULL);

	ncols = cmpack_tab_ncolumns(m_Handle);
	cols = (int*)g_malloc(ncols*sizeof(int));
	count = 0;
	for (i=0; i<ncols; i++) {
		if (i==m_PKColumn) {
			if (!ignore_ids)
				cols[count++] = i;
		}
		index = m_ChannelsX.FindColumn(i);
		if (index>=0) {
			if (m_ChannelsX.GetExported(index))
				cols[count++] = i;
		}
		index = m_ChannelsY.FindColumn(i);
		if (index>=0) {
			if (m_ChannelsY.GetExported(index)) {
				if (!no_errors || m_ChannelsY.Get(index)->Column()==i) 
					cols[count++] = i;
			}
		}
	}

	int flags = (no_header ? CMPACK_SAVE_NO_HEADER : 0);
	res = cmpack_tab_save(m_Handle, f, flags, cols, count);
	g_free(f);
	g_free(cols);
	if (result)
		*result = res;
	return (res==0);
}

// Show/hide this row from exports
void CTable::Show(bool show)
{
	if (m_Handle)
		cmpack_tab_show(m_Handle, show);
}

// Find row by primary key
bool CTable::Find(int value) const
{
	if (m_Handle) {
		if (m_PKColumn>=0) {
			cmpack_tab_rewind(m_Handle);
			return cmpack_tab_ftdi(m_Handle, m_PKColumn, value)!=0;
		} else {
			// Note: Return codes of cmpack_tab_ftdi and cmpack_tab_setpos are different!
			return cmpack_tab_setpos(m_Handle, value)==0;
		}
	}
	return false;
}

void CTable::Delete(void)
{
	if (m_Handle) 
		cmpack_tab_delete(m_Handle);
}
