/**************************************************************

tracklist_dlg.cpp (C-Munipack project)
The 'Plot track-list' dialog
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#include <stdlib.h>
#include <string.h>
#include <glib.h>

#include "project.h"
#include "configuration.h"
#include "progress_dlg.h"
#include "trackcurve_dlg.h"
#include "frameinfo_dlg.h"
#include "main.h"
#include "utils.h"
#include "proc_classes.h"
#include "ctxhelp.h"
#include "chartexport_dlg.h"

//-------------------------   LOCAL MENU   --------------------------------

enum tMenuId
{
	MENU_FILE = 1,
	MENU_VIEW,
	MENU_TOOLS,
	MENU_HELP
};

enum tCommandId
{
	CMD_CLOSE = 100,
	CMD_EXPORT,
	CMD_REBUILD,
	CMD_PRINT,
	CMD_SAVE,
	CMD_HELP,
	CMD_HIDE,
	CMD_DELETE,
	CMD_PREVIEW,
	CMD_FRAMEINFO,
	CMD_GRID,
	CMD_STATISTICS,
	CMD_MEASUREMENT
};

static const CMenuBar::tMenuItem FileMenu[] = {
	{ CMenuBar::MB_ITEM,		CMD_SAVE,		"_Save curve" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,		CMD_EXPORT,		"_Export as image" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,		CMD_REBUILD,	"_Rebuild" },
	{ CMenuBar::MB_SEPARATOR },
	{ CMenuBar::MB_ITEM,		CMD_CLOSE,		"_Close" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ViewMenu[] = {
	{ CMenuBar::MB_CHECKBTN,	CMD_GRID,		"_Grid" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem ToolsMenu[] = {
	{ CMenuBar::MB_CHECKBTN,	CMD_STATISTICS,	"_Statistics" },
	{ CMenuBar::MB_CHECKBTN,	CMD_MEASUREMENT, "_Measurement" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenuItem HelpMenu[] = {
	{ CMenuBar::MB_ITEM,		CMD_HELP,		"_Show help", "help" },
	{ CMenuBar::MB_END }
};

static const CMenuBar::tMenu TrackListMenu[] = {
	{ "_File",	MENU_FILE,	FileMenu },
	{ "_View",	MENU_VIEW,	ViewMenu },
	{ "_Tools", MENU_TOOLS,	ToolsMenu },
	{ "_Help",	MENU_HELP,	HelpMenu,	CMenuBar::MBF_RIGHT },
	{ NULL }
};

static const CPopupMenu::tPopupMenuItem GraphContextMenu[] = {
	{ CPopupMenu::MB_ITEM, CMD_PREVIEW,		"_Show frame" },
	{ CPopupMenu::MB_ITEM, CMD_FRAMEINFO,	"_Show properties" },
	{ CPopupMenu::MB_SEPARATOR },
	{ CPopupMenu::MB_ITEM, CMD_HIDE,		"_Delete from data set" },
	{ CPopupMenu::MB_ITEM, CMD_DELETE,		"_Remove from project" },
	{ CPopupMenu::MB_END }
};

//------------------------  TOOLBAR HELPERS   ---------------------------------

static GtkToolItem *toolbar_new_button_from_stock(GtkWidget *toolbar, const gchar *stock_id, 
	const gchar *tooltip)
{
	GtkToolItem *tbtn = gtk_tool_button_new_from_stock(stock_id);
	gtk_widget_set_tooltip_text(GTK_WIDGET(tbtn), tooltip);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

static GtkToolItem *toolbar_new_widget(GtkWidget *toolbar, GtkWidget *widget)
{
	GtkToolItem *item = gtk_tool_item_new();
	gtk_container_add(GTK_CONTAINER(item), widget);
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), item, -1);
	return item;
}

static GtkWidget *toolbar_new_combo(GtkWidget *toolbar, const gchar *tooltip)
{
	GtkWidget *entry = gtk_combo_box_new();
	gtk_widget_set_size_request(entry, 120, -1);
	gtk_widget_set_tooltip_text(entry, tooltip);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

static GtkWidget *toolbar_new_label(GtkWidget *toolbar, const char *caption)
{
	GtkWidget *entry = gtk_label_new(caption);
	gtk_misc_set_padding(GTK_MISC(entry), 8, 0);
	gtk_misc_set_alignment(GTK_MISC(entry), 0, 0.5);
	toolbar_new_widget(toolbar, entry);
	return entry;
}

static GtkToolItem *toolbar_new_separator(GtkWidget *toolbar)
{
	GtkToolItem *tbtn = gtk_separator_tool_item_new();
	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), tbtn, -1);
	return tbtn;
}

//-------------------------   TRACK CURVE WINDOW   --------------------------------

CTrackCurveDlg::CTrackCurveDlg(void):m_UpdatePos(true), m_LastPosValid(false), 
	m_LastPosX(0), m_LastPosY(0), m_LastFocus(-1), m_InFiles(0), m_OutFiles(0), 
	m_Column(-1), m_ChannelY(-1), m_InfoMode(INFO_NONE)
{
	gchar buf[512];
	GtkWidget *tbox, *scrwnd;
	GtkCellRenderer *renderer;

	m_ShowGrid   = CConfig::GetBool("TrackCurve", "Grid", false);

	// Dialog caption
	sprintf(buf, "%s - %s", "Track curve", g_AppTitle);
	gtk_window_set_title(GTK_WINDOW(m_pDlg), buf);

	// Menu bar
	m_Menu.Create(TrackListMenu, false);
	m_Menu.RegisterCallback(MenuCallback, this);
	gtk_box_pack_start(GTK_BOX(m_MainBox), m_Menu.Handle(), FALSE, FALSE, 0);

	// Toolbar
	tbox = gtk_toolbar_new();
	gtk_toolbar_set_style(GTK_TOOLBAR(tbox), GTK_TOOLBAR_ICONS);
	gtk_toolbar_set_orientation(GTK_TOOLBAR(tbox), GTK_ORIENTATION_HORIZONTAL);
	gtk_box_pack_start(GTK_BOX(m_MainBox), tbox, false, false, 0);

	m_YLabel = toolbar_new_label(tbox, "Y axis");
	m_YChannels = gtk_list_store_new(2, GTK_TYPE_INT, GTK_TYPE_STRING);
	m_YCombo = toolbar_new_combo(tbox, "Column shown on the vertical axis of the graph");
	g_signal_connect(G_OBJECT(m_YCombo), "changed", G_CALLBACK(changed), this);
	renderer = gtk_cell_renderer_text_new();
	gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(m_YCombo), renderer, TRUE);
	gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(m_YCombo), renderer, "text", 1);

	toolbar_new_separator(tbox);

	toolbar_new_label(tbox, "Zoom");
	m_ZoomFit = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_FIT, "Zoom to all points");
	g_signal_connect(G_OBJECT(m_ZoomFit), "clicked", G_CALLBACK(button_clicked), this);
	m_ZoomOut = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_OUT, "Zoom out");
	g_signal_connect(G_OBJECT(m_ZoomOut), "clicked", G_CALLBACK(button_clicked), this);
	m_ZoomIn = toolbar_new_button_from_stock(tbox, GTK_STOCK_ZOOM_IN, "Zoom in");
	g_signal_connect(G_OBJECT(m_ZoomIn), "clicked", G_CALLBACK(button_clicked), this);

	GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
	gtk_box_pack_start(GTK_BOX(m_MainBox), hbox, TRUE, TRUE, 0);

	// Graph data
	m_GraphData = cmpack_graph_data_new();
	m_Graph = cmpack_graph_view_new_with_model(m_GraphData);
	cmpack_graph_view_set_mouse_control(CMPACK_GRAPH_VIEW(m_Graph), TRUE);
	cmpack_graph_view_set_selection_mode(CMPACK_GRAPH_VIEW(m_Graph), GTK_SELECTION_MULTIPLE);
	cmpack_graph_view_set_scales(CMPACK_GRAPH_VIEW(m_Graph), TRUE, TRUE);
	g_signal_connect(G_OBJECT(m_Graph), "mouse-moved", G_CALLBACK(mouse_moved), this);
	g_signal_connect(G_OBJECT(m_Graph), "button_press_event", G_CALLBACK(button_press_event), this);
	g_signal_connect(G_OBJECT(m_Graph), "selection-changed", G_CALLBACK(selection_changed), this);
	gtk_widget_set_size_request(m_Graph, 300, 200);
	scrwnd = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrwnd), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_scrolled_window_set_shadow_type(GTK_SCROLLED_WINDOW(scrwnd), GTK_SHADOW_ETCHED_IN);
	gtk_container_add(GTK_CONTAINER(scrwnd), m_Graph);
	gtk_box_pack_start(GTK_BOX(hbox), scrwnd, TRUE, TRUE, 0);

	// Info box
	m_InfoBox.RegisterCallback(InfoBoxCallback, this);
	gtk_box_pack_start(GTK_BOX(hbox), m_InfoBox.Handle(), FALSE, FALSE, 0);
	m_InfoBox.SetCaption("Statistics");

	// Measurement tool
	m_MeasBox.RegisterCallback(InfoBoxCallback, this);
	gtk_box_pack_start(GTK_BOX(hbox), m_MeasBox.Handle(), FALSE, FALSE, 0);
	m_MeasBox.SetGraphView(CMPACK_GRAPH_VIEW(m_Graph));

	// Timers
	m_TimerId = g_timeout_add_full(G_PRIORITY_DEFAULT_IDLE, 100, GSourceFunc(timer_cb), this, NULL);

	// Popup menu
	m_GraphMenu.Create(GraphContextMenu);

	// Show the dialog
	gtk_widget_show_all(m_MainBox);
	m_InfoBox.Show(false);
	m_MeasBox.Show(false);
}

CTrackCurveDlg::~CTrackCurveDlg()
{
	// Disconnect graph signals
	g_signal_handlers_disconnect_by_func(G_OBJECT(m_Graph), (gpointer)mouse_moved, this);
	g_signal_handlers_disconnect_by_func(G_OBJECT(m_Graph), (gpointer)button_press_event, this);

	// Release objects, free allocated memory
	g_source_remove(m_TimerId);
	g_object_unref(m_YChannels);
	g_object_unref(m_GraphData);
}

bool CTrackCurveDlg::Make(GtkWindow *parent, bool selected_files)
{
	m_MeasBox.SetTable(NULL);
	m_Table.Clear();
	m_FrameSet.Clear();
	m_ChannelY = -1;
	m_DataSaved = false;

	if (!InitFileList(parent, selected_files))
		return false;

	if (!RebuildData(parent))
		return false;

	SetInfoMode(INFO_NONE);
	UpdateTrackCurve();
	UpdateChannels();
	UpdateGraph(TRUE, TRUE);
	SetStatus(NULL);
	UpdateControls();
	return true;
}

void CTrackCurveDlg::OnFrameSetChanged(void)
{
	UpdateTrackCurve();
	UpdateGraph(TRUE, TRUE);
	UpdateStatus();
	UpdateTools();
	UpdateControls();
}

void CTrackCurveDlg::UpdateTrackCurve(void)
{
	int res;

	m_MeasBox.SetTable(NULL);
	m_Table.Clear();
	res = CmpackTrackCurve(NULL, m_Table, m_FrameSet);
	if (res==0) {
		m_MeasBox.SetTable(&m_Table);
		m_MeasBox.SetChannel(CMPACK_AXIS_X, 0);
	}
}

void CTrackCurveDlg::UpdateControls()
{
	m_Menu.Check(CMD_GRID, m_ShowGrid);
	m_Menu.Check(CMD_STATISTICS, m_InfoMode == INFO_STATISTICS);
	m_Menu.Check(CMD_MEASUREMENT, m_InfoMode == INFO_MEASUREMENT);
}

void CTrackCurveDlg::UpdateGraph(gboolean autozoom_x, gboolean autozoom_y)
{
	cmpack_graph_view_set_model(CMPACK_GRAPH_VIEW(m_Graph), NULL);
	cmpack_graph_data_clear(m_GraphData);
	m_Table.Render(m_GraphData, 0, m_ChannelY, false);
	m_Table.SetView(CMPACK_GRAPH_VIEW(m_Graph), 0, m_ChannelY, false);
	cmpack_graph_view_set_grid(CMPACK_GRAPH_VIEW(m_Graph), m_ShowGrid, m_ShowGrid);
	cmpack_graph_view_set_model(CMPACK_GRAPH_VIEW(m_Graph), m_GraphData);
	cmpack_graph_view_reset_zoom(CMPACK_GRAPH_VIEW(m_Graph), autozoom_x, autozoom_y);
}

void CTrackCurveDlg::UpdateChannels(void)
{
	int i;
	GtkTreeIter iter;
	CChannels *cy = m_Table.ChannelsY();

	// Update list of dependent channels
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_YCombo), NULL);
	gtk_list_store_clear(m_YChannels);
	for (i=0; i<cy->Count(); i++) {
		gtk_list_store_append(m_YChannels, &iter);
		gtk_list_store_set(m_YChannels, &iter, 0, i, 1, cy->GetName(i), -1);
	}
	gtk_combo_box_set_model(GTK_COMBO_BOX(m_YCombo), GTK_TREE_MODEL(m_YChannels));
	if (m_ChannelY<0 || m_ChannelY>=cy->Count()) 
		m_ChannelY = (cy->Count()>0 ? 0 : -1);
	if (m_ChannelY>=0) {
		m_MeasBox.SetChannel(CMPACK_AXIS_Y, m_ChannelY);
		gtk_combo_box_set_active(GTK_COMBO_BOX(m_YCombo), m_ChannelY);
	}
	gtk_widget_set_sensitive(m_YCombo, cy->Count()>1);
	gtk_widget_set_sensitive(m_YLabel, cy->Count()>1);
}

void CTrackCurveDlg::MenuCallback(CCBObject *sender, int message, int wparam, void *lparam, void *cb_data)
{
	CTrackCurveDlg *pMe = (CTrackCurveDlg*)cb_data;

	switch (message)
	{
	case CMenuBar::CB_ACTIVATE:
		pMe->OnCommand(wparam);
		break;
	}
}

void CTrackCurveDlg::OnCommand(int cmd_id)
{
	switch (cmd_id)
	{
	// File menu
	case CMD_CLOSE:
		Close();
		break;
	case CMD_SAVE:
		SaveData();
		break;
	case CMD_EXPORT:
		ExportAsImage();
		break;
	case CMD_REBUILD:
		RebuildData(GTK_WINDOW(m_pDlg));
		break;

	// View menu
	case CMD_GRID:
		m_ShowGrid = m_Menu.IsChecked(CMD_GRID);
		UpdateGraph(FALSE, FALSE);
		CConfig::SetBool("TrackCurve", "Grid", m_ShowGrid);
		break;

	// Tools menu
	case CMD_STATISTICS:
		if (m_Menu.IsChecked(CMD_STATISTICS)) 
			SetInfoMode(INFO_STATISTICS);
		else if (m_InfoMode == INFO_STATISTICS)
			SetInfoMode(INFO_NONE);
		break;
	case CMD_MEASUREMENT:
		if (m_Menu.IsChecked(CMD_MEASUREMENT)) 
			SetInfoMode(INFO_MEASUREMENT);
		else if (m_InfoMode == INFO_MEASUREMENT)
			SetInfoMode(INFO_NONE);
		break;

	// Help menu
	case CMD_HELP:
		g_MainWnd->ShowHelp(GTK_WINDOW(m_pDlg), IDH_TRACK_CURVE);
		break;
	}
}

void CTrackCurveDlg::changed(GtkWidget *pBtn, CTrackCurveDlg *pMe)
{
	pMe->OnChanged(pBtn);
}

void CTrackCurveDlg::OnChanged(GtkWidget *pBtn)
{
	int ch;
	GtkTreeIter iter;

	if (pBtn == m_YCombo) {
		if (gtk_combo_box_get_active_iter(GTK_COMBO_BOX(m_YCombo), &iter)) {
			gtk_tree_model_get(GTK_TREE_MODEL(m_YChannels), &iter, 0, &ch, -1);
			if (ch!=m_ChannelY) {
				m_ChannelY = ch;
				UpdateGraph(FALSE, TRUE);
				UpdateTools();
			}
		}
	}
}

void CTrackCurveDlg::button_clicked(GtkWidget *button, CTrackCurveDlg *pDlg)
{
	pDlg->OnButtonClicked(button);
}

void CTrackCurveDlg::OnButtonClicked(GtkWidget *pBtn)
{
	double zoom;

	if (pBtn==GTK_WIDGET(m_ZoomIn)) {
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X, zoom + 5.0);
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y, zoom + 5.0);
	} else 
	if (pBtn==GTK_WIDGET(m_ZoomOut)) {
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X, zoom - 5.0);
		zoom = cmpack_graph_view_get_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y);
		cmpack_graph_view_set_zoom(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y, zoom - 5.0);
	} else 
	if (pBtn==GTK_WIDGET(m_ZoomFit)) {
		cmpack_graph_view_reset_zoom(CMPACK_GRAPH_VIEW(m_Graph), TRUE, TRUE);
	}
}

void CTrackCurveDlg::mouse_moved(GtkWidget *button, CTrackCurveDlg *pDlg)
{
	pDlg->m_UpdatePos = true;
}

//
// Mouse button handler
//
gint CTrackCurveDlg::button_press_event(GtkWidget *widget, GdkEventButton *event, CTrackCurveDlg *pMe)
{
	int focused;

	if (event->type==GDK_BUTTON_PRESS && event->button==3) {
		gtk_widget_grab_focus(widget);
		if (widget==pMe->m_Graph) {
			focused = cmpack_graph_view_get_focused(CMPACK_GRAPH_VIEW(widget));
			if (focused>=0 && !cmpack_graph_view_is_selected(CMPACK_GRAPH_VIEW(widget), focused)) {
				cmpack_graph_view_unselect_all(CMPACK_GRAPH_VIEW(widget));
				cmpack_graph_view_select(CMPACK_GRAPH_VIEW(widget), focused);
			}
		}
		pMe->OnContextMenu(widget, event);
		return TRUE;
	}
	return FALSE;
}

//
// Show context menu
//
void CTrackCurveDlg::OnContextMenu(GtkWidget *widget, GdkEventButton *event)
{
	int res, selected;

	if (widget==m_Graph) {
		selected = cmpack_graph_view_get_selected_count(CMPACK_GRAPH_VIEW(m_Graph));
		m_GraphMenu.Enable(CMD_DELETE, selected>0);
		m_GraphMenu.Enable(CMD_HIDE, selected>0);
		m_GraphMenu.Enable(CMD_FRAMEINFO, selected==1);
		m_GraphMenu.Enable(CMD_PREVIEW, selected==1);
		res = m_GraphMenu.Execute(event);
		switch (res) 
		{
		case CMD_PREVIEW:
			// Show frame preview
			ShowFramePreview();
			break;
		case CMD_FRAMEINFO:
			// Show frame properties
			ShowFrameInfo();
			break;
		case CMD_HIDE:
			// Remove selected points from data set
			RemoveFromDataSet();
			break;
		case CMD_DELETE:
			// Delete selected points from project
			DeleteFromProject();
			break;
		}
	}
}

gboolean CTrackCurveDlg::timer_cb(CTrackCurveDlg *pDlg)
{
	if (pDlg->m_UpdatePos) {
		pDlg->m_UpdatePos = false;
		pDlg->UpdateStatus();
	}
	return TRUE;
}

void CTrackCurveDlg::selection_changed(GtkWidget *pChart, CTrackCurveDlg *pDlg)
{
	pDlg->OnSelectionChanged();
}

//
// Left button click
//
void CTrackCurveDlg::OnSelectionChanged(void)
{
	UpdateTools();
}

void CTrackCurveDlg::UpdateStatus(void)
{
	char	msg[256], buf1[256], buf2[256];
	gdouble	dx, dy;

	int item = cmpack_graph_view_get_focused(CMPACK_GRAPH_VIEW(m_Graph));
	if (item>=0) {
		if (m_LastFocus!=item) {
			m_LastFocus = item;
			int frame_id = (int)cmpack_graph_data_get_param(m_GraphData, item),
				xcol = m_Table.ChannelsX()->GetColumn(0),
				ycol = m_Table.ChannelsY()->GetColumn(m_ChannelY);
			if (m_Table.Find(frame_id) && m_Table.GetDbl(xcol, &dx) && m_Table.GetDbl(ycol, &dy)) {
				PrintKeyValue(buf1, dx, m_Table.ChannelsX()->Get(0));
				PrintKeyValue(buf2, dy, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "Frame #%d: %s, %s", frame_id, buf1, buf2);
			} else {
				sprintf(msg, "Frame #%d", frame_id);
			}
			SetStatus(msg);
		}
	} else {
		m_LastFocus = -1;
		if (m_ChannelY>=0 && cmpack_graph_view_mouse_pos(CMPACK_GRAPH_VIEW(m_Graph), &dx, &dy)) {
			if (!m_LastPosValid || dx!=m_LastPosX || dy!=m_LastPosY) {
				m_LastPosValid = true;
				m_LastPosX = dx;
				m_LastPosY = dy;
				PrintKeyValue(buf1, dx, m_Table.ChannelsX()->Get(0));
				PrintKeyValue(buf2, dy, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "Cursor: %s, %s", buf1, buf2);
				SetStatus(msg);
			}
		} else {
			if (m_LastPosValid) {
				m_LastPosValid = false;
				SetStatus(NULL);
			}
		}
	}
}

void CTrackCurveDlg::PrintValue(char *buf, double value, const CChannel *channel)
{
	const gchar *unit = channel->Unit();
	int prec = channel->Precision();

	if (unit)
		sprintf(buf, "%.*f %s", prec, value, unit);
	else
		sprintf(buf, "%.*f", prec, value);
}

void CTrackCurveDlg::PrintKeyValue(char *buf, double value, const CChannel *channel)
{
	const gchar *name = channel->Name(), *unit = channel->Unit();
	int prec = channel->Precision();

	if (unit)
		sprintf(buf, "%s = %.*f %s", name, prec, value, unit);
	else
		sprintf(buf, "%s = %.*f", name, prec, value);
}

void CTrackCurveDlg::UpdateTools(void)
{
	char	buf[256], msg[256];
	double	dx, dy, dmin, dmax;
	int		xcol, ycol;

	if (m_InfoMode == INFO_MEASUREMENT) {
		// Update measurement tool
		m_MeasBox.SetChannel(CMPACK_AXIS_Y, m_ChannelY);
	} else 
	if (m_InfoMode == INFO_STATISTICS) {
		m_InfoBox.BeginUpdate();
		m_InfoBox.Clear();
		// Show information about selected star
		xcol = m_Table.ChannelsX()->GetColumn(0);
		ycol = m_Table.ChannelsY()->GetColumn(m_ChannelY);
		int selected_count = cmpack_graph_view_get_selected_count(CMPACK_GRAPH_VIEW(m_Graph));
		if (selected_count==1) {
			int row = cmpack_graph_view_get_selected(CMPACK_GRAPH_VIEW(m_Graph));
			int frame_id = (int)cmpack_graph_data_get_param(m_GraphData, row);
			if (frame_id>=0) {
				sprintf(buf, "Frame #%d", frame_id);
				m_InfoBox.AddTitle(1, buf);
				if (m_Table.Find(frame_id) && m_Table.GetDbl(xcol, &dx) && m_Table.GetDbl(ycol, &dy)) {
					PrintKeyValue(buf, dx, m_Table.ChannelsX()->Get(0));
					m_InfoBox.AddText(buf);
					PrintKeyValue(buf, dy, m_Table.ChannelsY()->Get(m_ChannelY));
					m_InfoBox.AddText(buf);
				}
			}
		} else {
			// Compute statistics
			int count = 0, length = (selected_count>0 ? selected_count : m_Table.Rows());
			double *x = new double[length], *y = new double[length];
			if (selected_count>0) {
				GList *list = cmpack_graph_view_get_selected_rows(CMPACK_GRAPH_VIEW(m_Graph));
				for (GList *ptr=list; ptr!=NULL; ptr=ptr->next) {
					int frame_id = (int)cmpack_graph_data_get_param(m_GraphData, (gint)(intptr_t)ptr->data);
					if (m_Table.Find(frame_id) && m_Table.GetDbl(xcol, x+count) && m_Table.GetDbl(ycol, y+count))
						count++;
				}
				g_list_free(list);
			} else {
				bool ok = m_Table.Rewind();
				while (ok) {
					if (m_Table.GetDbl(xcol, x+count) && m_Table.GetDbl(ycol, y+count))
						count++;
					ok = m_Table.Next();
				}
			}
			if (selected_count>0)
				sprintf(buf, "There are %d selected points.", count);
			else
				sprintf(buf, "There are %d points in total.", count);
			m_InfoBox.AddText(buf);
			if (ComputeMinMax(count, x, &dmin, &dmax)) {
				m_InfoBox.AddText("");
				sprintf(msg, "Data %s:", m_Table.ChannelsX()->GetName(0));
				m_InfoBox.AddTitle(1, msg);
				PrintValue(buf, dmin, m_Table.ChannelsX()->Get(0));
				sprintf(msg, "%s: %s", "Min.", buf);
				m_InfoBox.AddText(msg);
				PrintValue(buf, dmax, m_Table.ChannelsX()->Get(0));
				sprintf(msg, "%s: %s", "Max.", buf);
				m_InfoBox.AddText(msg);
			}
			if (ComputeMinMax(count, y, &dmin, &dmax)) {
				m_InfoBox.AddText("");
				sprintf(buf, "Data %s:", m_Table.ChannelsY()->GetName(m_ChannelY));
				m_InfoBox.AddTitle(1, buf);
				PrintValue(buf, dmin, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "%s: %s", "Min.", buf);
				m_InfoBox.AddText(msg);
				PrintValue(buf, dmax, m_Table.ChannelsY()->Get(m_ChannelY));
				sprintf(msg, "%s: %s", "Max.", buf);
				m_InfoBox.AddText(msg);
			}
			delete[] x;
			delete[] y;
		}
		m_InfoBox.EndUpdate();
	} else {
		// No tool is open
		cmpack_graph_view_set_cursors(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_X, 0);
		cmpack_graph_view_set_cursors(CMPACK_GRAPH_VIEW(m_Graph), CMPACK_AXIS_Y, 0);
	}
}

void CTrackCurveDlg::SetInfoMode(tInfoMode mode)
{
	m_InfoMode = mode;
	switch (m_InfoMode) 
	{
	case INFO_STATISTICS:
		m_MeasBox.Show(false);
		m_InfoBox.Show(true);
		break;
	case INFO_MEASUREMENT:
		m_InfoBox.Show(false);
		m_MeasBox.Show(true);
		break;
	default:
		m_MeasBox.Show(false);
		m_InfoBox.Show(false);
		break;
	}
	UpdateTools();
	UpdateControls();
}

void CTrackCurveDlg::InfoBoxCallback(CCBObject *sender, int message, int wparam, void* lparam, void* cb_data)
{
	CTrackCurveDlg *pMe = (CTrackCurveDlg*)cb_data;

	switch (message)
	{
	case CInfoBox::CB_CLOSED:
		pMe->OnInfoBoxClosed();
		break;
	}
}

void CTrackCurveDlg::OnInfoBoxClosed(void)
{
	m_InfoMode = INFO_NONE;
	UpdateControls();
}

void CTrackCurveDlg::SaveData(void)
{
	int res;
	char *filename, *folder, *fpath;
	
	GtkWidget *pSaveDlg = gtk_file_chooser_dialog_new("Save track curve",
		GTK_WINDOW(m_pDlg), GTK_FILE_CHOOSER_ACTION_SAVE, GTK_STOCK_CANCEL, 
		GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
	gtk_file_chooser_standard_tooltips(GTK_FILE_CHOOSER(pSaveDlg));
	gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(pSaveDlg), true);

	folder = g_Project->GetStr("Output", "Folder", NULL);
	if (!folder)
		folder = CConfig::GetStr("Output", "Folder", NULL);
	if (!folder)
		folder = g_path_get_dirname(g_Project->Path());
	if (folder && g_file_test(folder, G_FILE_TEST_IS_DIR)) 
		gtk_file_chooser_set_current_folder(GTK_FILE_CHOOSER(pSaveDlg), folder);
	g_free(folder);
	filename = g_Project->GetStr("TrackCurve", "File", "trackcurve.txt");
	if (filename) 
		gtk_file_chooser_set_current_name(GTK_FILE_CHOOSER(pSaveDlg), filename);
	g_free(filename);

	if (gtk_dialog_run(GTK_DIALOG(pSaveDlg))==GTK_RESPONSE_ACCEPT) {
		folder = gtk_file_chooser_get_current_folder(GTK_FILE_CHOOSER(pSaveDlg));
		g_Project->SetStr("Output", "Folder", folder);
		CConfig::GetStr("Output", "Folder", folder);
		g_free(folder);
		fpath = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(pSaveDlg));
		g_Project->SetStr("TrackCurve", "File", g_basename(fpath));
		m_Table.Export(fpath, true, false, false, &res);
		g_free(fpath);
		g_Project->Save();
	}
	gtk_widget_destroy(pSaveDlg);

	m_DataSaved = true;
}

void CTrackCurveDlg::ExportAsImage(void)
{
	CGraphExportDlg pDlg(GTK_WINDOW(m_pDlg));
	pDlg.Execute(CMPACK_GRAPH_VIEW(m_Graph));
}

GList *CTrackCurveDlg::GetSelectedFrames(void)
{
	GList *frames = NULL;

	GList *rows = cmpack_graph_view_get_selected_rows(CMPACK_GRAPH_VIEW(m_Graph));
	if (rows) {
		for (GList *ptr=rows; ptr!=NULL; ptr=ptr->next) {
			int frame = cmpack_graph_data_get_param(m_GraphData, (intptr_t)ptr->data);
			GtkTreePath *path = g_Project->GetFilePath(frame);
			if (path)
				frames = g_list_prepend(frames, path);
		}
		g_list_free(rows);
	}
	return g_list_reverse(frames);
}

GtkTreePath *CTrackCurveDlg::GetSelectedFrame(void)
{
	int row = cmpack_graph_view_get_selected(CMPACK_GRAPH_VIEW(m_Graph));
	if (row>=0) {
		int frame = (gint)cmpack_graph_data_get_param(m_GraphData, row);
		return g_Project->GetFilePath(frame);
	}
	return NULL;
}
