/**************************************************************

utils.h (C-Munipack project)
Helper functions
Copyright (C) 2008 David Motl, dmotl@volny.cz

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

**************************************************************/

#ifndef MUNIWIN_UTILS_H
#define MUNIWIN_UTILS_H

#include <gtk/gtk.h>

/* PI number */
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

// Constants
#define MAX_APERTURES		12
#define MAX_SELECTION		64
#define MAX_PROJECT_LEN		FILENAME_MAX
#define MAX_OBJECT_LEN		260
#define MAX_OBSERVER_LEN	260
#define MAX_LOCATION_LEN	260

// Return nearest integer number with limit checking
int RoundToInt(double x);

// Bound value to limits
double LimitValue(double value, double min, double max);
int LimitValue(int value, int min, int max);

// Compute minimum and maximum
bool ComputeMinMax(int count, const double *data, double *min, double *max);

// Get path to the directory with user configuration files
const char *get_user_config_dir(void);

// Get path to the directory with user data files
const char *get_user_data_dir(void);

// Get path to the directory with shared files
const char *get_share_dir(void);

// Get path to the icon file
char *get_icon_file(const gchar *icon);

// Get a button from dialog's action area by a response id
GtkWidget *get_dialog_widget_by_response(GtkDialog *pDlg, gint response_id);

// Set standard tooltips for standard dialog buttons
void gtk_dialog_widget_standard_tooltips(GtkDialog *pDlg);

// Set standard tooltips for file chooser dialog
void gtk_file_chooser_standard_tooltips(GtkFileChooser *pDlg);

// Set tooltip text to a button from dialog's action area 
void gtk_dialog_set_tooltip_by_response(GtkDialog *pDlg, gint response_id, const gchar *tooltip);

// Make directory tree
bool force_directory(const gchar *path);

// Open a file (file name is UTF-8 encoded)
FILE *open_file(const gchar *fpath, const gchar *mode);

// Copy a file (file name is UTF-8 encoded)
bool copy_file(const gchar *source, const gchar *target, bool fail_if_exists, GError **error);

// Copy all files in directory (file name is UTF-8 encoded)
bool copy_all_files(const gchar *source, const gchar *target, bool recursive, GError **error);

// Show user's manual
void ShowHelp(GtkWindow *pParent, int context_id);

// Show PDF document
void ShowDocument(GtkWindow *pParent, const gchar *file);

// Show message box with error message
void ShowError(GtkWindow *pParent, const char *message, bool play_sound = false);

// Show message box with warning message
void ShowWarning(GtkWindow *pParent, const char *message, bool play_sound = false);

// Show message box with information
void ShowInformation(GtkWindow *pParent, const char *message, bool play_sound = false);

// Show configuration message box
bool ShowConfirmation(GtkWindow *pParent, const char *message);

// Test file or directory and ask for confirmation before replacing it
// If the file/directory does not exist, returns true immediately.
// Otherwise, shows a message and returns true if the user confirmed it.
bool ConfirmOverwrite(GtkWindow *pParent, const gchar *path);
bool ConfirmOverwriteDir(GtkWindow *pParent, const gchar *path);

// Set/change file extension
char *SetFileExtension(const char *path, const char *ext);

// Remove file extension
gchar *StripFileExtension(const gchar *filename);

// Check file extension
bool CheckFileExtension(const char *path, const char *ext);

// Check given string if it is valid file base name 
bool CheckFileBaseName(const char *filename);

// Case sensitive comparison or two UTF-8 strings, they can be NULL
int StrCmp0(const gchar *str1, const gchar *str2);

// Case insensitive comparison or two UTF-8 strings, they can be NULL
int StrCaseCmp0(const gchar *str1, const gchar *str2);

// Case sensitive substring search for two UTF-8 strings, they can be NULL
// If either str1 or str2 is NULL, the result is always NULL
const gchar *StrStr0(const gchar *str1, const gchar *str2);

// Case insensitive substring search for two UTF-8 strings, they can be NULL
// If either str1 or str2 is NULL, the result is always NULL
const gchar *StrCaseStr0(const gchar *str1, const gchar *str2);

// Case sensitive substring search in memory buffer
const gchar *MemStr(const gchar *buffer, const gchar *substr, gsize buflength);

// Compare two absolute UTF-8 paths, they can be NULL
int ComparePaths(const gchar *fpath1, const gchar *fpath2);

//
// File lock
//
class FileLock
{
public:
	// Constructor
	FileLock();

	// Destructor
	virtual ~FileLock();

	// Acquire lock
	// params:
	//	fpath - file to be locked, this is NOT the name of the special file
	//			that keeps the status of the lock
	bool Acquire(const gchar *fpath);

	// Release lock
	void Release(void);

private:
#ifdef _WIN32
	void *m_File;
#else
	int	m_File;
#endif
};

#endif
